/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-04-11
**
***************************************************************************/

#include <QHash>

#include "qtbabstractnumericalcache.h"
#include "qtbabstractnumericalkey.h"
#include "qtbtrace.h"
#include "qtbmemorychecker.h"

// QHash does not work with a pointer nor with a virtual class as key
class QTBTOOLS_EXPORT QtbHashNumericalKey
{
public:
  QtbHashNumericalKey() { _key = 0; }
  QtbHashNumericalKey( QtbAbstractNumericalKey * key ) { _key = key; }

  void operator=( const QtbHashNumericalKey& o ) { _key = o._key; }
  inline bool operator==( const QtbHashNumericalKey& o ) const;
  int hash() const { return _key ? _key->hash() : 0; }
private:
  QtbAbstractNumericalKey * _key;
};

inline bool QtbHashNumericalKey::operator==( const QtbHashNumericalKey& o ) const
{
  if ( _key && o._key ) {
    if ( _key->id() == o._key->id() ) {
      return *_key == *o._key;
    } else {
      return false;
    }
  } else {
    return _key==o._key;
  }
}

int qHash( const QtbHashNumericalKey& key )
{
  return key.hash();
}

/*!
  \class QtbAbstractNumericalCache qtbabstractnumericalcache.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

QMutex QtbAbstractNumericalCache::_mutex;
QHash<QtbHashNumericalKey, QtbAbstractNumericalCache *> QtbAbstractNumericalCache::_caches;

/*!
  Takes the ownership over \a key. If a new cache is created, the cache tackes its ownership.
*/
const QtbAbstractNumericalCache * QtbAbstractNumericalCache::begin( QtbAbstractNumericalKey * key )
{
  QtbAbstractNumericalCache * cache;
  QtbHashNumericalKey k(key);
  _mutex.lock();
  QHash<QtbHashNumericalKey, QtbAbstractNumericalCache *>::iterator it = _caches.find( k );
  if (it!=_caches.end()) {
    cache = it.value();
    cache->setLocked( true );
    _mutex.unlock();
    delete key; // A similar key already exist, drop this one
  } else {
    cache = add( key );
    cache->setLocked( true );
    _mutex.unlock();
    cache->init();
  }
  return cache;
}

void QtbAbstractNumericalCache::end( const QtbAbstractNumericalCache * cache )
{
  cache->setLocked( false );
}

void QtbAbstractNumericalCache::clear()
{
  QMutexLocker ml( &_mutex );
  qDeleteAll(_caches);
  _caches.clear();
}

QtbAbstractNumericalCache * QtbAbstractNumericalCache::add( QtbAbstractNumericalKey * key )
{
  QtbAbstractNumericalCache * cache = key->createCache();
  _caches.insert( QtbHashNumericalKey(key), cache );
  if (_caches.count()>100) {
    QHash<QtbHashNumericalKey, QtbAbstractNumericalCache *>::iterator it = _caches.begin();
    QHash<QtbHashNumericalKey, QtbAbstractNumericalCache *>::iterator itOld = it;
    clock_t t = it.value()->lastUse();
    for (++it;it!=_caches.end();it++) {
      if (it.value()->lastUse()<t) {
        itOld = it;
        t = it.value()->lastUse();
      }
    }
    if (!it.value()->locked()) {
      delete itOld.value();
      _caches.remove(itOld.key());
    }
  }
  return cache;
}
