/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-04-11
**
***************************************************************************/

#ifndef QTB_ABSTRACTNUMERICALCACHE_H
#define QTB_ABSTRACTNUMERICALCACHE_H

#include <QMutex>

#include "qtbtoolsdllexport.h"
#include "qtbglobal.h"
#include "qtbcacheitem.h"
#include "qtbabstractnumericalkey.h"

class QtbHashNumericalKey;

class QTBTOOLS_EXPORT QtbAbstractNumericalCache
{
public:
  inline QtbAbstractNumericalCache( QtbAbstractNumericalKey * key );
  virtual ~QtbAbstractNumericalCache() { delete _key; }

  virtual void init() = 0;
  const QtbAbstractNumericalKey& key() const { return *_key; }

  static const QtbAbstractNumericalCache * begin( QtbAbstractNumericalKey * key );
  static void end( const QtbAbstractNumericalCache * cache );
  static void clear();
private:
  static QtbAbstractNumericalCache * add( QtbAbstractNumericalKey * key );

  clock_t lastUse() const { return _lastUse; }
  void use() const { _lastUse = clock(); }

  inline void setLocked( bool l ) const;
  bool locked() const { return !_locked.testAndSetOrdered(0, 0); }

  QtbAbstractNumericalKey * _key;
  mutable clock_t _lastUse;
  mutable QAtomicInt _locked;

  static QMutex _mutex;
  static QHash<QtbHashNumericalKey, QtbAbstractNumericalCache *> _caches;
  static int _currentByteCount;
  static int _maximumByteCount;
};

inline QtbAbstractNumericalCache::QtbAbstractNumericalCache( QtbAbstractNumericalKey * key )
{
  _key = key;
  _lastUse = clock();
  _locked = 0;
}

inline void QtbAbstractNumericalCache::setLocked( bool l ) const
{
  if ( l ) {
    _locked.fetchAndAddOrdered ( 1 );
  } else {
    QTB_ASSERT( _locked.fetchAndAddOrdered ( -1 )>=0 );
  }
}

#endif // QTB_ABSTRACTNUMERICALCACHE_H
