/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-02-20
**
***************************************************************************/

#include "qtbabstractprogress.h"
#include "qtbglobal.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbAbstractProgress qtbabstractprogress.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
QtbAbstractProgress::QtbAbstractProgress()
{
  QTB_TRACE;
  _currentValue = 0;
  _maxValue = -1;
  _lastTime = time(0);
}

const QString& QtbAbstractProgress::caption() const
{
  QMutexLocker ml(&_mutex);
  return _caption;
}

void QtbAbstractProgress::setCaption( QString c )
{
  QTB_TRACE;
  QMutexLocker ml(&_mutex);
  _caption = c;
}

int QtbAbstractProgress::maximumValue() const
{
  QMutexLocker ml(&_mutex);
  return _maxValue;
}

void QtbAbstractProgress::setMaximumValue( int val )
{
  QTB_TRACE;
  QMutexLocker ml(&_mutex);
  _deltaValue = 0;
  _currentValue = 0;
  _maxValue = val;
  _lastTime.fetchAndStoreOrdered( time(0) );
}

inline void QtbAbstractProgress::show()
{
  QTB_TRACE;
  paint( _caption, _currentValue, _maxValue );
}

void QtbAbstractProgress::increaseValueInternal()
{
  QTB_TRACE;
  QMutexLocker ml(&_mutex);
  _currentValue += _deltaValue;
  show();
}

void QtbAbstractProgress::setValueInternal( int val )
{
  QTB_TRACE;
  QMutexLocker ml(&_mutex);
  _currentValue = val;
  show();
}

QString QtbAbstractProgress::valueString( int value, int maxValue )
{
  if (value>maxValue) {
    return QString::number( value );
  } else {
    static const QString fmt("%1 %");
    return fmt.arg( ( 100 * value ) / maxValue );
  }
}

