/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-02-20
**
***************************************************************************/

#ifndef QTB_ABSTRACTPROGRESS_H
#define QTB_ABSTRACTPROGRESS_H

#include <time.h>

#include <QString>
#include <QMutex>

#include "qtbtoolsdllexport.h"
#include "qtbcompatqt43.h"
#include "qtbtrace.h"

class QTBTOOLS_EXPORT QtbAbstractProgress
{
public:
  QtbAbstractProgress();
  virtual ~QtbAbstractProgress() {}

  virtual void setMaximumValue( int val );
  int maximumValue() const;

  virtual void setCaption( QString c );
  const QString& caption() const;

  inline void setValue( int val );
  inline void increaseValue( int val = 1 );
protected:
  virtual void paint( QString caption, int value, int maxValue ) = 0;
  QString valueString( int value, int maxValue );

  mutable QMutex _mutex;
  QString _caption;
  int _currentValue;
  int _maxValue;
private:
  inline void show();
  void setValueInternal( int val );
  void increaseValueInternal();

  QAtomicInt _lastTime;
  int _deltaValue;
};

inline void QtbAbstractProgress::increaseValue( int val )
{
  QTB_TRACE;
  _deltaValue += val;
  int now = time(0);
  if ( _lastTime.fetchAndStoreOrdered(now) != now ) {
    increaseValueInternal();
    _deltaValue = 0;
  }
}

inline void QtbAbstractProgress::setValue( int val )
{
  QTB_TRACE;
  int now = time(0);
  if ( _lastTime.fetchAndStoreOrdered(now) != now ) {
    setValueInternal(val);
  }
}

#endif // QTB_ABSTRACTPROGRESS_H
