/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-02-09
**
***************************************************************************/

#include <QCoreApplication>

#include "qtbapplicationhelp.h"
#include "qtbcoreapplication.h"

/*!
  \class QtbApplicationHelp qtbapplicationhelp.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
QtbApplicationHelp::QtbApplicationHelp()
{
  QTB_TRACE;
}

/*!
  Description of destructor still missing
*/
QtbApplicationHelp::~QtbApplicationHelp()
{
  QTB_TRACE;
}

void QtbApplicationHelp::addGroup(QString title, QByteArray section)
{
  QTB_TRACE;
  OptionGroup g;
  g.title = title;
  g.section = section;
  _options.append(g);
}

void QtbApplicationHelp::addOption(QString option, QString comments)
{
  QTB_TRACE;
  OptionGroup& g = _options.last();
  Option o;
  o.option = option;
  o.comments = comments;
  g.options.append( o );
}

void QtbApplicationHelp::addExample(QString command, QString comments)
{
  QTB_TRACE;
  Example o;
  o.command = command;
  o.comments = comments;
  _examples.append( o );
}

void QtbApplicationHelp::print(const OptionGroup& g)
{
  QTB_TRACE;
  if (g.options.isEmpty()) {
    print("\n"+g.title, "", 0);
  } else {
    print("\n"+g.title+" options:\n", "", 0);
    QString indentStr;
    indentStr.fill(' ',28);
    for( QList<Option>::const_iterator it = g.options.begin(); it!=g.options.end(); it++ ) {
      const Option& o = *it;
      QString cmt = o.comments;
      QString cmtLine = getLine(cmt, qtbApp.terminalCols()-28);
      if (o.option.length()<25) {
        qtbApp << "  " << o.option.leftJustified(25) << " " << cmtLine << "\n";
      } else {
        qtbApp << "  " << o.option << "\n";
        qtbApp << indentStr << cmtLine << "\n";
      }
      print( cmt, QString::null, 28 );
    }
  }
}

/*!
  Returns the list of sections
*/
QList<QByteArray> QtbApplicationHelp::sections()
{
  QList<QByteArray> l;
  for( QList<OptionGroup>::iterator it = _options.begin(); it!=_options.end(); it++ ) {
    l << it->section;
  }
  if ( !_examples.isEmpty() ) {
    l << "examples";
  }
  return l;
}

void QtbApplicationHelp::exec(const char * group)
{
  QTB_TRACE;
  if (group) {
    if (strcmp(group,"wiki")==0) {
      execWiki();
      return;
    }
    if (strcmp(group,"latex")==0) {
      execLatex();
      return;
    }
  }
  // make sure stdout is used for output
  qtbApp.setStream( new QtbLogStream(stdout) );

  qtbApp << tr("Usage: %1 %2\n").arg(QCoreApplication::applicationName()).arg(_optionSummary);
  print(_comments,"", 0);
  if (group) {
    QByteArray helpOption(group);
    if ( helpOption == "all" ) {
      for( QList<OptionGroup>::iterator it = _options.begin(); it!=_options.end(); it++ ) {
        print( *it );
      }
    } else if ( helpOption.startsWith("no-") ) {
      helpOption = helpOption.mid(3);
      for( QList<OptionGroup>::iterator it = _options.begin(); it!=_options.end(); it++ ) {
        if ( it->section != helpOption) {
          print( *it );
        }
      }
    } else {
      for( QList<OptionGroup>::iterator it = _options.begin(); it!=_options.end(); it++ ) {
        if ( it->section == helpOption) {
          print( *it );
        }
      }
    }
  } else if (!_options.isEmpty()) {
    print( _options.first() );
    if ( _options.count()>1 ) print( _options.last() );
  }
  if ( !_examples.isEmpty() ) {
    qtbApp << "\nExamples:\n";
    for ( QList<Example>::iterator it = _examples.begin(); it != _examples.end(); it++ ) {
      Example& o = *it;
      qtbApp << "\n";
      print(o.command, QString::null, 9);
      qtbApp << "\n";
      print(o.comments, QString::null, 2);
    }
  }
  if ( !_seeAlso.isEmpty() ) {
    qtbApp << "\nSee also:\n";
    print(_seeAlso, QString::null, 2);
  }
  qtbApp << "\nCopyright:\n";
  print( qtbApp.copyright(), QString::null, 2 );
  qtbApp << flush;
}

void QtbApplicationHelp::execWiki()
{
  QTB_TRACE;
  // TODO Generate a format suitable for Wiki)
#if 0
  qtbApp << "<a name=\"name\">&nbsp;</a>"
  qtbApp << htmlFilter(QCoreApplication::applicationName().toUpper());
  qtbApp << "\n"
            "\\end{center}\n"
            "\\noindent\\textbf{NAME}\\\\\n"
            "\\hspace*{2cm}\\begin{minipage}{14cm}\n";
  qtbApp << htmlFilter(QCoreApplication::applicationName());
  qtbApp << " - manual page for version ";
  qtbApp << htmlFilter(QtbCoreApplication::version(QCoreApplication::applicationName()));
  
<H1>GPELL</H1>
<HR>

<A NAME="lbAB">&nbsp;</A>
<H2>NAME</H2>

gpell - Computation of ellipticity curves
<A NAME="lbAC">&nbsp;</A>
<H2>SYNOPSIS</H2>

<B>gpell</B>

[<I>OPTIONS</I>] [<I>FILE</I>]
<A NAME="lbAD">&nbsp;</A>
<H2>DESCRIPTION</H2>

Compute ellipticity curve for layered models given through stdin or FILE.
<A NAME="lbAE">&nbsp;</A>
<H3>Format for layered models:</H3>

<DL COMPACT>
<DT>Line 1 :<DD>
&lt;number of layers including half-space for first model&gt;
<DT>Line 2 :<DD>
&lt;thickness (m)&gt; &lt;Vp (m/s)&gt; &lt;Vs (m/s)&gt; &lt;Density (kg/m3)&gt;[ &lt;Qs&gt; &lt;Qp&gt;]


<DT>....<DD>
<DT>Line n :<DD>
0 &lt;Vp (m/s)&gt; &lt;Vs (m/s)&gt; &lt;Density (kg/m3)&gt;[ &lt;Qs&gt; &lt;Qp&gt;]
<DT>Line n+1 :<DD>

&lt;number of layers including half-space for second model&gt;
<DT>....<DD>
</DL>
<P>

Quality factors are not mandatory. If not specified, pure elastic computation is
performed. Any number of models can be given as input.
<A NAME="lbAF">&nbsp;</A>
<H3>Generic options:</H3>

<DL COMPACT>
<DT><B>-help</B><DD>
Show help about options

<DT><B>-version</B><DD>
Show version information
<DT><B>-reportbug</B><DD>
Start bug report dialog, information about bug is
passed through stdin. This option is used internally
to report bugs if option <B>-nobugreport</B> is not
specified.
<DT><B>-nobugreport</B><DD>
Do not generate bug report in case of error
</DL>
<A NAME="lbAG">&nbsp;</A>
<H3>Gpell options:</H3>

<DL COMPACT>
<DT><B>-n</B> &lt;count&gt;<DD>
Number of samples (default=100)
<DT><B>-R</B> &lt;n modes&gt;<DD>
Number of Rayleigh modes (default=1)
<DT><B>-s</B> &lt;sampling&gt;<DD>
Defines the sampling type:
<ul>

  <li>period: regular sampling in period</li>
  <li>frequency:  regular sampling in frequency</li>
  <li>log: regular sampling in log(frequency)  --&gt; default</li>
</ul>
<DT><DD>
<DT><B>-min</B> &lt;min&gt;<DD>

Minimum of range for ellipticity curve (default=0.2)
<DT><B>-max</B> &lt;max&gt;<DD>
Maximum of range for ellipticity curve (default=20)
<DT><B>-n</B> &lt;count&gt;<DD>
Number of samples (default=100)
<DT><B>-p</B><DD>
Output frequency peaks only
<DT><B>-c</B><DD>
Output curves only

<DT><B>-pc</B><DD>
Output curves only with peak refinements
</DL>
<A NAME="lbAH">&nbsp;</A>
<H2>EXAMPLES</H2>

<P>
<DL COMPACT>
<pre>
gpell &lt; <a href="test.model">test.model</a>
</pre>
Calculate fundamental ellipticity curve from 0.2 Hz to 20 Hz for model
'<a href="test.model">test.model</a>'.

<pre>
gpell &lt; <a href="test.model">test.model</a> | figue <B>-c</B>
</pre>
Calculate the same ellipticity curve and plot it.
<pre>
gpell &lt; <a href="test.model">test.model</a> | figue <B>-c</B> <B>-m</B> <a href="ell.mkup">ell.mkup</a>

</pre>
Show the same ellipticity curve on a log-log plot. '<a href="ell.mkup">ell.mkup</a>' is a tar.gz
file containing a <a href="ell.mkup.contents.xml">xml description of the graphic format</a>, it can be
generated from figue's interface.
</DL>

<A NAME="copyright">&nbsp;</A>
<H2>COPYRIGHT</H2>

<p>Copyright  2002-2007 Marc Wathelet. All rights reserved.<br/>
<p>Permission is granted to copy, distribute and/or modify this document under the terms of the GNU Free Documentation License, Version 1.2 or any later version published by the Free Software Foundation; with the Invariant Section being "GNU Free Documentation License". A copy of the license is found in geopsy manual in section entitled "<a href="http://www.geopsy.org/documentation/geopsy/license-fdl.html">GNU Free Documentation License</a>".</p>
#endif
}

QString latexFilter( QString str )
{
  str.replace("\\","\\\\");
  str.replace("_","\\_");
  str.replace("$","\\$");
  str.replace("\n","\\\\\n");
  str.replace("<","\\textless ");
  str.replace(">","\\textgreater ");
  str.replace("#","\\#");
  str.replace("^","\\textasciicircum");
  str.replace("|","\\textbar");
  return str;
}

void QtbApplicationHelp::execLatex()
{
  QTB_TRACE;
  qtbApp << "\\documentclass[a4paper,10pt]{article}\n"
            "\n"
            "\\usepackage[a4paper, tmargin=2.1cm, bmargin=2.4cm, hmargin=2.5cm]{geometry}\n"
            "\\usepackage[latin1]{inputenc}\n"
            "\n"
            "\\begin{document}\n"
            "\\begin{center}\n"
            "\\noindent\\large ";
  qtbApp << latexFilter(QCoreApplication::applicationName().toUpper());
  qtbApp << "\n"
            "\\end{center}\n"
            "\\noindent\\textbf{NAME}\\\\\n"
            "\\hspace*{2cm}\\begin{minipage}{14cm}\n";
  qtbApp << latexFilter(QCoreApplication::applicationName());
  qtbApp << " - manual page for version ";
  qtbApp << latexFilter(QtbCoreApplication::version(QCoreApplication::applicationName()));
  qtbApp << "\n\\end{minipage}\n"
            "\n"
            "\\vspace{0.5cm}\n"
            "\\noindent\\textbf{SYNOPSIS}\\\\\n"
            "\\hspace*{2cm}\\begin{minipage}{14cm}\n";
  qtbApp << latexFilter(_optionSummary);
  qtbApp << "\n\\end{minipage}\n"
            "\n"
            "\\vspace{0.5cm}\n"
            "\\noindent\\textbf{DESCRIPTION}\\\\\n"
            "\\hspace*{2cm}\\begin{minipage}{14cm}\n";
  qtbApp << latexFilter(_comments);
  qtbApp << "\n\\end{minipage}\n";
  for( QList<OptionGroup>::iterator it = _options.begin(); it!=_options.end(); it++ ) {
    const OptionGroup& g = *it;
    if (g.options.isEmpty()) {
      qtbApp << "\n"
                "\\vspace{0.5cm}\n"
                "\\noindent\\hspace*{1cm}\\textbf{";
      qtbApp << latexFilter(g.title);
      qtbApp << "}\n";
    } else {
      qtbApp << "\n"
                "\\vspace{0.5cm}\n"
                "\\noindent\\hspace*{1cm}\\textbf{";
      qtbApp << latexFilter(g.title);
      qtbApp << " options:}\\\\\n";
      for( QList<Option>::const_iterator it = g.options.begin(); it!=g.options.end(); it++ ) {
        const Option& o = *it;
        qtbApp << "\n"
                  "\\noindent\\hspace*{2cm}\\textbf{";
        qtbApp << latexFilter(o.option);
        qtbApp << "}\\\\\n"
                  "\\hspace*{4cm}\\begin{minipage}{12cm}\n";
        qtbApp << latexFilter(o.comments);
        qtbApp << "\n\\end{minipage}\n\n";
      }
    }
  }
  if ( !_examples.isEmpty() ) {
    qtbApp << "\n"
              "\\vspace{0.5cm}\n"
              "\\noindent\\textbf{EXAMPLES}\n\n";
    for ( QList<Example>::iterator it = _examples.begin(); it != _examples.end(); it++ ) {
      Example& o = *it;
      qtbApp << "\\vspace{0.2cm}\n";
      qtbApp << "\\noindent\\hspace*{3cm}\\begin{minipage}{12cm}\n";
      qtbApp << latexFilter(o.command);
      qtbApp << "\n\\end{minipage}\n\n";
      qtbApp << "\\vspace{0.2cm}\n";
      qtbApp << "\\noindent\\hspace*{2cm}\\begin{minipage}{14cm}\n";
      qtbApp << latexFilter(o.comments);
      qtbApp << "\n\\end{minipage}\n\n";
    }
  }
  if ( !_seeAlso.isEmpty() ) {
    qtbApp << "\n"
              "\\vspace{0.5cm}\n"
              "\\noindent\\textbf{SEE ALSO}\\\\\n"
              "\\hspace*{2cm}\\begin{minipage}{14cm}\n";
    qtbApp << latexFilter(_seeAlso);
    qtbApp << "\n\\end{minipage}\n";
  }
  qtbApp << "\n"
            "\\vspace{0.5cm}\n"
            "\\noindent\\textbf{COPYRIGHT}\\\\\n"
            "\\hspace*{2cm}\\begin{minipage}{14cm}\n";
  qtbApp << latexFilter(qtbApp.copyright());
  qtbApp << "\n\\end{minipage}\n";

  qtbApp << "\\end{document}\n";
  qtbApp << flush;
}

/*!
  Print string \a p to stdout. Prefix \a linePrefix and indent \a indent are
  inserted before each line. qtbApp.terminalCols() is the maximum number of
  characters per line.
*/
void QtbApplicationHelp::print( QString p, QString linePrefix, int indent )
{
  QTB_TRACE;
  QString line;
  QString mainIndentStr, customIndentStr;
  bool newLine = true;
  mainIndentStr.fill(' ',indent);
  while (!p.isEmpty()) {
    bool lastNewLine = newLine;
    line = getLine(p, qtbApp.terminalCols()-indent-linePrefix.count()-customIndentStr.count(), &newLine);
    qtbApp << linePrefix << mainIndentStr << customIndentStr << line << "\n";
    if (lastNewLine && !newLine) {
      // Set indent by the position of the last double space
      customIndentStr = getCustomIndent( line );
      newLine = false;
    } else if(newLine) {
      customIndentStr = "";
    }
  }
}

QString QtbApplicationHelp::getCustomIndent( QString& text )
{
  int i = text.lastIndexOf("  ");
  if (i>-1) {
    QString customIndentStr;
    customIndentStr.fill(' ',i+2);
    return customIndentStr;
  } else {
    return QString::null;
  }
  /*int nBlanks=0;
  int n = text.count();
  while (nBlanks<n && text[nBlanks]==' ') nBlanks++;
  if (nBlanks>0) {
    QString customIndentStr;
    customIndentStr.fill(' ',nBlanks);
    text = text.mid(nBlanks);
    return customIndentStr;
  } else {
    return QString::null;
  }*/
}

/*!
  Returns the first line of \a text. The first line is removed from \a text.
  \a newLine is set to true if a '\n' is encountered else it is set to false.
*/
QString QtbApplicationHelp::getLine( QString& text, int maxLength, bool * newLine )
{
  QTB_TRACE;
  if (maxLength<10) {
    QString line = text;
    text = "";
    return line;
  }
  QString line = text;
  int index = text.indexOf("\n");
  if (index>-1) {
    if(index<=maxLength) {
      text = text.mid(index+1);
      line.truncate(index);
      if (newLine) *newLine = true;
      return line;
    }
  }
  if(text.length()<=maxLength) {
    text.clear();
    if (newLine) *newLine = false;
    return line;
  }
  line.truncate(maxLength);
  index = line.lastIndexOf(" ");
  if( maxLength>20 && index>maxLength-20 ) {
    text = text.mid(index+1); // skip the blank
    line.truncate(index);
    if (newLine) *newLine = false;
    return line;
  }
  text = text.mid(maxLength);
  if (newLine) *newLine = false;
  return line;
}
