/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-02-09
**
***************************************************************************/

#ifndef QTB_APPLICATIONHELP_H
#define QTB_APPLICATIONHELP_H

#include "qtbtoolsdllexport.h"
#include "qtbtranslations.h"

class QTBTOOLS_EXPORT QtbApplicationHelp
{
  QTB_TRANSLATIONS("QtbApplicationHelp");
public:
  QtbApplicationHelp();
  ~QtbApplicationHelp();

  void exec( const char * group = 0 );

  void addGroup(QString title, QByteArray section);
  void addOption(QString option, QString comments);
  void setOptionSummary(QString optionSummary) { _optionSummary = optionSummary; }
  void setComments(QString comments) { _comments = comments; }
  void setSeeAlso(QString seeAlso) { _seeAlso = seeAlso; }
  void addExample(QString command, QString comments);

  QList<QByteArray> sections();
  static void print( QString p, QString linePrefix = QString::null, int indent = 0 );
  static QString getLine( QString& text, int maxLength, bool * newLine  = 0 );
private:
  struct Option {
    QString option;
    QString comments;
  };
  struct OptionGroup {
    QList<Option> options;
    QString title;
    QByteArray section;
  };
  struct Example {
    QString command;
    QString comments;
  };
  QList<OptionGroup> _options;
  QList<Example> _examples;
  QString _optionSummary;
  QString _comments;
  QString _seeAlso;
  void print(const OptionGroup& g);
  void execWiki();
  void execLatex();
  static QString getCustomIndent( QString& text );
};

#endif // QTB_APPLICATIONHELP_H
