/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-07-06
**
***************************************************************************/

#ifndef QTB_CACHE_H
#define QTB_CACHE_H

#include "qtbtoolsdllexport.h"
#include "qtbtranslations.h"

#define QTB_INV_MEGABYTES 9.5367431640625e-07
#define QTB_MEGABYTES 1048576.0

class QtbCacheItem;

class QtbAllocatedCacheItem : private QList<const QtbCacheItem *>
{
public:
  QtbAllocatedCacheItem() {}
  ~QtbAllocatedCacheItem() {}

  const QtbCacheItem * bestToFree( int priority );

  void add( const QtbCacheItem * sig ) { append( sig ); }
  void remove( const QtbCacheItem * sig ) { removeAt( indexOf( sig ) ); }
  void clear() { QList<const QtbCacheItem *>::clear(); }
  int count() const { return QList<const QtbCacheItem *>::count(); }

  QString humanInfo() const;
};

class QTBTOOLS_EXPORT QtbCache
{
  QTB_TRANSLATIONS("QtbCache");
public:
  QtbCache();
  ~QtbCache() {}

  // Parameters, not changed while cache is running
  void setSize(double nMegaBytes) { _maxLoadedBytes = nMegaBytes * QTB_MEGABYTES; }
  void setSwapDir(const QDir& d);

  // Memory allocation and desallocation
  bool makeAvailable( const QtbCacheItem * item );
  void free( QtbCacheItem * item, bool saveTemp = true );
  bool enlarge( QtbCacheItem * item, double finalSize );

  double freeBytes() const { return _maxLoadedBytes -_loadedBytes;}
  double freeMegaBytes() const { return freeBytes() * QTB_INV_MEGABYTES;}
private:
  friend class QtbCacheItem;
  bool free( double byteSize, int priority );

  QtbAllocatedCacheItem _allocatedList;
  double _maxLoadedBytes;
  double _loadedBytes;
  QDir _swapDir;
  mutable QMutex _mutex;
};

#endif // QTB_CACHE_H
