/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-07-06
**
***************************************************************************/

#ifndef QTB_CACHEITEM_H
#define QTB_CACHEITEM_H

#include <time.h>

#include <QDir>
#include <QMutex>
#include <QReadWriteLock>

#include "qtbglobal.h"
#include "qtbtoolsdllexport.h"
#include "qtbtrace.h"

class QTBTOOLS_EXPORT QtbCacheItem
{
public:
  inline QtbCacheItem( );
  inline QtbCacheItem( const QtbCacheItem& o );
  inline virtual ~QtbCacheItem();

  void setPriority( int priority ) const;
  int priority() const;

  int savePriority( const QtbCacheItem * item ) const;
  void restorePriority( int priority ) const;
protected:
  friend class QtbCache;
  friend class QtbAllocatedCacheItem;

  inline void lockData() const;
  inline void unlockData() const;

  // Memory management for cached data
  bool isSaved() const { return _saved; }
  virtual bool isAllocated() const = 0;
  virtual int dataSize() const = 0;
  virtual bool allocate() const = 0;
  virtual void free() const = 0;
  virtual void save( QDir& d ) const = 0;
  virtual void load( QDir& d ) const = 0;

  inline QString swapFileName() const { return QString("cache_data_%1").arg( ( ulong ) this, 0, 16 ); }
  virtual QString debugName() const { return swapFileName(); }
private:
  mutable QMutex _dataLock;
  mutable clock_t _lastAccess;
  //mutable QMutex _priorityMutex; // protect on priority
  //mutable int _priority:30;
  //mutable bool _priorityReadOnly:1;
  mutable bool _saved:1;
};

inline QtbCacheItem::QtbCacheItem( ) : _dataLock(QMutex::Recursive)
{
  QTB_TRACE;
  _lastAccess = clock();
  _saved = false;
  //_priority = 0;
  //_priorityReadOnly = false;
}

inline QtbCacheItem::QtbCacheItem( const QtbCacheItem& /*o*/ ) : _dataLock(QMutex::Recursive)
{
  QTB_TRACE;
  _lastAccess = clock();
  _saved = false;
  //_priority = o._priority;
  //_priorityReadOnly = false;
}

inline QtbCacheItem::~QtbCacheItem()
{
  QTB_ASSERT( !_saved );
  //QTB_ASSERT( !_priorityReadOnly );
}

inline void QtbCacheItem::lockData() const
{
  _dataLock.lock();
}

inline void QtbCacheItem::unlockData() const
{
  _dataLock.unlock();
}

#endif // QTB_CACHEITEM_H
