/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-08-07
**
***************************************************************************/

#ifndef QTB_COMPATQT43_H
#define QTB_COMPATQT43_H

#include <qglobal.h>

#if (QT_VERSION >= QT_VERSION_CHECK(4, 4, 0))
// Use "" to avoid include in stable.h
#include "QAtomicInt"
#else
#include "qatomic.h"

class QAtomicInt
{
public:
  inline QAtomicInt(int value = 0)
  {
    _value = value;
  }

  inline QAtomicInt(const QAtomicInt &other)
  {
    _value = other._value;
  }

  inline QAtomicInt &operator=(int value)
  {
    _value = value;
    return *this;
  }

  inline QAtomicInt &operator=(const QAtomicInt &other)
  {
    _value = other._value;
    return *this;
  }

  inline bool operator==(int value) const
  {
    return _value == value;
  }

  inline bool operator!=(int value) const
  {
    return _value != value;
  }

  inline bool operator!() const
  {
    return _value == 0;
  }

  inline operator int() const
  {
    return _value;
  }

  inline int fetchAndStoreOrdered(int newValue)
  {
    return q_atomic_set_int(&_value, newValue);
  }

  inline bool testAndSetOrdered(int expectedValue, int newValue)
  {
    return q_atomic_test_and_set_int(&_value, expectedValue, newValue);
  }
private:
  volatile int _value;
};

#endif

#endif // QTB_COMPATQT43_H
