/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-04-11
**
***************************************************************************/

#ifndef QTB_COMPLEXEXPONENTIAL_H
#define QTB_COMPLEXEXPONENTIAL_H

#include "qtbtoolsdllexport.h"
#include "qtbcomplex.h"

class QTBTOOLS_EXPORT QtbComplexExponential
{
public:
  enum Sign { Positive, Negative };
  QtbComplexExponential( int n, Sign s );
  ~QtbComplexExponential();

  inline const QtbComplex& value( long int x );
private:
  QtbComplex * _cache;
  Sign _s;
  int _n;
  double _dn;
  double _invn;
};

inline const QtbComplex& QtbComplexExponential::value( long int x )
{
  if ( x >= _n ) {
    x = x % _n;
  }
  QtbComplex& c = _cache[x];
  if ( c.isNull() ) {
    switch (_s) {
    case Positive:
      c.setUnitExp( 2 * M_PI * (double)x*_invn );
      break;
    case Negative:
      c.setUnitExp( -2 * M_PI * (double)x*_invn );
      break;
    }
  }
  return c;
}

#endif // QTB_COMPLEXEXPONENTIAL_H
