/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-07-15
**
***************************************************************************/

#ifndef QTB_COMPLEXMATRIX_H
#define QTB_COMPLEXMATRIX_H

#include "qtbmatrix.h"
#include "qtbmatrixiterator.h"
#include "qtbcomplex.h"
#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbComplexMatrix: public QtbMatrix<QtbComplex>
{
public:
  QtbComplexMatrix() : QtbMatrix<QtbComplex>() {}
  QtbComplexMatrix( int ndim ) : QtbMatrix<QtbComplex>( ndim, ndim ) {}
  QtbComplexMatrix( int nrow, int ncol ) : QtbMatrix<QtbComplex>( nrow, ncol ) {}
  QtbComplexMatrix( const QtbMatrix<QtbComplex> m ) : QtbMatrix<QtbComplex>( m ) {}
  QtbComplexMatrix( const QtbComplexMatrix& m ) : QtbMatrix<QtbComplex>( m ) {}

  inline QtbComplexMatrix conjugate() const;
};

inline QtbComplexMatrix QtbComplexMatrix::conjugate() const
{
  QtbComplexMatrix r(rowCount(),columnCount());
  QtbMatrixIterator<QtbComplex> itThis(*this);
  QtbMutableMatrixIterator<QtbComplex> itR(r);
  while(itThis.hasNext()) {
    itThis.next();
    itR.next();
    *itR = ::conjugate(*itThis);
  }
  return r;
}

QTBTOOLS_EXPORT QTextStream& operator<<( QTextStream& s, const QtbComplexMatrix& m );

#endif // QTB_COMPLEXMATRIX_H
