/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-07-10
**
***************************************************************************/

#include <time.h>

#include "qtbconsoleprogress.h"
#include "qtbglobal.h"
#include "qtbapplicationhelp.h"
#include "qtbcoreapplication.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbConsoleProgress qtbconsoleprogress.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

void QtbConsoleProgress::setCaption( QString c )
{
  QTB_TRACE;
  QString line;
  QString str;
  while (c.length()>0) {
    line = QtbApplicationHelp::getLine(c, qtbApp.terminalCols());
    str+=line+"\n";
  }
  str.chop(1);
  QMutexLocker ml(&_mutex);
  _caption=str;
}

void QtbConsoleProgress::paint( QString caption, int value, int maxValue )
{
  static const QString fmt("%1: %2");
  QString msg = fmt.arg(caption).arg( valueString( value, maxValue ) );
  msg=msg.leftJustified(qtbApp.terminalCols()-1);
  fprintf(stderr,"%c[1A%s\n", 0x1B, msg.toAscii().data());
#ifdef Q_WS_WIN
  fflush(stderr);
#endif
}

void QtbConsoleProgress::begin()
{
  QTB_TRACE;
  QMutexLocker ml(&_mutex);
  fprintf(stderr,"\n");
}

void QtbConsoleProgress::end( int val )
{
  QTB_TRACE;
  QString msg;
  QMutexLocker ml(&_mutex);
  if (val>_maxValue) {
    msg = QString("%1: %2").arg(_caption).arg(val);
  } else {
    msg = QString("%1: 100 %").arg(_caption);
  }
  msg=msg.leftJustified(qtbApp.terminalCols()-1);
  fprintf(stderr,"%c[1A%s\n", 0x1B, msg.toAscii().data());
}
