/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-11-12
**
***************************************************************************/

#include <math.h>

#include "qtbdirectionalsearch.h"
#include "qtbglobal.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbDirectionalSearch qtbdirectionalsearch.h
  \brief Search a grid for maxima (global or local) along one direction only

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
QtbDirectionalSearch::QtbDirectionalSearch()
{
  QTB_TRACE;
  _precision=0.001;
  _nr = 0;
  _r = 0;
}

/*!
  Description of destructor still missing
*/
QtbDirectionalSearch::~QtbDirectionalSearch()
{
  QTB_TRACE;
  delete [] _r;
}

/*!
  Set current azimuth of search
*/
void QtbDirectionalSearch::setAzimuth( double azimuth )
{
  QTB_TRACE;
  _cx=cos(azimuth);
  _sy=sin(azimuth);
}

/*!
  Init sampling in radius. setFunction() must be called before.
*/
void QtbDirectionalSearch::setGrid(double minR, double maxR, double dR)
{
  QTB_TRACE;
  QTB_ASSERT(_function);

  int nr=(int)floor((maxR-minR)/dR+0.5)+1;
  bool nChanged = false;
  if (nr != _nr) {
    delete [] _r;
    _r=new double [nr];
    _nr = nr;
    nChanged = true;
  }

  _function->initGrid(_nr);
  for (int i=0;i<_nr;i++) {
    _r[i]=minR+dR*(double)i;
    _function->initGrid(_cx*_r[i], _sy*_r[i], i);
  }
}

/*!
  Search current azimuth for its maximum and refine it
*/
double QtbDirectionalSearch::globalMax()
{
  QTB_TRACE;
  int maxi = 0;
  double val,maxVal=-1e99;
  for (int i=0; i<_nr; i++) {
    val=_function->value(_cx*_r[i],_sy*_r[i], i);
    if (val>maxVal) {
      maxVal=val;
      maxi=i;
    }
  }
  // Refine search by recursion and re-regridding
  return refineMax(lowLimit(maxi, _r),highLimit(maxi, _r, _nr));
}

QtbFunction2MaximaIterator QtbDirectionalSearch::localMax(int nMax, double absThres, double relThres)
{
  QTB_TRACE;
  double * map=new double [_nr];
  for (int i=0; i<_nr; i++) {
    map[i]=_function->value(_cx*_r[i],_sy*_r[i]);
  }
  _localMaxima.clear();
  int nm=_nr-1;
  for (int i=1;i<nm;i++) {
    if (map[i-1]>0 && map[i]>map[i-1] &&
        map[i+1]>0 && map[i]>map[i+1]) {
      double val = refineMax(_r[i-1], _r[i+1]);
      _localMaxima.append(QtbFunction2SearchMaximum(val, _pos));
    }
  }
  delete [] map;
  return QtbFunction2Search::localMax(nMax, absThres, relThres);
}

inline void QtbDirectionalSearch::checkForMax(double r, double& maxVal, double& maxR)
{
  QTB_TRACE;
  double val=_function->value( _cx*r, _sy*r );
  if (val>maxVal) {
    maxVal=val;
    maxR=r;
  }
}

/*!
  Refines the maximum of the grid in an area defined by radius limits.
  return value of the refined maximum
*/
double QtbDirectionalSearch::refineMax(double minR, double maxR)
{
  QTB_TRACE;
  double r=(minR+maxR)*0.5;
  double dr=(maxR-minR)*0.5;

  // get value for (x) supposed to be the maxVal
  double maxVal=_function->value( _cx*r, _sy*r );
  double absPrecision;
  if (_relative) {
    absPrecision=fabs(_precision*r);
  } else {
    absPrecision=_precision;
  }

  while (dr>absPrecision) {
    dr*=0.5;

    double r1=r-dr;
    double r2=r+dr;

    checkForMax(r1, maxVal, r);
    checkForMax(r2, maxVal, r);
  }
  _pos.setX(_cx*r);
  _pos.setY(_sy*r);
  return maxVal;
}
