/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-11-12
**
***************************************************************************/

#ifndef QTB_DIRECTIONALSEARCH_H
#define QTB_DIRECTIONALSEARCH_H

#include "qtbtoolsdllexport.h"
#include "qtbfunction2search.h"

class QTBTOOLS_EXPORT QtbDirectionalSearch : public QtbFunction2Search
{
public:
  QtbDirectionalSearch();
  ~QtbDirectionalSearch();

  void setAzimuth( double azimuth );
  void setGrid(double minR, double maxR, double dR);
  void setFunction( QtbAbstractFunction2 * f ) { _function = f; }

  double globalMax();
  QtbFunction2MaximaIterator localMax(int nMax, double absThres=0.0, double relThres=0.0);
  void setRelativePrecision(double p) {_precision=p; _relative = true; }
  void setAbsolutePrecision(double p) {_precision=p; _relative = false; }

  double value( double r ) const { return _function->value( _cx*r, _sy*r ); }
protected:
  double _cx, _sy;
  int _nr;
  double * _r;
  double _precision;
private:
  double refineMax(double minR, double maxR);
  inline void checkForMax(double r, double& maxVal, double& maxR);
};

#endif // QTB_DIRECTIONALSEARCH_H
