/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-07-15
**
***************************************************************************/

#include "qtbdoublematrix.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbMatrix3x3 qtbdoublematrix.h
  \brief Brief description of class still missing

  3x3 matrix with double values
*/

/*!
  \class QtbDoubleMatrix qtbdoublematrix.h
  \brief Brief description of class still missing

  NxM matrix with double values stored in the same way as LAPACK: iCol*nRows+iRow
*/

QTextStream& operator<<( QTextStream& s, const QtbDoubleMatrix& m )
{
  QTB_TRACE;
  s << "\t";
  for ( int j = 0;j < m.columnCount();j++ ) s << j << "\t";
  s << "\n";
  for ( int i = 0;i < m.rowCount();i++ ) {
    QtbMatrixRowIterator<double> it( m, i );
    s << i << ":\t";
    while ( it.hasNext() ) {
      it.next();
      s << *it << "\t";
    }
    s << "\n";
  }
  s << flush;
  return s;
}

/*!
  Set matrix as a rotation matrix around axis \a ax. The angle is counted using the right hand convention.
  Angle is in radians.
*/
void QtbMatrix3x3::rotation(Axis ax, double angle)
{
  QTB_TRACE;
  double cangle=cos(angle);
  double sangle=sin(angle);
  double * values = _d->values();
  switch (ax) {
  case XAxis:
    values[0]=1;        // (0,0)
    values[3]=0;        // (0,1)
    values[6]=0;        // (0,2)
    values[1]=0;        // (1,0)
    values[4]=cangle;   // (1,1)
    values[7]=sangle;   // (1,2)
    values[2]=0;        // (2,0)
    values[5]=-sangle;  // (2,1)
    values[8]=cangle;   // (2,2)
    break;
  case YAxis:
    values[0]=cangle;   // (2,2)
    values[3]=0;        // (2,1)
    values[6]=sangle;   // (2,2)
    values[1]=0;        // (1,0)
    values[4]=1;        // (1,1)
    values[7]=0;        // (1,2)
    values[2]=-sangle;  // (2,0)
    values[5]=0;        // (2,1)
    values[8]=cangle;   // (2,2)
    break;
  case ZAxis:
    values[0]=cangle;   // (0,0)
    values[3]=sangle;   // (0,1)
    values[6]=0;        // (0,2)
    values[1]=-sangle;  // (1,0)
    values[4]=cangle;   // (1,1)
    values[7]=0;        // (1,2)
    values[2]=0;        // (2,0)
    values[5]=0;        // (2,1)
    values[8]=1;        // (2,2)
    break;
  }
}
