/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-07-15
**
***************************************************************************/

#ifndef QTB_DOUBLEMATRIX_H
#define QTB_DOUBLEMATRIX_H

#include "qtbtoolsdllexport.h"
#include "qtbmatrix.h"
#include "qtbmatrixiterator.h"

class QTBTOOLS_EXPORT QtbDoubleMatrix: public QtbMatrix<double>
{
public:
  QtbDoubleMatrix() : QtbMatrix<double>() {}
  QtbDoubleMatrix( int ndim ) : QtbMatrix<double>( ndim ) {}
  QtbDoubleMatrix( int nrow, int ncol ) : QtbMatrix<double>( nrow, ncol ) {}
  QtbDoubleMatrix( const QtbMatrix<double> m ) : QtbMatrix<double>( m ) {}
  QtbDoubleMatrix( const QtbDoubleMatrix& m ) : QtbMatrix<double>( m ) {}
};

QTBTOOLS_EXPORT QTextStream& operator<<( QTextStream& s, const QtbDoubleMatrix& m );

class QTBTOOLS_EXPORT QtbMatrix3x3: public QtbDoubleMatrix
{
public:
  QtbMatrix3x3() : QtbDoubleMatrix( 3, 3 ) {}
  QtbMatrix3x3( const QtbMatrix3x3& o) : QtbDoubleMatrix( static_cast<const QtbDoubleMatrix&>(o) ) {}

  enum Axis {XAxis, YAxis, ZAxis};
  void rotation( Axis ax, double angle );

  inline QtbPoint operator*( const QtbPoint& p ) const;
};

inline QtbPoint QtbMatrix3x3::operator*( const QtbPoint& p ) const
{
  QTB_TRACE;
  QtbPoint rp;
  const double * values = _d->values();
  rp.setX(values[0]*p.x()+values[3]*p.y()+values[6]*p.z());
  rp.setY(values[1]*p.x()+values[4]*p.y()+values[7]*p.z());
  rp.setZ(values[2]*p.x()+values[5]*p.y()+values[8]*p.z());
  return rp;
}

#endif // QTB_DOUBLEMATRIX_H
