/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-04-20
**
***************************************************************************/

#ifndef QTB_EXPRESSIONCONTEXT_H
#define QTB_EXPRESSIONCONTEXT_H

#include <QVariant>
#include <QVector>

#include "qtbtoolsdllexport.h"

class QtbExpressionStorage;
class QtbExpressionAction;

class QTBTOOLS_EXPORT QtbExpressionContext
{
public:
  QtbExpressionContext( QtbExpressionContext * parent = 0 ) { _parent = parent; }
  virtual ~QtbExpressionContext();

  void addVariable( const QString& name, const QVariant& val) { addVariable(name, 0, val); }
  void addVariable( const QString& name, int index, const QVariant& val);
  int addVariable( const QString& name, QtbExpressionStorage * var);

  QStringList variables() const;
  bool isValidVariable(const QString& name) const;
  QVariant variableValue( const QString& name, int index=0 ) const;
  QtbExpressionStorage * variable(int varIndex) const { return _varValues[varIndex]; }

  virtual QStringList functions() const;
  virtual bool isValidFunction(const QString& name) const;
  virtual QVariant functionValue(const QString& name, const QVector<QtbExpressionAction *>& args) const;
private:
  QtbExpressionContext * _parent;
  QMap<QString, int> _varNames;
  QVector<QtbExpressionStorage *> _varValues;
};

#endif // QTB_EXPRESSIONCONTEXT_H
