/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-04-10
**
***************************************************************************/

#include "qtbexpressionstring.h"
#include "qtbexpressionparser.h"
#include "qtbcoreapplication.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbExpressionString qtbexpressionstring.h
  \brief Brief description of class still missing

  Q string containing header formulas between {}
*/

void QtbExpressionString::clear()
{
  QTB_TRACE;
  for (QList<QtbExpressionActions>::iterator ite = _engines.begin();ite!=_engines.end();ite++ ) {
    qDeleteAll(*ite);
  }
  _engines.clear();
}

bool QtbExpressionString::setPattern( QString pattern, QtbExpressionContext * context )
{
  QTB_TRACE;
  _pattern="";
  const QChar * startExpression = 0;
  const QChar * startString = pattern.data();
  const QChar * ptr = startString;
  while ( true ) {
    switch ( ptr->unicode() ) {
    case 0x0000:   // '\0'
      if (startString) {
        QtbStringSection( startString, ptr ).appendTo(_pattern);
      } else if (startExpression) {
        qtbApp << tr( "Unmatched { before reaching the end of buffer" ) << endl;
        return false;
      }
      return true;
    case 0x007B:  // '{'
      QtbStringSection( startString, ptr ).appendTo( _pattern );
      startExpression = ptr+1;
      startString = 0;
      break;
    case 0x007D:   // '}'
      {
        QtbExpressionParser p;
        QtbExpressionActions e=p.parse( QtbStringSection( startExpression, ptr ).toString(), context );
        if (e.isEmpty()) {
          return false;
        }
        _pattern += "%"+QString::number(_engines.count());
        _engines.append(e);
      }
      startExpression = 0;
      startString = ptr+1;
    case 0x0022:   // '"'
      if (startExpression) {
        if (!skipString(ptr)) return false;
      }
      break;
    default:
      break;
    }
    ptr++;
  }
}

QString QtbExpressionString::value( )
{
  QTB_TRACE;
  QString s = _pattern;
  for (QList<QtbExpressionActions>::iterator ite = _engines.begin();ite!=_engines.end();ite++ ) {
    QtbExpressionActions& a = *ite;
    for (QtbExpressionActions::iterator ita=a.begin();ita!=a.end();++ita) {
      (*ita)->value();
    }
    s = s.arg( a.context()->variableValue("Return").toString() );
  }
  return s;
}

bool QtbExpressionString::skipString( const QChar *& ptr)
{
  QTB_TRACE;
  while ( true ) {
    switch ( ptr->unicode() ) {
    case 0x0000:
      qtbApp << tr( "Unmatched \" before reaching the end of buffer" ) << endl;
      return false;
    case 0x005C:   // '\\'
      ptr++;
      break;
    case 0x0022:   // '"'
      return true;
    }
    ptr++;
  }
}
