/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2004-03-10
**
***************************************************************************/

#ifndef QTB_FILE_H
#define QTB_FILE_H

#include <zlib.h>

#include <QString>
#include <QDir>

#include "qtbtoolsdllexport.h"

class QTextStream;

namespace QtbFile {
  QTBTOOLS_EXPORT QStringList expand(const QStringList& fileNames);
  QTBTOOLS_EXPORT QStringList expand(const QString& fileName);
  QStringList expand(QDir d, QStringList filters, QString levelName, int level);
  QTBTOOLS_EXPORT QString uniqueName( QString fileName, const QDir& d );
  // Text file reading util functions
  QTBTOOLS_EXPORT char * readAll(const char * fileName);
  QTBTOOLS_EXPORT bool readLine(char *& buf, int& bufLen, gzFile f, bool delEOL=false);
  QTBTOOLS_EXPORT bool readLine(char *& buf, int& bufLen, FILE * f, bool delEOL=false);
  QTBTOOLS_EXPORT QString readLine(FILE * f, bool delEOL=false);
  QTBTOOLS_EXPORT void readBreakLine(char *& buf, int& bufLen, FILE * f);
  QTBTOOLS_EXPORT bool readCleanUp(char * buf, bool returnValue);
  QTBTOOLS_EXPORT void readLineNoComments(char * buf, int& bufLen, FILE *  f, QString * comments);
  QTBTOOLS_EXPORT QString readLineNoComments( QTextStream& s, QString * comments );
  QTBTOOLS_EXPORT char * stripWhiteSpace(char * buf);
  QTBTOOLS_EXPORT const char * stripWhiteSpace(const char * buf, int& len);
  QTBTOOLS_EXPORT char * nextField(char *& buf, const char * sep);
  QTBTOOLS_EXPORT void getKeyValue(const char * fileNAme, const char * varName, QString& value);
  // Util function for library or plugin management
  QTBTOOLS_EXPORT QStringList getLibList(QStringList pathList);
  // Util for reading fixed file formats
  QTBTOOLS_EXPORT qint32 fromBigEndian( qint32 v );
  QTBTOOLS_EXPORT qint32 fromLittleEndian( qint32 v );
  QTBTOOLS_EXPORT qint16 fromBigEndian( qint16 v );
  QTBTOOLS_EXPORT qint16 fromLittleEndian( qint16 v );
}

#endif // QTB_FILE_H
