/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-11-12
**
***************************************************************************/

#include "qtbfunction2search.h"
#include "qtbabstractfunction2.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbFunction2Search qtbfunction2search.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
QtbFunction2Search::QtbFunction2Search()
{
  QTB_TRACE;
  _relative = true;
  _function = 0;
}

/*!
  Delete current function. To prevent automatic deletion, call takeFunction() before.
*/
QtbFunction2Search::~QtbFunction2Search()
{
  QTB_TRACE;
  delete _function;
}

/*!
  Set current function. Ownership is transfered to this object. To get it back you must
  call takeFunction().
*/
void QtbFunction2Search::setFunction( QtbAbstractFunction2 * f )
{
  QTB_TRACE;
  delete _function;
  _function = f;
}

/*!
  Take ownership of the function object
*/
QtbAbstractFunction2 * QtbFunction2Search::takeFunction()
{
  QTB_TRACE;
  QtbAbstractFunction2 * f = _function;
  _function = 0;
  return f;
}

QtbFunction2MaximaIterator QtbFunction2Search::localMax(int nMax, double absThres, double relThres)
{
  QTB_TRACE;
  qSort(_localMaxima);
  // filter the ensemble of maxima
  // With an absolute value for the maximum
  if (absThres>0.0) {
    int i=_localMaxima.count()-1;
    while (i>=0 && _localMaxima.at(i).value()<absThres) {
      _localMaxima.removeLast();
      i--;
    }
  }
  // With a value for the maximum defined as a ratio of the global max
  if (relThres>0.0 && _localMaxima.count()>0) {
    absThres=_localMaxima.at(0).value()*relThres*0.01;
    int i=_localMaxima.count()-1;
    while (i>=0 && _localMaxima.at(i).value()<absThres) {
      _localMaxima.removeLast();
      i--;
    }
  }
  // Remove all maxima exceeding nMax
  while (_localMaxima.count()>nMax) _localMaxima.removeLast();
  return _localMaxima.begin();
}
