/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-11-12
**
***************************************************************************/

#ifndef QTB_FUNCTIONSEARCH_H
#define QTB_FUNCTIONSEARCH_H

#include "qtbtoolsdllexport.h"
#include "qtbabstractfunction2.h"
#include "qtbpoint2d.h"

class QtbFunction2SearchMaximum
{
public:
  QtbFunction2SearchMaximum(double value, const QtbPoint2D& pos) {
    _value=value;
    _pos=pos;
  }
  ~QtbFunction2SearchMaximum() {}

  bool operator<(const QtbFunction2SearchMaximum& o) const {return _value>o._value;}
  double value() const {return _value;}
  const QtbPoint2D& pos() const { return _pos; }
private:
  double _value;
  QtbPoint2D _pos;
};

typedef QList<QtbFunction2SearchMaximum>::iterator QtbFunction2MaximaIterator;

class QTBTOOLS_EXPORT QtbFunction2Search
{
public:
  QtbFunction2Search();
  ~QtbFunction2Search();

  void setFunction( QtbAbstractFunction2 * f );
  QtbAbstractFunction2 * takeFunction();

  double value(double x, double y) const { return _function->value(x,y); }
  double value(double x, double y, int index) const { return _function->value(x,y, index); }

  const QtbPoint2D& pos() const { return _pos; }
  QtbFunction2MaximaIterator localMax(int nMax, double absThres=0.0, double relThres=0.0);
  QtbFunction2MaximaIterator localMaxEnd() {return _localMaxima.end();}
protected:
  static inline double lowLimit(int i, double * axis);
  static inline double highLimit(int i, double * axis, int nAxis);

  QtbPoint2D _pos;
  QtbAbstractFunction2 * _function;
  bool _relative;
  QList<QtbFunction2SearchMaximum> _localMaxima;
};

inline double QtbFunction2Search::lowLimit(int i, double * axis)
{
  QTB_TRACE;
  return i>0 ? axis[i-1] : axis[0];
}

inline double QtbFunction2Search::highLimit(int i, double * axis, int nAxis)
{
  QTB_TRACE;
  return i<nAxis-1 ? axis[i+1] : axis[nAxis-1];
}

#endif // QTB_FUNCTIONSEARCH_H
