/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2003-10-09
**
***************************************************************************/

#include <QDir>
#include <QStringList>

#include "qtbgetopt.h"

#ifndef Q_OS_LINUX

#include <string.h>
#include <stdio.h>

#include "qtbmemorychecker.h"

int optind=1;
char * optarg;

Option_Type getOptType(char option, const char * options)
{
  const char * end_options=options+strlen(options);
  const char * ptr_options=options;
  while (ptr_options<end_options) {
    if (option==*ptr_options) {
      if (*(ptr_options+1)==':') return ArgValue;
      else return ArgNoValue;
    }
    ptr_options++;
  }
  return ArgNotValid;
}

Option_Type getOptType(const char * option, const struct option * longopts,
                       int * longind)
{
  const struct option * ptr_options=longopts;
  *longind=0;
  while (ptr_options->name!=0) {
    //printf("Arg=%s - %i - %s\n",option, *longind, ptr_options->name);
    int argLen=strlen(ptr_options->name);
    if (strncmp(option,ptr_options->name,argLen)==0) {
      if (option[argLen]=='\0' || option[argLen]=='=') {
        switch(ptr_options->has_arg) {
        case 1:
          return ArgValue;
        case 2:
          return ArgOptValue;
        default:
          return ArgNoValue;
        }
      }
    }
    ptr_options++;
    (*longind)++;
  }
  return ArgNotValid;
}

int getOptionChar(Option_Type type, int argc, char *argv[],
                  const char * options)
{
  switch (type) {
  case ArgValue:
    if (optind==argc) {
      if (options[0]==':') return ':';
      else {
        fprintf(stderr,"%s: option `-%s' requires an argument",argv[0],argv[optind-1]);
        return '?';
      }
    }
    else {
      optarg=argv[optind];
      optind++;
      return argv[optind-2][1];
    }  
  case ArgNoValue:
    return argv[optind-1][1];
  default:
    fprintf(stderr,"%s: unrecognized option '-%s'\n",argv[0],argv[optind-1]);
    return '?';
  }
}

/*!
  Function to extract one character options
  
  TODO: swaping argv pointers for non-options arguments
*/
int getopt(int argc, char *argv[], const char * options)
{
  int i=optind;
  while (i<argc && (strlen(argv[optind])!=2 || argv[optind][0]!='-')) i++;
  if (i==argc) return -1;
  optind=i+1;
  return getOptionChar(getOptType(argv[i][1], options), argc, argv, options);
}

/*!
  Function to extract long name options
  
  TODO: swaping argv pointers for non-options arguments
*/
int getopt_long(int argc, char *argv[], const char * options,
                const struct option * longopts, int * longind,
                bool onlyLong=false)
{
  int i=optind;
  while (i<argc && argv[i][0]!='-') i++;
  if (i==argc) return -1;
  optind=i+1;
  char * argVal=argv[i]+1;
  if ((argv[i][1]=='-' && ++argVal) || onlyLong) {
    //printf("Arg=%s\n",argVal);
    int argLen;
    switch (getOptType(argVal, longopts, longind)) {
    case ArgValue:
      argLen=strlen(longopts[*longind].name);
      if (argVal[argLen]=='=') {
        optarg=argVal+argLen+1;
        //printf("Arg=%s, %i\n",argVal,argLen);
        return longopts[*longind].val;
      }
      else if (optind==argc) {
        if (options[0]==':') return ':';
        else {
          fprintf(stderr,"%s: option `-%s' requires an argument\n",argv[0],argVal);
          return '?';
        }
      }
      else {
        optarg=argv[optind];
        optind++;
        //printf("arg=%s optarg=%s\n",argVal, optarg ? optarg : "(NULL)");
        return longopts[*longind].val;
      }
    case ArgOptValue:
      argLen=strlen(longopts[*longind].name);
      if (argVal[argLen]=='=') {
        optarg=argVal+argLen+1;
        //printf("Arg=%s, %i\n",argVal,argLen);
        return longopts[*longind].val;
      }
      else if (optind<argc && argv[optind][0]!='-') {
        optarg=argv[optind];
        optind++;
      }
      else optarg=0;
      //printf("arg=%s optarg=%s\n",argVal, optarg ? optarg : "(NULL)");
      return longopts[*longind].val;
    case ArgNoValue:
      if (longopts[*longind].flag) {
        *longopts[*longind].flag=longopts[*longind].val;
        return 0;
      }
      else return longopts[*longind].val;
    default:
      fprintf(stderr,"%s: unrecognized option '-%s'\n",argv[0],argVal);
      return '?';
    }
  }
  else {
    return getOptionChar(getOptType(argv[i][1], options), argc, argv, options);
  }
}

int getopt_long(int argc, char *argv[], const char * options,
                const struct option * longopts, int * longind)
{
  return getopt_long(argc, argv, options, longopts, longind, false);
}

int getopt_long_only(int argc, char *argv[], const char * options,
                     const struct option * longopts, int * longind)
{
  return getopt_long(argc, argv, options, longopts, longind, true);
}

#endif // Q_OS_LINUX
