/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2004-10-23
**
***************************************************************************/

#include <QFile>

#include "qtblogstream.h"
#include "qtbcoreapplication.h"
#include "qtbmemorychecker.h"

QtbLogStream::QtbLogStream( FILE * fileHandle )
    : QTextStream( fileHandle, QIODevice::WriteOnly )
{
  QTB_TRACE;
  _frozen = false;
  _device = 0;
}

QtbLogStream::QtbLogStream( QString * string )
    : QTextStream( string, QIODevice::WriteOnly )
{
  QTB_TRACE;
  _frozen = false;
  _device = 0;
}

QtbLogStream::QtbLogStream( QString logFile )
    : QTextStream()
{
  QTB_TRACE;
  _device = new QFile( logFile );
  if ( _device->open( QIODevice::WriteOnly ) ) {
    setDevice( _device );
  } else {
    _device->open( stdout, QIODevice::WriteOnly );
    setDevice( _device );
    *this << "WARNING: Cannot open log file " << logFile << endl;
  }
  _frozen = false;
}

QtbLogStream::~QtbLogStream()
{
  QTB_TRACE;
  if ( _device ) delete _device;
  qtbApp.streamDestroyed(this);
}

QtbLogStream& QtbLogStream::operator<<(const char * val)
{
  if (!_frozen) {
    if (_prefix.isEmpty()) {
      QTextStream::operator<< (val);
    } else {
      QTextStream::operator<< (_prefix);
      if (strchr(val,'\n')) {
        QString str(val);
        str.replace("\n","\n"+_prefix);
        QTextStream::operator<< (str);
      } else {
        QTextStream::operator<< (val);
      }
    }
  }
  return *this;
}

QtbLogStream& QtbLogStream::operator<<( const QString& val )
{
  if (!_frozen) {
    if (_prefix.isEmpty()) {
      QTextStream::operator<< (val);
    } else {
      QTextStream::operator<< (_prefix);
      if (val.contains("\n")) {
        QString str(val);
        str.replace("\n","\n"+_prefix);
        QTextStream::operator<< (str);
      } else {
        QTextStream::operator<< (val);
      }
    }
  }
  return *this;
}

QtbLogStream& QtbLogStream::operator<<( double val )
{
  if (!_frozen) {
    QTextStream::operator<< (_prefix);
    QTextStream::operator<< (val);
  }
  return *this;
}

QtbLogStream& QtbLogStream::operator<<( int val )
{
  if (!_frozen) {
    QTextStream::operator<< (_prefix);
    QTextStream::operator<< (val);
  }
  return *this;
}
