/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2004-10-23
**
***************************************************************************/

#ifndef QTB_LogStream_H
#define QTB_LogStream_H

#include <QTextStream>
#include <QMap>
#include <QThread>

#include "qtbtoolsdllexport.h"
#include "qtbtrace.h"

class QFile;

class QTBTOOLS_EXPORT QtbLogStream : public QTextStream
{
public:
  QtbLogStream( FILE * fileHandle );
  QtbLogStream( QString * string );
  QtbLogStream( QString logFile );
  virtual ~QtbLogStream();

  void setPrefix( const QString & prefix ) { _prefix = prefix; }
  void freeze( bool b ) { _frozen = b; }
  bool frozen() const { return _frozen; }
  virtual bool detach() { return true; }
  virtual int id() { return 0; }

  virtual QtbLogStream& operator<<( const char * val );
  virtual QtbLogStream& operator<<( const QString& val );
  virtual QtbLogStream& operator<<( double val );
  virtual QtbLogStream& operator<<( int val );
protected:
  bool _frozen;
  QString _prefix;
  QFile * _device;
};

inline QtbLogStream& endl(QtbLogStream &s)
{
  QTB_TRACE;
  if (!s.frozen()) s << endl;
  return s;
}

inline QtbLogStream& operator<<(QtbLogStream &s, QTextStreamFunction f)
{
  if (!s.frozen()) (*f)(s);
  return s;
}

inline QtbLogStream& operator<<(QtbLogStream &s, QTextStreamManipulator m)
{
  if (!s.frozen()) m.exec(s);
  return s;
}

#endif
