/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-07-15
**
***************************************************************************/

#include "qtbmatrixmultiply.h"
#include "qtbglobal.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbMatrixMultiply qtbmatrixmultiply.h
  \brief Brief description of class still missing

  A cache for fast matrix multiplication. The maximum dimension is 256. The size occupied for one set
  of index is 8*n^3 (for squared matrices).
*/

QTB_IMPLEMENT_NUMERICALKEY(QtbMatrixMultiplyKey)

QtbMatrixMultiply::QtbMatrixMultiply( QtbMatrixMultiplyKey * key )
  : QtbAbstractNumericalCache(key)
{
  _indexes = 0;
}

QtbMatrixMultiply::~QtbMatrixMultiply()
{
  delete [] _indexes;
}

void QtbMatrixMultiply::init()
{
  const QtbMatrixMultiplyKey& mKey = static_cast<const QtbMatrixMultiplyKey&>(key());
  QTB_ASSERT(mKey.nRows1()<256);
  QTB_ASSERT(mKey.commonDim()<256);
  QTB_ASSERT(mKey.nColumns2()<256);
  _indexes = new IndexMap[indexCount()];
  IndexMap * pIndex = _indexes;
  for( int i = 0; i<mKey.nRows1(); i++ ) {
    for( int j = 0; j<mKey.nColumns2(); j++ ) {
      int resultIndex = i+j*mKey.nRows1();
      for( int k = 0; k<mKey.commonDim(); k++ ) {
        (pIndex++)->set( resultIndex, i+k*mKey.nRows1(), k+j*mKey.commonDim() );
      }
    }
  }
}
