/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-01-29
**
***************************************************************************/

#ifndef QTB_MEMORYCHECKERPRIVATE_H
#define QTB_MEMORYCHECKERPRIVATE_H

#ifdef QTB_MEMORY_CHECKER

#include <QMutex>

#include "qtbtoolsdllexport.h"

#ifdef Q_WS_X11           // dll mechanism does not allow a secure allocation
                          // unless Qt is linked with this dll (this was the reason for qtbdebugmem.dll)
#define DO_MEMORY_CHECK
#endif
#define ABORT_ON_WARNING

// defines for new,... must be removed because they are defined here as functions
#undef new
#undef isBlockDamaged
#undef printBlockInfo

#ifdef DO_MEMORY_CHECK

struct QtbBlockInfo
{
  void * blockAddress;
  unsigned int sz;
  bool record; // true if this block must be reported as undeleted if still in memory
  char * fName;
  int fLine;
  QtbBlockInfo * next;
  QtbBlockInfo * previous;
};

#define QTB_NOMANSLAND_SIZE 16
#define QTB_HEADER_SIZE (sizeof(QtbBlockInfo *)+QTB_NOMANSLAND_SIZE)
#define QTB_TAIL_SIZE QTB_NOMANSLAND_SIZE

class QTBTOOLS_EXPORT QtbMemoryCheckerPrivate
{
public:
  static inline void* allocateBlock( std::size_t sz, char* file, int line );
  static inline void freeBlock( void* v );

  static void setEnabled( bool e) { _enabled = e; }
  static bool isEnabled() { return _enabled; }

  static void setRecord( bool r ) { _record = r; }
  static void setVerbose( bool v ) { _verbose = v; }

  static void checkBlocks( bool showLib, bool showUndeleted, char* file, int line);
  static bool isBlockDamaged(void* v, char* file, int line);
  static void printBlockInfo(void * ptr, char* file, int line);
private:
  static void printHeader(FILE * f, const char * prefix, unsigned char * ptr );
  static inline bool testNoMansLand( unsigned char * ptr );
  static inline QtbBlockInfo * addBlock( void * blockAddress, unsigned int sz, char * fName, int fLine );
  static inline void delBlock( QtbBlockInfo * b );
  static inline QtbBlockInfo * findBlock( void * blockAddress );
  static inline void srcLocation( char * fName, int fLine );
  static inline void srcLocation( QtbBlockInfo * b );

  static bool _enabled;
  static bool _record;
  static bool _verbose;
  static QtbBlockInfo * _infos;
  static QMutex _mutex;
};

#endif // DO_MEMORY_CHECK
#endif // QTB_MEMORY_CHECKER
#endif // QTB_MEMORYCHECKERPRIVATE_H
