/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-07-25
**
***************************************************************************/

#ifndef QTB_NAMEDPOINT_H
#define QTB_NAMEDPOINT_H

#include "qtbpoint.h"
#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbNamedPoint : public QtbPoint
{
public:
  QtbNamedPoint() : QtbPoint() {}
  QtbNamedPoint( double x, double y, double z = 0.0 ) : QtbPoint( x, y, z ) {}
  QtbNamedPoint( const QtbNamedPoint& p ) : QtbPoint( p ) { _name = p._name; }
  QtbNamedPoint( const QString& name, const QtbPoint& p ) : QtbPoint( p ) { _name = name; }
  QtbNamedPoint( const QtbPoint& p ) : QtbPoint( p ) {}
  QtbNamedPoint( const QtbPoint2D& p ) : QtbPoint( p ) {}

  inline QtbNamedPoint& operator=(const QtbPoint& p);
  inline QtbNamedPoint& operator=(const QtbPoint2D& p);
  inline QtbNamedPoint& operator=(const QtbNamedPoint& p);

  const QString& name() const { return _name; }
  void setName( QString n ) { _name = n; }

  void setCoordinates( const QtbPoint& p ) { *this = p; }
  const QtbPoint& coordinates() const { return *this; }

  bool fromString(QtbStringSection str);
  QString toString(int precision=6) const;
private:
  QString _name;
};

// Copy operators
inline QtbNamedPoint& QtbNamedPoint::operator=(const QtbPoint& p)
{
  QTB_TRACE;
  QtbPoint::operator=(p);
  return *this;
}

inline QtbNamedPoint& QtbNamedPoint::operator=(const QtbPoint2D& p)
{
  QTB_TRACE;
  QtbPoint::operator=(p);
  return *this;
}

inline QtbNamedPoint& QtbNamedPoint::operator=(const QtbNamedPoint& p)
{
  QTB_TRACE;
  QtbPoint::operator=(p);
  _name = p._name;
  return *this;
}

#endif // QTB_NAMEDPOINT_H
