/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-03-28
**
***************************************************************************/

#ifndef QTB_NUMBER_H
#define QTB_NUMBER_H

#include <QList>

#include "qtbtoolsdllexport.h"
#include "qtbtrace.h"
#include "qtbtime.h"

#ifdef QTB_BIGNUM
#include <mp/mpreal.h>
#endif

class QTBTOOLS_EXPORT QtbNumber
{
public:
  enum Type { Fixed = 0, Scientific, Time };

  static Type type( char t );
  static inline double toDouble( QString str, Type t, bool * ok = 0 );
  static inline QString toString( double val, Type t, int precision );
  static double toDouble( float f );
  static double toDouble( double a ) { return a;}
#ifdef QTB_BIGNUM
  static double toDouble( mp_real a ) { return dble( a );}
#endif

  static QList<int> divisors( int n );
};

inline double QtbNumber::toDouble( QString str, Type t, bool * ok )
{
  QTB_TRACE;
  switch(t) {
  case Fixed:
  case Scientific:
    return str.toDouble( ok );
  case Time:
    return QtbTime::timeToSeconds( str );
  }
  return 0.0;
}

inline QString QtbNumber::toString( double val, Type t, int precision )
{
  QTB_TRACE;
  switch(t) {
  case Fixed:
    return QString::number( val, 'f', precision );
  case Scientific:
    return QString::number( val, 'e', precision );
  case Time:
    return QtbTime::secondsToTime( val, precision );
  }
  return QString::null;
}

#endif // QTB_NUMBER_H
