/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-02-08
**
***************************************************************************/

#ifndef QTB_PACKAGEINFO_H
#define QTB_PACKAGEINFO_H

#include <QString>
#include <QList>

#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbPackageInfo
{
public:
  QtbPackageInfo( QString package,
                  QString binDir,
                  QString libDir,
                  QString includeDir,
                  QString shareDir,
                  QString version,
                  QString versionTime,
                  QString versionType,
                  QString distribtion,
                  QString copyright,
                  QString contact );
  QtbPackageInfo( const QtbPackageInfo& o );

  const QString & package() const {return _package;}
  const QString & binDir() const {return _binDir;}
  const QString & libDir() const {return _libDir;}
  const QString & includeDir() const {return _includeDir;}
  const QString & shareDir() const {return _shareDir;}
#ifdef Q_WS_WIN
  const QString & dllDir() const { return binDir(); }
#else
  const QString & dllDir() const { return libDir(); }
#endif
  const QString & version() const {return _version;}
  const QString & versionTime() const {return _versionTime;}
  const QString & versionType() const {return _versionType;}
  const QString & distribution() const {return _distribution;}
  const QString & copyright() const {return _copyright;}
  const QString & contact() const {return _contact;}

  static QString getInstallDir( QString organization, QString package, QString dir, QString defaultDir);
  static const QtbPackageInfo * package( QString package);
  static QList<QtbPackageInfo> * list();
private:
  QString _package;
  QString _binDir;
  QString _libDir;
  QString _includeDir;
  QString _shareDir;
  QString _version;
  QString _versionTime;
  QString _versionType;
  QString _distribution;
  QString _copyright;
  QString _contact;

  static QList<QtbPackageInfo> * _list;
};

#define QTB_PACKAGE_INFO(package, PACKAGE) \
  class package##InfoInit \
  { \
  public: \
    package##InfoInit() { \
      QtbPackageInfo::list()->append( QtbPackageInfo( # package, \
            QtbPackageInfo::getInstallDir("geopsy", # package, "binDir", PACKAGE##_BINDIR), \
            QtbPackageInfo::getInstallDir("geopsy", # package, "libDir", PACKAGE##_LIBDIR), \
            QtbPackageInfo::getInstallDir("geopsy", # package, "includeDir", PACKAGE##_INCDIR), \
            QtbPackageInfo::getInstallDir("geopsy", # package, "shareDir", PACKAGE##_SHAREDIR), \
            PACKAGE##_VERSION, \
            PACKAGE##_VERSION_TIME, \
            PACKAGE##_VERSION_TYPE, \
            PACKAGE##_DISTRIBUTION, \
            PACKAGE##_COPYRIGHT, \
            PACKAGE##_CONTACT ) ); \
    } \
  }; \
  package##InfoInit autoInit##package;

#endif // QTB_PACKAGEINFO_H
