/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2002-04-15
**
***************************************************************************/

#ifndef QTB_RECT_H
#define QTB_RECT_H

#include "qtbtoolsdllexport.h"
#include "qtbpoint.h"
#include "qtbpoint2d.h"
#include "qtbcurve.h"
#include "qtbxmlclass.h"

class QTBTOOLS_EXPORT QtbRect: public QtbXMLClass
{
public:
  enum Area {AllRect,
             AboveAscDiag,
             BelowAscDiag,
             AboveDescDiag,
             BelowDescDiag};
  QtbRect();
  QtbRect( const QtbPoint& p1, const QtbPoint& p2 );
  QtbRect( double nx1, double ny1, double nx2, double ny2 );
  virtual ~QtbRect() {};

  virtual const QString& xml_tagName() const { return xmlRectTag;}
  static const QString xmlRectTag;

  void setLimits ( double nx1, double ny1, double nx2, double ny2 );
  bool includes ( const QtbRect & r ) const;
  bool includes ( const QtbPoint & p ) const;
  bool includes ( const QtbPoint2D & p ) const;
  bool includes ( const QtbPoint2D & p, Area inArea ) const;
  inline bool includes ( const QtbPoint2D& p1, const QtbPoint2D& p2 ) const;
  bool includes ( double x1, double y1, double x2, double y2 ) const;
  bool isNull() { return _x1==_x2 && _y1==_y2; }
  double x1() const { return _x1;}
  double y1() const { return _y1;}
  double x2() const { return _x2;}
  double y2() const { return _y2;}
  double width() const { return _x2 -_x1;}
  double height() const { return _y2 -_y1;}
  void setX1( double nx1 );
  void setY1( double ny1 );
  void setX2( double nx2 );
  void setY2( double ny2 );
  void add( double x, double y );
  void add( const QtbPoint& p );
  void add( const QtbPoint2D& p );
  void add(const QtbRect& r);
  QtbRect intersect( const QtbRect& r ) const;
  void intersect( QtbPoint& p1, QtbPoint& p2 ) const;
  void enlarge( double dx, double dy, Qtb::SamplingOption xScale, Qtb::SamplingOption yScale );
  void enlarge( double ratio, Qtb::SamplingOption xScale, Qtb::SamplingOption yScale );
  void printDebug() const;
protected:
  virtual void xml_writeProperties( XML_WRITEPROPERTIES_ARGS ) const;
  virtual bool xml_setProperty( XML_SETPROPERTY_ARGS );
  virtual QtbXMLMember xml_member( XML_MEMBER_ARGS );
protected:
  double _x1, _y1, _x2, _y2;
};

inline bool QtbRect::includes (const QtbPoint2D& p1, const QtbPoint2D& p2) const
{
  QTB_TRACE;
  return includes(p1.x(), p1.y(), p2.x(), p2.y());
}

#endif
