/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2004-03-04
**
***************************************************************************/

#include <math.h>

#include "qtbsampling.h"
#include "qtbtrace.h"

/*!
  \class QtbSampling qtbsampling.h
  \brief An axis sampling definition

  Full description of class still missing
*/

QtbSampling::QtbSampling()
{
  QTB_TRACE;
  _min=0.0;
  _max=0.0;
  _step = 0.0;
  _type=Linear;
  _n=0;
}

void QtbSampling::operator=( const QtbSampling& o )
{
  QTB_TRACE;
  _min = o._min;
  _max = o._max;
  _step = o._step;
  _n = o._n;
  _type = o._type;
}

void QtbSampling::setStep()
{
  QTB_TRACE;
  if (_n<=1) {
    _step=0.0;
    return;
  }
  if ( _max<_min) {
    double tmp = _min;
    _min = _max;
    _max = tmp;
  }
  if(_type==Linear)
    _step = ( _max - _min ) / (double)(_n-1);
  else if ( _min > 0.0 )
    _step = _max / _min;
}

void QtbSampling::inverse()
{
  QTB_TRACE;
  double tmp = _min;
  _min = 1.0/_max;
  _max = 1.0/tmp;
  setStep();
}

double QtbSampling::value( int i ) const
{
  QTB_TRACE;
  if(_type==Linear)
    return _min + _step * i;
  else {
    if (_n==1) return _min;
    else return _min*pow(_step, (double)i / (double)(_n-1));
  }
}
