/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2004-05-28
**
***************************************************************************/

#include <math.h>

#include "qtbsegment2d.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbSegment2D qtbsegment2d.h
  \brief A 2D segment made of two points

  Full description of class still missing
*/

void QtbSegment2D::set( const QtbPoint2D& p1, const QtbPoint2D& p2)
{
  QTB_TRACE;
  _p1=p1;
  _p2=p2;
  _limits.setLimits(_p1.x(), _p1.y(), _p2.x(), _p2.y());
}

void QtbSegment2D::set(double x1, double y1, double x2, double y2)
{
  QTB_TRACE;
  _p1.setX(x1);
  _p1.setY(y1);
  _p2.setX(x2);
  _p2.setY(y2);
  _limits.setLimits(_p1.x(), _p1.y(), _p2.x(), _p2.y());
}

/*!
  Return true is this segment intersect segment \a seg. The intersection point is returned in \a p.
*/
bool QtbSegment2D::intersect(const QtbSegment2D& seg, QtbPoint2D& p) const
{
  QTB_TRACE;
  double dx1=_p2.x()-_p1.x();
  double dy1=_p2.y()-_p1.y();
  double dx2=seg._p2.x()-seg._p1.x();
  double dy2=seg._p2.y()-seg._p1.y();
  double fac=dx2*dy1-dx1*dy2;
  if (fabs(fac)<1e-20) return false; // parallel segments
  double tmp = dx1*dx2*(seg._p1.y()-_p1.y())+dx2*dy1*_p1.x()-dx1*dy2*seg._p1.x();
  p.setX(tmp/fac);
  if (fabs(dx1)>fabs(dx2))
    p.setY(_p1.y()+dy1/dx1*(p.x()-_p1.x()));
  else
    p.setY(seg._p1.y()+dy2/dx2*(p.x()-seg._p1.x()));
  if (_limits.includes(p) && seg._limits.includes(p)) return true;
  else return false;
}

bool QtbSegment2D::includes(QtbPoint2D& p) const
{
  QTB_TRACE;
  if (!_limits.includes(p)) return false;
  double dx1=_p2.x()-_p1.x();
  double dy1=_p2.y()-_p1.y();
  if (fabs(p.y()-_p1.y()+dy1/dx1*(p.x()-_p1.x()))<1e-10) return true; else return false;  
}

double QtbSegment2D::distanceTo(QtbPoint2D& p) const
{
  QTB_TRACE;
  double dx1=_p2.x()-_p1.x();
  double dy1=_p2.y()-_p1.y();
  double dx2=dy1;
  double dy2=-dx1;
  double fac=dx2*dy1-dx1*dy2;
  QtbPoint2D pi;
  pi.setX(dx1*dx2*(p.y()-_p1.y())+dx2*dy1*_p1.x()-dx1*dy2*p.x());
  pi.setX( pi.x()/fac );
  if (fabs(dx1)>fabs(dx2))
    pi.setY(_p1.y()+dy1/dx1*(pi.x()-_p1.x()));
  else
    pi.setY(p.y()+dy2/dx2*(pi.x()-p.x()));
  if (_limits.includes(pi)) {
    return p.distanceTo(pi);
  }
  else {
    double d1=p.distanceTo(_p1);
    double d2=p.distanceTo(_p2);
    if (d1<d2) return d1; else return d2;
  }
}
