/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-09-25
**
***************************************************************************/

#ifndef QTB_STATPOINT_H
#define QTB_STATPOINT_H

#include <math.h>

#include "qtbstatvalue.h"
#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbStatPoint : public QtbStatValue
{
public:
  QtbStatPoint() {}
  QtbStatPoint(double x, double y)
    : QtbStatValue(y), _x(x) {}

  virtual const QString& xml_tagName() const { return xmlStatPointTag;}
  static const QString xmlStatPointTag;

  inline void operator=( const QtbStatPoint& o );
  inline void operator=( const QtbStatValue& o );

  bool operator<(const QtbStatPoint& p) const {return _x < p._x || ( _x == p._x && mean()<p.mean()); }
  bool operator==(const QtbStatPoint& p) const {return _x == p._x && QtbStatValue::operator==(p); }

  void setX( double v ) { _x = v;}
  double x() const {return _x;}

  void setY( double v ) { setMean(v);}
  double y() const {return mean();}

  inline void interpole( double valX, const QtbStatPoint& p1, const QtbStatPoint& p2 );

  // I/O functions
  bool fromString(QtbStringSection str);
  QString toString(int precision=6) const;
protected:
  void xml_writeProperties(XML_WRITEPROPERTIES_ARGS) const;
  QtbXMLMember xml_member(XML_MEMBER_ARGS);
  bool xml_setProperty(XML_SETPROPERTY_ARGS);
private:
  double _x;
};

inline void QtbStatPoint::operator=( const QtbStatPoint& o )
{
  QTB_TRACE;
  QtbStatValue::operator=( o );
  _x = o._x;
}

inline void QtbStatPoint::operator=( const QtbStatValue& o )
{
  QTB_TRACE;
  QtbStatValue::operator=( o );
}

inline void QtbStatPoint::interpole( double valX, const QtbStatPoint& p1, const QtbStatPoint& p2 )
{
  QTB_TRACE;
  double factor = ( valX - p1.x() ) / ( p2.x()-p1.x() );
  setX( valX );
  setMean( p1.mean()+ ( p2.mean()-p1.mean() ) * factor );
  setStddev( p1.stddev()+ ( p2.stddev()-p1.stddev() ) * factor );
  setWeight( p1.weight()+ ( p2.weight()-p1.weight() ) * factor );
}

#endif // QTB_STATPOINT_H
