/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-12-26
**
***************************************************************************/

#ifndef QTB_THREAD_H
#define QTB_THREAD_H

#include <QThread>
#include <QMutex>

#include "qtbtrace.h"
#include "qtbtracebug.h"
#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbThread : public QThread
{
  Q_OBJECT
public:
  QtbThread( QObject * parent = 0 );
  ~QtbThread();

  static QStringList threadNames();
  static void lockThreads() { _threadsMutex.lock(); }
  static void unlockThreads() { _threadsMutex.unlock(); }
  
  static QtbThread * currentThread() { return static_cast<QtbThread *>(QThread::currentThread()); }
  void start();
  static void sleep( unsigned long secs );
  static void msleep( unsigned long msecs );
  static void usleep( unsigned long usecs );
  void setSleeping( bool s ) { _sleeping.fetchAndStoreOrdered(s); }
  bool sleeping() { return _sleeping.testAndSetOrdered(true, true); }
  static int idealThreadCount();
  
  static QList< QStack<QtbTraceBug *> * > bugStacks();
  QStack<QtbTraceBug  *> * bugStack() { return &_bugStack; }
#ifdef QTB_TRACE_ENABLED
  static QList< QStack<const QtbTraceStamp *> * > stacks();
  QStack<const QtbTraceStamp *> * stack() { return &_stack; }
private:
  QStack<const QtbTraceStamp *> _stack;
#endif
  QStack<QtbTraceBug *> _bugStack;
private:
  static QMutex _threadsMutex;
  static QList<QtbThread *> _threads;
  static QSet<QtbThread *> _runningThreads;
  QAtomicInt _sleeping;
};

#endif // QTB_THREAD_H
