/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2004-10-28
**
***************************************************************************/

#include <QCoreApplication>

#include "qtbthreadtimer.h"

QtbThreadTimer::QtbThreadTimer()
{
  QTB_TRACE;
  _nofRunningThreads=0;
  connect((QTimer*)this,SIGNAL(timeout()),this,SLOT(synchronize()));
  setSingleShot(true);
}

void QtbThreadTimer::threadStarted()
{
  QTB_TRACE;
  if (_nofRunningThreads==0) {
    start(200);
  }
  _nofRunningThreads++;
}

void QtbThreadTimer::threadStopped()
{
  QTB_TRACE;
  _nofRunningThreads--;
  if (_nofRunningThreads<=0) {
    stop();
    QTimer::singleShot(500, this, SIGNAL(synchroTimeout()));
  }
}

void QtbThreadTimer::synchronize()
{
  QTB_TRACE;
  QTime chrono;
  chrono.start();
  emit synchroTimeout();
  QCoreApplication::processEvents();
  int updateDelay=5*chrono.elapsed();
  if (updateDelay<1000) updateDelay=1000;
  start(updateDelay);
}
