/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-12-26
**
***************************************************************************/

#ifndef QTB_TRACE_H
#define QTB_TRACE_H

#include <QStack>

#include "qtbtoolsdllexport.h"
#include "qtbtracestamp.h"

#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
#define QTB_TRACE_ENABLED
#undef QTB_TRACE_DISABLED
#else
#define QTB_TRACE_DISABLED
#undef QTB_TRACE_ENABLED
#endif

#ifdef QTB_TRACE_ENABLED

#define QTB_TRACE \
  static QtbTraceStamp _traceStamp_(__PRETTY_FUNCTION__, __LINE__); \
  QtbTrace _trace_0( &_traceStamp_ );

// Better to use ##__COUNTER__ macro but not working with gcc 4.1.3, maybe gcc 4.3
#define QTB_TRACE_N(index) \
  static QtbTraceStamp _traceStamp_##index(__PRETTY_FUNCTION__, __LINE__); \
  QtbTrace _trace_##index( &_traceStamp_##index );

class QTBTOOLS_EXPORT QtbTrace
{
public:
  QtbTrace( const QtbTraceStamp * stamp );
  ~QtbTrace();

  static QStack<const QtbTraceStamp *> * mainStack() { return &_mainStack; }
  static QString backTrace(const QStack<const QtbTraceStamp *> * stack);
private:
  QStack<const QtbTraceStamp *> * _myStack;
  static QStack<const QtbTraceStamp *> _mainStack;
};

#else
#define QTB_TRACE
#endif

#endif // QTB_TRACE_H
