/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-10-20
**
***************************************************************************/

#ifndef QTB_TRACEBUG_H
#define QTB_TRACEBUG_H

#include <QStack>

#include "qtbtoolsdllexport.h"
#include "qtbtraceinfo.h"

#define QTB_TRACE_BUG \
  static QtbTraceStamp _traceBugStamp_(__PRETTY_FUNCTION__, __LINE__); \
  QtbTraceBug _traceBug_0( &_traceBugStamp_ ); \
  QtbTraceBug * _traceBug_ = &_traceBug_0; \
  Q_UNUSED(_traceBug_);

// Better to use ##__COUNTER__ macro but not working with gcc 4.1.3, maybe gcc 4.3
#define QTB_TRACE_BUG_N(index) \
  static QtbTraceStamp _traceBugStamp_##index(__PRETTY_FUNCTION__, __LINE__); \
  QtbTraceBug _traceBug_##index( &_traceBugStamp_##index ); \
  _traceBug_ = &_traceBug_##index;

#define QTB_TRACE_POINTER(val) \
  _traceBug_->append( # val,val );

#define QTB_TRACE_INT(val) \
  _traceBug_->append( # val,val );

#define QTB_TRACE_BOOL(val) \
  _traceBug_->append( # val,val );

#define QTB_TRACE_DOUBLE(val) \
  _traceBug_->append( # val,val );

#define QTB_TRACE_STRING(val) \
  _traceBug_->append( # val,val );

class QtbTraceStamp;

class QTBTOOLS_EXPORT QtbTraceBug
{
public:
  QtbTraceBug( const QtbTraceStamp * stamp );
  ~QtbTraceBug();

  inline void append( const char * valName, const void * val );
  inline void append( const char * valName, QVariant val );

  static QStack<QtbTraceBug *> * mainStack() { return &_mainStack; }
  static QString backTrace(const QStack<QtbTraceBug *> * stack);
private:
  QString toString() const;

  QStack<QtbTraceBug *> * _myStack;
  static QStack<QtbTraceBug *> _mainStack;

  const QtbTraceStamp * _stamp;
  QList<QtbTraceInfo> _infos;
};

inline void QtbTraceBug::append( const char * valName, const void * val )
{
  _infos.append(QtbTraceInfo( valName, val ) );
}

inline void QtbTraceBug::append( const char * valName, QVariant val )
{
  _infos.append(QtbTraceInfo( valName, val ) );
}

#endif // QTB_TRACEBUG_H
