/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-11-24
**
***************************************************************************/

#ifndef QTB_TREEITEM_H
#define QTB_TREEITEM_H

#include "qtbxmlclass.h"
#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbTreeItem : public QtbXMLClass
{
public:
  QtbTreeItem( QtbTreeItem * parent=0 );
  ~QtbTreeItem();

  virtual const QString& xml_tagName() const { return xmlTreeItemTag; }
  static const QString xmlTreeItemTag;

  QtbTreeItem * parent() const { return _parent; }
  void setParent( QtbTreeItem * parent );

  bool isFolder() const { return _folder; }
  void setFolder() { _folder=true; }

  bool isModified() const;
  void setModified( bool m );

  void setRemoved( bool r );
  bool isRemoved() const { return _removed; }
  bool isParentRemoved() const;
  void removeChild( int index );
  void removeChildren();
  QList<QtbTreeItem *> takeChildren();

  int childCount() const { return _children.count(); }
  int folderCount() const;

  int indexOfChild( QtbTreeItem * item ) const { return _children.indexOf(item); }
  int indexOfFolder( QtbTreeItem * g ) const;

  QtbTreeItem * childAt( int index ) const { return _children.at(index); }
  QtbTreeItem * folderAt( int index );

  int childRow();
  int folderRow();

  typedef QList<QtbTreeItem *>::iterator iterator;
  typedef QList<QtbTreeItem *>::const_iterator const_iterator;
  iterator begin() { return _children.begin(); }
  iterator end() { return _children.end(); }
  const_iterator begin() const { return _children.begin(); }
  const_iterator end() const { return _children.end(); }
protected:
  virtual QtbTreeItem * createNewItem( QtbTreeItem * parent )=0 ;

  virtual void xml_writeProperties( XML_WRITEPROPERTIES_ARGS ) const;
  virtual void xml_writeChildren(XML_WRITECHILDREN_ARGS) const;
  virtual bool xml_setProperty( XML_SETPROPERTY_ARGS );
  virtual QtbXMLMember xml_member( XML_MEMBER_ARGS );
  virtual void xml_polish(XML_POLISH_ARGS);
private:
  void addChild( QtbTreeItem * item ) { _children.append(item); _modified=true; }

  QtbTreeItem * _parent;
  QList<QtbTreeItem *> _children;
  uint _modified:1;
  uint _folder:1;
  uint _removed:1;
  uint _unused:29;
};

#endif // QTB_TREEITEM_H
