/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-04-25
**
***************************************************************************/

#include "qtbxmlbytearraystream.h"
#include "qtbcoreapplication.h"
#include "qtbglobal.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbXMLByteArrayStream qtbxmlbytearraystream.h
  \brief Brief description of class still missing

  Store XMLClass information in a QByteArray with format:

  QString        Tag "QtbXMLClass"
  qint32         Version (currently 0)
  File1
    QString      File name
    QByteArray   File content
  File2
    QString      File name
    QByteArray   File content
  ...
  FileN
    QString      File name
    QByteArray   File content
*/

QtbXMLByteArrayStream::QtbXMLByteArrayStream( QIODevice::OpenMode m )
 : QtbXMLStream( _xml = new QByteArray, m)
{
  QTB_TRACE;
  _mode = m;
  _lastPos = 0;
  if (_mode == QIODevice::WriteOnly ) {
    QDataStream s( &_data, QIODevice::WriteOnly );
    QString tag("QtbXMLClass");
    s << tag;
    qint32 version = 0;
    s << version;
  }
}

QtbXMLByteArrayStream::~QtbXMLByteArrayStream()
{
  QTB_TRACE;
  delete _xml;
}

bool QtbXMLByteArrayStream::open(QByteArray data)
{
  QTB_TRACE;
  QTB_ASSERT (_mode == QIODevice::ReadOnly );
  _data = data;
  QDataStream s(_data);
  QString tag;
  s >> tag;
  if (tag!="QtbXMLClass") {
    qtbApp << tr("Wrong tag, not a QtbXMLClass byte array") << endl;
    return false;
  }
  qint32 version;
  s >> version;
  if (version>0) {
    qtbApp << tr("Wrong version(%1), current version is %2").arg(version).arg(0) << endl;
    return false;
  }
  _lastPos = s.device()->pos();
  return true;
}

void QtbXMLByteArrayStream::addFile( QString fileName, const QByteArray& data )
{
  QTB_TRACE;
  QDataStream s( &_data, QIODevice::Append );
  s << fileName;
  _files.insert( fileName, data );
  s << data;
}

bool QtbXMLByteArrayStream::file( QString fileName, QByteArray& data )
{
  QTB_TRACE;
  QMap<QString, QByteArray>::iterator it = _files.find(fileName);
  if (it == _files.end() ) {
    QDataStream s( &_data, QIODevice::ReadOnly );
    s.device()->seek(_lastPos);
    while (_lastPos<_data.size()) {
      QString readFileName;
      s >> readFileName;
      s >> data;
      _files.insert(readFileName, data);
      _lastPos = s.device()->pos();
      if (readFileName==fileName) return true;
    }
    return false;
  } else {
    data = it.value();
    return true;
  }
}
