/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2004-11-27
**
***************************************************************************/

#ifndef QTB_XMLCLASSFACTORY_H
#define QTB_XMLCLASSFACTORY_H

#include <QString>
#include <QMap>

#include "qtbtoolsdllexport.h"

class QtbXMLClass;
class QtbXMLClassCreator;
class QtbXMLClassWorkshop;

class QTBTOOLS_EXPORT QtbXMLClassFactory
{
public:
  ~QtbXMLClassFactory();

  static QtbXMLClassFactory * instance() { return & _self; }

  QtbXMLClass * create( QString workshop, QString tag ) const;
  QtbXMLClassCreator * creator( QString workshop, QString tag ) const;
  QStringList registeredTags( QString workshop ) const;

  void registerClass( QString workshop, QString tag, QtbXMLClassCreator * creator );
  void registerSynonym( QString workshop, QString syn, QString tag );
protected:
  static QtbXMLClassFactory _self;
private:
  QtbXMLClassWorkshop * workshop( QString workshop );
  QtbXMLClassWorkshop * workshop( QString workshop ) const;

  QMap<QString, QtbXMLClassWorkshop *> _workshops;
};

class QTBTOOLS_EXPORT QtbXMLClassWorkshop
{
public:
  QtbXMLClassCreator * creator( QString tag ) const;
  QStringList registeredTags() const;
  void registerClass( QString tag, QtbXMLClassCreator * creator );
  void registerSynonym( QString syn, QString tag );
private:
  QMap<QString, QtbXMLClassCreator *> _registeredClasses;
  QMap<QString, QString> _registeredSynonyms;
};

class QTBTOOLS_EXPORT QtbXMLClassCreator
{
public:
  QtbXMLClassCreator( QString workshop, QString tag ) {
    QtbXMLClassFactory::instance()->registerClass( workshop, tag, this );
  }
  virtual ~QtbXMLClassCreator() {}
  virtual QString tagName() const = 0;
private:
  friend class QtbXMLClassFactory;
  virtual QtbXMLClass * create() const = 0;
};

#define QTB_XMLREGISTER_HEAD( _workshop_, _creator_, _className_, _tagName_ ) \
  class _className_##XMLClassCreator : public _creator_ \
  { \
  public: \
    _className_##XMLClassCreator() \
      : _creator_( _workshop_, _tagName_ ) {} \
    virtual QString tagName() const { return _tagName_; }

#define QTB_XMLREGISTER_TAIL( _className_ ) \
  private: \
    virtual QtbXMLClass * create() const { return new _className_; } \
  }; \
  _className_##XMLClassCreator register##_className_;

#define QTB_XMLSYNONYM( _workshop_, _synName_, _tagName_ ) \
  class _synName_##XMLClassSynonym \
  { \
  public: \
    _synName_##XMLClassSynonym() { \
      QtbXMLClassFactory::instance()->registerSynonym( _workshop_, # _synName_, _tagName_ ); \
    } \
  }; \
  _synName_##XMLClassSynonym register##_synName_;


#endif // QTB_XMLCLASSFACTORY_H
