/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-11-25
**
***************************************************************************/

#include "qtbxmlgenericitem.h"

/*!
  \class QtbXMLGenericItem qtbxmlgenericitem.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

QtbXMLGenericItem::QtbXMLGenericItem( QtbXMLGenericItem * parent )
  : QtbTreeItem(parent)
{
  QTB_TRACE;
  setFolder();
}

QtbXMLGenericItem::QtbXMLGenericItem( QString tag, QtbXMLGenericItem * parent )
  : QtbTreeItem(parent)
{
  QTB_TRACE;
  setFolder();
  _tag = tag;
}

/*!
  Return item with \a tag. Only direct children are searched. \a tag can be a path to a sub-child
*/
QList<QtbXMLGenericItem *> QtbXMLGenericItem::children( QString tag ) const
{
  QList<QtbXMLGenericItem *> cList;
  int pathSeparator = tag.indexOf("/");
  if (pathSeparator==-1) {
    for (const_iterator it = begin();it!=end();it++) {
      QtbXMLGenericItem * item = static_cast<QtbXMLGenericItem *>(*it);
      if ( item->_tag==tag ) {
        cList.append(item);
      }
    }
  } else {
    QString tag0 = tag.left(pathSeparator);
    for (const_iterator it = begin();it!=end();it++) {
      QtbXMLGenericItem * item = static_cast<QtbXMLGenericItem *>(*it);
      if ( item->_tag==tag0 ) {
        cList += children( tag.mid(pathSeparator+1) );
      }
    }
  }
  return cList;
}

/*!
  Get the list of attributes
*/
bool QtbXMLGenericItem::xml_setAttributes( XML_SETATTRIBUTES_ARGS )
{
  QTB_TRACE;
  Q_UNUSED(context);
  _attributes.clear();
  for (QtbXMLRestoreAttributeIterator it = attributes.begin();it!=attributes.end();it++) {
    _attributes.add( it.key().toString(), it.value().toString() );
  }
  return true;
}

/*!
  Accept all members
*/
QtbXMLMember QtbXMLGenericItem::xml_member( XML_MEMBER_ARGS )
{
  QTB_TRACE;
  QtbXMLGenericItem * o = new QtbXMLGenericItem( this );
  o->setTag( tag.toString() );
  // TODO: check double setAttributes
  o->xml_setAttributes(attributes, context);
  // Return a member either child or properties, we don't know at this time
  return QtbXMLMember(o, false);
}

bool QtbXMLGenericItem::xml_setProperty( XML_SETPROPERTY_ARGS )
{
  QTB_TRACE;
  Q_UNUSED(attributes);
  Q_UNUSED(memberID);
  Q_UNUSED(context);
  _value = content.toString();
  return true;
}

bool QtbXMLGenericItem::xml_setBinaryData( XML_SETBINARYDATA_ARGS )
{
  QTB_TRACE;
  Q_UNUSED(s);
  Q_UNUSED(context);
  return true;
}

void QtbXMLGenericItem::xml_writeProperties( XML_WRITEPROPERTIES_ARGS ) const
{
  QTB_TRACE;
  Q_UNUSED(context);
  for (const_iterator it = begin();it!=end();it++) {
    QtbXMLGenericItem * item = static_cast<QtbXMLGenericItem *>(*it);
    if ( item->childCount()==0 ) { // A children with no children is a property
      writeProperty(s, item->xml_tagName(), item->_attributes, item->value().toString() );
    }
  }
}

void QtbXMLGenericItem::xml_writeChildren( XML_WRITECHILDREN_ARGS ) const
{
  QTB_TRACE;
  Q_UNUSED(context);
  for (const_iterator it = begin();it!=end();it++) {
    QtbXMLGenericItem * item = static_cast<QtbXMLGenericItem *>(*it);
    if ( item->childCount()>0 ) { // A children with children is a "real" children in the QtbXMLClass sense
      item->xml_save( s, context, item->_attributes );
    }
  }
}

void QtbXMLGenericItem::find( QList<QtbXMLGenericItem *>& itemList, const QtbXMLGenericItem * queryItem ) const
{
  QTB_TRACE;
  for (const_iterator it = begin();it!=end();it++) {
    QtbXMLGenericItem * item = static_cast<QtbXMLGenericItem *>(*it);
    if ( queryItem->xml_tagName().isEmpty() || item->xml_tagName() == queryItem->xml_tagName() ) {
      if ( queryItem->_attributes.isEmpty() || item->_attributes.isSimilar( queryItem->_attributes ) ) {
        if (queryItem->childCount()>0) {
          item->find( itemList, queryItem->childAt(0) );
        } else {
          itemList.append(item);
        }
      }
    } else if (!queryItem->parent()) {
      item->find(itemList, queryItem);
    }
  }
}
