/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-11-25
**
***************************************************************************/

#ifndef QTB_XMLGENERICITEM_H
#define QTB_XMLGENERICITEM_H

#include <QVariant>

#include "qtbtreeitem.h"
#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbXMLGenericItem : public QtbTreeItem
{
public:
  QtbXMLGenericItem( QtbXMLGenericItem * parent = 0 );
  QtbXMLGenericItem( QString tag, QtbXMLGenericItem * parent = 0 );

  inline QtbXMLGenericItem * childAt( int index ) const;
  inline QtbXMLGenericItem * parent() const;

  QVariant value() const { return _value; }
  void setValue(QVariant v) { _value = v; }

  void setTag( QString tag ) { _tag = tag; }
  virtual const QString& xml_tagName() const { return _tag;}

  virtual void xml_attributes( XML_ATTRIBUTES_ARGS ) const { Q_UNUSED(context); attributes = _attributes; }
  virtual bool xml_setAttributes( XML_SETATTRIBUTES_ARGS );
  void addAttribute( const QString& key, const QString& value = QString::null ) { _attributes.add(key, value); }

  QList<QtbXMLGenericItem *> children( QString tag ) const;
  void find( QList<QtbXMLGenericItem *>& itemList, const QtbXMLGenericItem * queryItem ) const;
protected:
  virtual QtbTreeItem * createNewItem( QtbTreeItem * ) { return 0; }
  virtual QtbXMLMember xml_member( XML_MEMBER_ARGS );
  virtual bool xml_setProperty( XML_SETPROPERTY_ARGS );
  virtual void xml_writeProperties( XML_WRITEPROPERTIES_ARGS ) const;
  virtual void xml_writeChildren( XML_WRITECHILDREN_ARGS ) const;
  virtual bool xml_setBinaryData( XML_SETBINARYDATA_ARGS );
private:
  QString _tag;
  QtbXMLSaveAttributes _attributes;
  QVariant _value;
};

inline QtbXMLGenericItem * QtbXMLGenericItem::childAt( int index ) const
{
  QTB_TRACE;
  return static_cast<QtbXMLGenericItem *>(QtbTreeItem::childAt(index));
}

inline QtbXMLGenericItem * QtbXMLGenericItem::parent() const
{
  QTB_TRACE;
  return static_cast<QtbXMLGenericItem *>(QtbTreeItem::parent());
}

#endif // QTB_XMLGENERICITEM_H
