/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-06-08
**
***************************************************************************/

#include "qtbxmlheader.h"
#include "qtbxmltarstream.h"
#include "qtbxmlstringstream.h"
#include "qtbxmlbytearraystream.h"
#include "qtbversion.h"
#include "qtbtoolsversion.h"
#include "qtbglobal.h"
#include "qtbcoreapplication.h"
#include "qtbtrace.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbXMLHeader qtbxmlheader.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

const QString QtbXMLHeader::xmlHeaderTag = "QtbXMLClass";

/*!
  Description of constructor still missing
*/
QtbXMLHeader::QtbXMLHeader(QtbXMLClass * object)
    : QtbXMLClass()
{
  _object=object;
  _polished=false;
}

/*!
  Create a compressed tar file for storing the xml part (contents.xml) and,
  if any, the binary parts ( random names stored in xml part).
*/
QtbXMLClass::Error QtbXMLHeader::xml_saveFile(QString fileName, QtbXMLContext * context, FileType ft) const
{
  QtbXMLSaveAttributes attributes;
  xml_attributes(attributes, context);
  if ( ft == TarFile ) {
    QtbXMLTarStream s ( QIODevice::WriteOnly );
    if ( s.open( fileName ) ) {
      xml_saveDocType(s);
      xml_save(s, context, attributes);
      if ( !s.addXmlFile() ) {
        qtbApp << tr("Cannot write to file %1.").arg(fileName) << endl;
        return ErrorWritingFile;
      }
      return NoError;
    } else {
      qtbApp << tr("Cannot open file %1 for writing.").arg(fileName) << endl;
      return ErrorFileNotOpen;
    }
  } else {
    QFile f(fileName);
    if ( f.open(QIODevice::WriteOnly) ) {
      QtbXMLStringStream s ( QIODevice::WriteOnly );
      xml_saveDocType(s);
      xml_save(s, context, attributes);
      QTextStream sf(&f);
      sf.setCodec("ISO-8859-1");
      //sf.setCodec("utf-16");
      sf.setGenerateByteOrderMark ( true );
      sf << s.toString();
      return NoError;
    } else {
      qtbApp << tr("Cannot open file %1 for writing.").arg(fileName) << endl;
      return ErrorFileNotOpen;
    }
  }
}

/*!
  Create a QTextStream attached to the given QString and export XML data.
  No binary data can be saved this way.
*/
QString QtbXMLHeader::xml_saveString(bool header, QtbXMLContext * context) const
{
  QTB_TRACE;
  QtbXMLStringStream s( QIODevice::WriteOnly );
  QtbXMLSaveAttributes attributes;
  xml_attributes(attributes, context);
  if (header) {
    xml_saveDocType(s);
    xml_save(s, context, attributes);
  } else {
    xml_save(s, context, attributes);
  }
  return s.toString();
}

/*!
  Create a QByteArray which contains the XML part and the attached files
*/
QByteArray QtbXMLHeader::xml_saveByteArray(QtbXMLContext * context) const
{
  QTB_TRACE;
  QtbXMLByteArrayStream s( QIODevice::WriteOnly );
  xml_saveDocType(s);
  QtbXMLSaveAttributes attributes;
  xml_attributes(attributes, context);
  xml_save(s, context, attributes);
  s.addXmlFile();
  return s.data();
}

/*!
  Open a file
*/
QtbXMLClass::Error QtbXMLHeader::xml_restoreFile( QString fileName, QtbXMLContext * context, FileType ft )
{
  QtbXMLTarStream s(QIODevice::ReadOnly);
  if ( s.open( fileName ) ) {
    QByteArray xmlArray;
    if ( (ft == TarFile) && s.file( "contents.xml", xmlArray ) ) {
      QTextStream xmlStream( xmlArray, QIODevice::ReadOnly );
      xmlStream.setCodec("ISO-8859-1");
      //xmlStream.setCodec("UTF-8");
      xmlStream.setAutoDetectUnicode ( true );
      QString buf = xmlStream.readAll();
      int line = 1;
      const QChar * ptr = buf.data();
      if ( checkDocType( ptr, line, false ) )
        return restore(ptr, line, &s, context);
      else
        return ErrorNoDocType;
    } else {  // check for compatibility with format before November 2006
      if ( ft == TarFile ) qtbApp << tr("No contents.xml found in archive, trying old versions...") << endl;
      QFile f ( fileName );
      if (f.open( QIODevice::ReadOnly ) ) {
        QTextStream xmlStream( &f );
        xmlStream.setCodec("ISO-8859-1");
        //xmlStream.setCodec("UTF-8");
        xmlStream.setAutoDetectUnicode ( true );
        QString buf = xmlStream.readAll();
        int line = 1;
        const QChar * ptr = buf.data();
        if ( checkDocType( ptr, line, false ) )
          return restore(ptr, line, &s, context);
        else
          return ErrorNoDocType;
      }
      return ErrorFileNotOpen;
    }
  }
  return ErrorFileNotOpen;
}

QtbXMLClass::Error QtbXMLHeader::xml_restoreString(QString buf, QtbXMLContext * context)
{
  int line = 1;
  const QChar * ptr = buf.data();
  QtbXMLStringStream s(QIODevice::ReadOnly);
  if ( checkDocType(ptr, line, false) )
    return restore(ptr, line, &s, context);
  else
    return ErrorNoDocType;
}

QtbXMLClass::Error QtbXMLHeader::xml_restoreByteArray(QByteArray data, QtbXMLContext * context)
{
  QByteArray xmlArray;
  QtbXMLByteArrayStream s(QIODevice::ReadOnly);
  if ( !s.open(data) || !s.file( "contents.xml", xmlArray ) ) {
    return ErrorFileNotOpen;
  }
  QTextStream xmlStream( xmlArray, QIODevice::ReadOnly );
  xmlStream.setCodec("ISO-8859-1");
  //xmlStream.setCodec("UTF-8");
  xmlStream.setAutoDetectUnicode ( true );
  QString buf = xmlStream.readAll();
  int line = 1;
  const QChar * ptr = buf.data();
  if (checkDocType(ptr, line, false))
    return restore(ptr, line, &s, context);
  else
    return ErrorNoDocType;
}
/*!
  Print the DOCTYPE line in header.
*/
void QtbXMLHeader::xml_saveDocType( QtbXMLStream& s )
{
  s << "<!DOCTYPE QtbXMLClass>" << endl;
}

bool QtbXMLHeader::checkDocType( const QChar *& ptr, int& line, bool strict )
{
  QTB_TRACE;
  QtbXMLClass::skipBlanks( ptr, line );
  /* old versions of QtbXMLClass did not include the DOCTYPE
     if a DOCTYPE is found, then the type must be QtbXMLClass */
  QtbStringSection tmp( ptr, 9 );
  if ( tmp != "<!DOCTYPE" ) {
    // probably the old type of QtbXMLClass files, try directly
    return !strict;
  }
  ptr += 9;
  QtbXMLClass::skipBlanks( ptr, line );
  tmp.set( ptr, 11 );
  if ( tmp != "QtbXMLClass" ) {
    qtbApp << translate( "QtbXMLHeader", "Wrong DOCTYPE at line " ) << line << endl;
    return false;
  }
  ptr += 11;
  QtbXMLClass::skipBlanks( ptr, line );
  if ( *ptr != 0x003E ) { // '>'
    qtbApp << translate( "QtbXMLHeader", "Missing '>' at line " ) << line << endl;
    return false;
  }
  ptr++;
  return true;
}

void QtbXMLHeader::xml_writeChildren( XML_WRITECHILDREN_ARGS ) const
{
  QTB_ASSERT(_object);
  QtbXMLSaveAttributes att;
  _object->xml_attributes(att, context);
  _object->xml_save(s, context, att);
}

QtbXMLMember QtbXMLHeader::xml_member( XML_MEMBER_ARGS )
{
  QTB_ASSERT(_object);
  Q_UNUSED(attributes);
  Q_UNUSED(context);
  if ( tag == "QtbXMLClass" ) return QtbXMLMember(this);
  else if ( tag == _object->xml_tagName() ) return QtbXMLMember(_object);
  else return QtbXMLMember(QtbXMLMember::Unknown);
}

void QtbXMLHeader::xml_attributes( XML_ATTRIBUTES_ARGS ) const
{
  Q_UNUSED(context);
  attributes.add( "version", QTBTOOLS_VERSION );
}

bool QtbXMLHeader::xml_setAttributes( XML_SETATTRIBUTES_ARGS )
{
  Q_UNUSED(context);
  for (QtbXMLRestoreAttributeIterator it = attributes.begin(); it!= attributes.end(); it++ ) {
    const QtbStringSection& att = it.key();
    if(att=="version") {
      QtbVersion v( it.value().toString() );
      static const QtbVersion refV( QTBTOOLS_VERSION );
      if ( v>refV ) {
        qtbApp << tr( "This XML has been generated with a newer release (%1).\n"
                      "  Your current release is %2. Please upgrade.\n"
                      "  ### WARNING ### : parsing XML at your own risks.\n" )
        .arg( it.value().toString() )
        .arg( QTBTOOLS_VERSION ) << endl;
      }
    } else return false;
  }
  return true;
}

void QtbXMLHeader::xml_polishChild(XML_POLISHCHILD_ARGS)
{
  Q_UNUSED(context);
  if(child==_object) _polished=true;
}
