/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-06-08
**
***************************************************************************/

#ifndef QTB_XMLHEADER_H
#define QTB_XMLHEADER_H

#include "qtbxmlclass.h"
#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbXMLHeader : public QtbXMLClass
{
  QTB_TRANSLATIONS("QtbXMLHeader")
public:
  QtbXMLHeader(QtbXMLClass * object);

  virtual const QString& xml_tagName() const {return xmlHeaderTag;}
  static const QString xmlHeaderTag;

  Error xml_saveFile(QString fileName, QtbXMLContext * context=0, FileType ft=TarFile) const;
  QString xml_saveString(bool header=true, QtbXMLContext * context=0) const;
  QByteArray xml_saveByteArray(QtbXMLContext * context=0) const;
  Error xml_restoreFile(QString fileName, QtbXMLContext * context=0, FileType ft=TarFile);
  Error xml_restoreString(QString str, QtbXMLContext * context=0);
  Error xml_restoreByteArray(QByteArray data, QtbXMLContext * context=0);
  static void xml_saveDocType(QtbXMLStream& s);
  inline static bool checkDocType(QString buf);
  static bool checkDocType(const QChar *& ptr, int& line, bool strict);
protected:
  virtual void xml_writeChildren(XML_WRITECHILDREN_ARGS) const;
  virtual void xml_attributes( XML_ATTRIBUTES_ARGS ) const;
  virtual bool xml_setAttributes(XML_SETATTRIBUTES_ARGS);
  virtual QtbXMLMember xml_member(XML_MEMBER_ARGS);
  virtual void xml_polishChild(XML_POLISHCHILD_ARGS);
  virtual bool xml_polished(XML_POLISHED_ARGS) {return _polished;}

  QtbXMLClass * _object;
  bool _polished;
};

inline bool QtbXMLHeader::checkDocType(QString buf)
{
  int line;
  const QChar * ptr = buf.data();
  return checkDocType( ptr, line, true );
}

#endif // QTB_XMLHEADER_H
