/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-07-12
**
***************************************************************************/

#ifndef QTB_XMLMAP_H
#define QTB_XMLMAP_H

#include <QMap>

#include "qtbxmlclass.h"
#include "qtbtoolsdllexport.h"

template <class Key, class T>
class QTBTOOLS_EXPORT QtbXMLMap : public QtbXMLClass
{
public:
  QtbXMLMap( QString tag, QMap<Key,T> * map );
  QtbXMLMap( QString tag, const QMap<Key,T> * map );

  virtual const QString& xml_tagName() const {return _tag;}
protected:
  virtual void xml_writeProperties(XML_WRITEPROPERTIES_ARGS) const;
  virtual bool xml_setProperty(XML_SETPROPERTY_ARGS);
  virtual QtbXMLMember xml_member(XML_MEMBER_ARGS);
private:
  QString _tag;
  QMap<Key,T> * _map;
};

template <class Key, class T>
QtbXMLMap<Key,T>::QtbXMLMap( QString tag, QMap<Key,T> * map )
{
  _tag = tag;
  _map = map;
}

template <class Key, class T>
QtbXMLMap<Key,T>::QtbXMLMap( QString tag, const QMap<Key,T> * map )
{
  _tag = tag;
  _map = const_cast<QMap<Key,T> *>(map);
}

template <class Key, class T>
void QtbXMLMap<Key,T>::xml_writeProperties(XML_WRITEPROPERTIES_ARGS) const
{
  QTB_TRACE;
  static const QString tag = "value";
  static const QString att = "key=\"%1\"";
  QMapIterator<Key,T> it(*_map);
  while(it.hasNext()) {
    it.next();
    writeProperty( s, tag, att.arg(it.key()), it.value() );
  }
}

template <class Key, class T>
QtbXMLMember QtbXMLMap<Key,T>::xml_member(XML_MEMBER_ARGS)
{
  QTB_TRACE;
  Q_UNUSED(attributes);
  if ( tag == "value" ) return QtbXMLMember(0);
  return QtbXMLMember(QtbXMLMember::Unknown);
}

template <class Key, class T>
bool QtbXMLMap<Key,T>::xml_setProperty(XML_SETPROPERTY_ARGS)
{
  QTB_TRACE;
  static const QString att = "key";
  if (memberID==0) {
    QtbXMLRestoreAttributeIterator it = attributes.find(att);
    if ( it != attributes.end() ) {
      Key tmpKey;
      T tmpT;
      _map->insert( it.value().to(tmpKey), content.to(tmpT) );
      return true;
    }
  }
  return false;
}

#endif // QTB_XMLMAP_H
