/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-06-07
**
***************************************************************************/

#ifndef QTB_XMLMEMBER_H
#define QTB_XMLMEMBER_H

#include "qtbtoolsdllexport.h"

class QtbXMLClass;

class QTBTOOLS_EXPORT QtbXMLMember
{
public:
  inline QtbXMLMember( int memberID );
  inline QtbXMLMember(QtbXMLClass * child, bool temp = false);

  enum SpecialPublicMemberIDs {Unknown=-1, Skip=-2};

  inline void operator=(const QtbXMLMember& o);
  inline QtbXMLMember& operator+( int idOffset );
  inline void reset();

  bool isValidMember() {return _memberID>Unknown;}
  bool isSkipMember() {return _memberID==Skip;}
  int memberID() {return _memberID;}
  QtbXMLClass * child() {return _child;}
  inline void release();
private:
  enum SpecialPrivateMemberIDs {PermanentChild=-3, TemporaryChild=-4};
  QtbXMLClass * _child;
  int _memberID;
};

inline QtbXMLMember::QtbXMLMember(int memberID)
{
  _child = 0;
  _memberID=memberID;
}

inline QtbXMLMember::QtbXMLMember(QtbXMLClass * child, bool temp)
{
  _child = child;
  _memberID=temp ? TemporaryChild : PermanentChild;
}

inline void QtbXMLMember::operator=(const QtbXMLMember& o)
{
  _child = o._child;
  _memberID = o._memberID;
}

inline QtbXMLMember& QtbXMLMember::operator+(int idOffset)
{
  if (_memberID>Unknown) _memberID+=idOffset;
  return *this;
}

inline void QtbXMLMember::reset()
{
  _child=0;
  _memberID=Unknown;
}

#endif // QTB_XMLMEMBER_H
