/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-06-07
**
***************************************************************************/

#include "qtbxmlparser.h"
#include "qtbcoreapplication.h"
#include "qtbtrace.h"
#include "qtbglobal.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbXMLParser qtbxmlparser.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
QtbXMLParser::QtbXMLParser(QtbXMLStream * s, QtbXMLContext * context)
    : _member(QtbXMLMember::Unknown)
{
  _stream=s;
  _context=context;
  _line=1;
}

/*!
  Description of destructor still missing
*/
QtbXMLParser::~QtbXMLParser()
{
  qDeleteAll(_tmpStrings);
}

void QtbXMLParser::finish(QtbXMLClass::Error err)
{
  _error=err;
  if(_error!=QtbXMLClass::NoError) {
    QtbXMLClass::errorMessage(_error, _line);
  }
  // Set pointer just before the end to force exit.
  while(_ptr->unicode()!='\0') _ptr++;
  _ptr--;
}

QtbXMLClass::Error QtbXMLParser::parse(QtbXMLClass * object)
{
  _object=object;
//#define XML_DEBUG
#ifdef XML_DEBUG
  printf("Entering context %s\n", _object->xml_tagName().toAscii().data());
#endif
  _error=QtbXMLClass::NoError;
  // In case restore is called for an XML item with no children, just content
  // For normal situations this should not happen. It is mostly used by QtbGenericItem and QtbXMLDummy
  _content.set(_ptr, 0);
  _specChar=0;
  // Information on current member which content is being scanned
  // Set currentMember as valid for member ID (0) for QtbXMLGenericItem which don't know if the next item
  // is a complete tree structure or just a property.
  // Negative memberID are never accepted by setProperty but let a true returned by QtbXMLMember::isValid()
  QtbXMLMember curMember(QtbXMLMember::Skip);
  /* The buffer may start with other data that must be skipped while not inside the right context.
     The buffer is parsed until it finds a root tag corresponding to tagName */
  while ( true ) {
    switch ( _ptr->unicode() ) {
    case 0x000A:    // '\n'
      _line++;
      break;
    case 0x0026: { // '&'
        if ( !_specChar ) _specChar = _ptr; // keep track of first occurence of & in content
        skipSpecChar();
      }
      break;
    case 0x003C: {    // '<'
        _content.setEnd(_ptr);
        setTag();
        switch(_tagType) {
        case Attributes:
          _attributes.clear();
          qDeleteAll(_tmpStrings);
          _tmpStrings.clear();
          setAttributes();
          switch(_tagType) {
          case OpenTagAttributes:
#ifdef XML_DEBUG
            printf("%5i = %5i === <%s att>\n", __LINE__, _line, _tag.toString().toAscii().data() );
#endif
            openContext();
            break;
          case SingleTagAttributes:
#ifdef XML_DEBUG
            printf("%5i = %5i === <%s att/>\n", __LINE__, _line, _tag.toString().toAscii().data() );
#endif
            smallContext();
            break;
          default:
            break;
          }
          break;
        case OpenTagNoAttributes:
#ifdef XML_DEBUG
          printf("%5i = %5i === <%s>\n", __LINE__, _line, _tag.toString().toAscii().data() );
#endif
          openContext();
          break;
        case CloseTag:
#ifdef XML_DEBUG
          printf("%5i = %5i === %s</%s>\n", __LINE__, _line, _content.toString().toAscii().data(),
                                              _tag.toString().toAscii().data() );
#endif
          closeContext();
          break;
        case SingleTagNoAttributes:
#ifdef XML_DEBUG
          printf("%5i = %5i === <%s/>\n", __LINE__, _line, _tag.toString().toAscii().data() );
#endif
          smallContext();
          break;
        default:
          break;
        }
      }
      break;
    case 0x0000:    // END
      if (_error!=QtbXMLClass::NoError) {
      } else if ( _tag.isEmpty() ) {
        finish(QtbXMLClass::ErrorEndTruncatedTag);
      } else if (_tagStack.size() > 2 ) {
        finish(QtbXMLClass::ErrorEndTruncatedContext);
      } else if(!_object->xml_polished()) {
        finish(QtbXMLClass::ErrorEndTagNotFound);
      }
      return _error;
    default:
      break;
    }
    _ptr++;
  }
}

void QtbXMLParser::openContext()
{
  _member = _object->xml_member(_tag, _attributes, _context);
#ifdef XML_DEBUG
  printf("%5i = %5i === xml_member(%s) for '%s': %i, %p\n", __LINE__, _line, _tag.toString().toAscii().data(),
                                                            _object->xml_tagName().toAscii().data(),
                                                            _member.memberID(), _member.child());
#endif
  if (_member.isValidMember()) {
  } else if (_member.child()) {
    if (!_member.child()->xml_setAttributes(_attributes, _context)) {
      finish(QtbXMLClass::ErrorSettingAttributes);
      return;
    }
    _objectStack.push(_object);
    _object=_member.child();
#ifdef XML_DEBUG
    printf("%5i = %5i =O= Enter object %s\n", __LINE__, _line, _object->xml_tagName().toAscii().data());
#endif
  } else { // Bad member name... skipping or checking for binDataTag
    if ( _tag != QtbXMLClass::binDataTag && _tag != QtbXMLClass::binData200510Tag) {
      // Voluntary skipping member, issue no warning
      if(!_member.isSkipMember()) {
        qtbApp << tr( "Unknown tag at line %1 for context %2: %3" )
          .arg( _line )
          .arg( currentContextTag() )
          .arg( _tag.toString() ) << endl;
      }
      skip();
      return;
    }
  }
  _content.set( _ptr + 1, 0 );
  _specChar = 0;
  _tagStack.push(Tag(_tag,_member));
}

void QtbXMLParser::closeContext()
{
  if ( _tagStack.size() > 0 ) {
    if (_tag == _tagStack.top().tag()) {
      _member=_tagStack.top().member();
      _tagStack.pop();
    } else {
      qtbApp << tr( "Looking for tag '%1', found '%2'" )
        .arg( currentContextTag() )
        .arg( _tag.toString() ) << endl;
      finish(QtbXMLClass::ErrorUnmatchedTag);
      return;
    }
  } else {
    finish(QtbXMLClass::ErrorEmptyContextStack);
    return;
  }
  // At this point we are sure that open/close tags match
  if ( _content.isValid() ) {
    if ( _member.isValidMember() ) {
      if ( _specChar ) {
        restoreSpecChar();
        if (_error!=QtbXMLClass::NoError) return;
      }
      if ( !_object->xml_setProperty(_member.memberID(), _attributes, _content, _context) ) {
        finish(QtbXMLClass::ErrorParsingContent);
        return;
      }
      _member.reset();
      _content.setInvalid();
    } else if ( _tag == QtbXMLClass::binDataTag && _stream->isMultiFile() ) {
      restoreBinaryData();
    } else if ( _tag == QtbXMLClass::binData200510Tag ) {  // Compatibility with version < 1.2.0 (November 2006)
      restoreBinaryData200510();
    } else {
      // We are in a situation where a "parent" item has in fact no member specified in XML stream.
      // Used by QtbXMLGenericItem, in this case before closing the current context we set properties
#ifdef XML_DEBUG
      printf("%5i = %5i =O= Leaving object %s\n", __LINE__, _line, _object->xml_tagName().toAscii().data());
#endif
      if ( _specChar ) {
        restoreSpecChar();
        if (_error!=QtbXMLClass::NoError) return;
      }
      // We ignore return value: for QtbXMLGenericItem, this function returns always true
      // For other objects, e.g. an empty "parent" item: there are good chances that this
      // function will return false (default implementation).
      _object->xml_setProperty(_member.memberID(), _attributes, _content, _context);
      _object->xml_polish(_context);
      QtbXMLClass * parent=_objectStack.pop();
      parent->xml_polishChild(_object, _context);
      _member.release();
      _object=parent;
      _content.setInvalid();
    }
  } else if (_member.child()) {
    // Usual exit of current context
#ifdef XML_DEBUG
    printf("%5i = %5i =O= Leaving object %s\n", __LINE__, _line, _object->xml_tagName().toAscii().data());
#endif
    _object->xml_polish(_context);
    QtbXMLClass * parent=_objectStack.pop();
    parent->xml_polishChild(_object, _context);
    _member.release();
    _object=parent;
    _content.setInvalid();
  } else if ( _tagStack.size() == 0 ) {
#ifdef XML_DEBUG
    printf("%5i === Exiting context %s (Success, context stack empty)\n", __LINE__, _object->xml_tagName().toAscii().data() );
#endif
    finish(QtbXMLClass::NoError);
  }
}

void QtbXMLParser::smallContext()
{
#ifdef XML_DEBUG
  printf("%5i === Member <%s/>\n", __LINE__, _tag.toString().toAscii().data() );
#endif
  // The current context is definitively not a just a property with a valid content
  _content.setInvalid();
  _member = _object->xml_member(_tag, _attributes, _context);
  if ( _member.isValidMember() ) {
    if ( !_object->xml_setProperty(_member.memberID(), _attributes, _content, _context) ) {
      finish(QtbXMLClass::ErrorParsingContent);
    }
  } else if ( _member.child() ) {
    if ( !_member.child()->xml_setAttributes(_attributes, _context)) {
      finish(QtbXMLClass::ErrorSettingAttributes);
      return;
    }
    _object->xml_polishChild(_member.child(), _context);
    _member.release();
  }
}

QString QtbXMLParser::currentContextTag()
{
  return _tagStack.size()>0 ? _tagStack.top().tag().toString() : "[top level]";
}

void QtbXMLParser::restoreBinaryData()
{
  if ( _specChar ) {
    restoreSpecChar();
    if (_error!=QtbXMLClass::NoError) return;
  }
  QByteArray binData;
  if ( _stream->file( _content.toString(), binData ) ) {
    QDataStream binStream( &binData, QIODevice::ReadOnly );
    QString binTag;
    binStream >> binTag;
    if ( _object->xml_tagName() == binTag ) {
      if ( !_object->xml_setBinaryData(binStream, _context) ) {
        finish(QtbXMLClass::ErrorSettingBinaryData);
      }
    } else {
      qtbApp << tr( "Expecting tag %1, and found %2" ).arg(_object->xml_tagName()).arg(binTag) << endl;
      finish(QtbXMLClass::ErrorWrongBinaryTag);
    }
  } else {
    finish(QtbXMLClass::ErrorBinaryFileNotFound);
  }
}

void QtbXMLParser::restoreBinaryData200510()
{
  if ( _specChar ) {
    restoreSpecChar();
    if (_error!=QtbXMLClass::NoError) return;
  }
  // Move to pointer specified by XML tag
  qint64 offset = _content.toLongLong();
  QFile f( _stream->fileName()+".bin" );
  if ( f.open( QIODevice::ReadOnly ) ) {
    QDataStream binStream(&f);
    f.seek( offset );
    /* For historical reasons the tag in binary files is set in ASCII encoding
      First try to read it with Unicode and then try with ascii if failed
    */
    QString binTag;
    binStream >> binTag;
    if ( _object->xml_tagName() == binTag ) {
      if ( !_object->xml_setBinaryData(binStream, _context)) {
        finish(QtbXMLClass::ErrorSettingBinaryData);
      }
    } else { // Compatibility with preceding version of QtbXMLClass (between Nov 2004 and Oct 2005)
      // Reset file pointer
      f.seek( offset );
      // Read again tag
      int n = _object->xml_tagName().size() + 1;
      char * binTag = new char[ n ];
      binStream.readRawData( binTag, sizeof( char ) * n );
      if ( _object->xml_tagName().toAscii() != binTag ) {
        finish(QtbXMLClass::ErrorWrongBinaryOffset);
      } else {
        if ( !_object->xml_setBinaryData200411(binStream, _context)) {
          finish(QtbXMLClass::ErrorSettingBinaryData);
        }
      }
      delete [] binTag;
    }
  } else {
    finish(QtbXMLClass::ErrorBinaryFileNotFound);
  }
}

/*!
  This function is called when special QCharacters sequence are encountered.
  It simply checks if the sequence is terminated by a ';' and does not contain
  any QtbLT. Return true if successful, else return false and send a message to
  the log stream.
*/
void QtbXMLParser::skipSpecChar()
{
  int line0 = _line;
  while ( true ) {
    switch ( _ptr->unicode() ) {
    case 0:
      _line=line0;
      finish(QtbXMLClass::ErrorUnmatchedAmpSemiColon);
      return;
    case 0x000A:    // '\n'
      _line++;
      break;
    case 0x0022:    // '"'
      finish(QtbXMLClass::ErrorUnmatchedAmpSemiColon);
      return;
    case 0x003C:    // '\<'
      finish(QtbXMLClass::ErrorUnmatchedAmpSemiColon);
      return;
    case 0x003B:    // ';'
      return;
    }
    _ptr++;
  }
}

/*!
  \a specChar must be null. It contains the pointer to the first occurence of a special character
  in the sting if any.
*/
void QtbXMLParser::skipString()
{
  QTB_TRACE;
  int line0 = _line;
  while ( true ) {
    switch ( _ptr->unicode() ) {
    case '\0':
      if (_error!=QtbXMLClass::NoError) {
        _ptr--; // Remain just before the end
        return;
      }
      _line=line0;
      finish(QtbXMLClass::ErrorEndTruncatedString);
      return;
    case 0x005C:    // '\\'
      _ptr++;
      break;
    case 0x000A:    // '\n'
      _line++;
      break;
    case 0x0026: { // '&'
        if ( !_specChar ) _specChar = _ptr; // keep track of first occurence of & in string
        skipSpecChar();
      }
      break;
    case 0x0022:    // '"'
      return;
    }
    _ptr++;
  }
}

/*!
  This function is called when at least one special character sequence has been found
  in the content. \a specChar is a pointer to the first occurence, hence *specChar=='&'.

  The general buffer is modified by this function, it is not possible to retrieve
  the original general buffer except by calling saveSpecChar()

  Return ErrorUnknowSpecialCharacter if \a specChar cannot be recognized.
*/
void QtbXMLParser::restoreSpecChar()
{
  //qtbApp << "Content before replacements ---" << content.toString() << "---" << endl;
  QtbStringSection cur( _content.data(), 0 );
  QtbStringSection code;
  QString& tmp = *new QString;
  const QChar * ptr = _specChar;
  const QChar * endPtr = _content.data() + _content.size();
  bool ok = true;
  while ( ptr < endPtr ) {
    switch ( ptr->unicode() ) {
    case 0x003B:    // ';'
      if ( code.isValid() ) {
        code.setEnd( ptr );
        tmp += specialCharacter( code, ok );
        if (!ok) {
          finish(QtbXMLClass::ErrorUnknowSpecialCharacter);
          return;
        }
        cur.set( ptr + 1, 0 );
        code.setInvalid();
      }
      break;
    case 0x0026:    // '&'
      cur.setEnd( ptr );
      cur.appendTo(tmp);
      code.set( ptr + 1, 0 );
      break;
    default:
      break;
    }
    ptr++;
  }
  cur.setEnd( ptr );
  cur.appendTo(tmp);
  _tmpStrings.append(&tmp);
  _content.set( tmp.data(), tmp.size() );
  //qtbApp << "Content after replacements ---" << restoredBuf << "---" << endl;
}

/*!
  _ptr points to first character '<'
*/
void QtbXMLParser::setTag()
{
  _ptr++;
  _tag.set( _ptr, 0 );
  while (true) {
    switch ( _ptr->unicode() ) {
    case '\0':
      finish(QtbXMLClass::ErrorEndTruncatedTag);
      return;
    case '\n':
      _tag.setEnd( _ptr );
      _tagType=Attributes;
      _line++;
      return;
    case '\r':
    case '\t':
    case ' ':
      _tag.setEnd( _ptr );
      _tagType=Attributes;
      return;
    case '?':
      if(_ptr[-1].unicode()=='<') {
        setTag();
        if(_ptr->unicode()=='>') {
          _tagType=QuestionTagNoAttributes;
        } else {
          _tagType=QuestionTagAttributes;
        }
      } else if(_ptr[1].unicode()=='>') {
      } else {
        finish(QtbXMLClass::ErrorEmptyTag);
        _tagType=Unknown;
      }
      return;
    case '/':
      if(_ptr[-1].unicode()=='<') {
        setTag();
        _tagType=CloseTag;
        return;
      }
      if(_ptr[1].unicode()!='>') {
        finish(QtbXMLClass::ErrorEmptyTag);
        _tagType=Unknown;
        return;
      }
      _tag.setEnd(_ptr);
      if(_tag.isEmpty()) finish(QtbXMLClass::ErrorEmptyTag);
      _ptr++;
      _tagType=SingleTagNoAttributes;
      return;
    case '>':
      _tag.setEnd( _ptr );
      if(_tag.isEmpty()) finish(QtbXMLClass::ErrorEmptyTag);
      _tagType=OpenTagNoAttributes;
      return;
    case '<':
      finish(QtbXMLClass::ErrorEmptyTag);
      _tagType=Unknown;
      return;
    default:
      break;
    }
    _ptr++;
  }
}

void QtbXMLParser::setAttributes()
{
  QtbStringSection attributeName;
  attributeName.set( _ptr, 0 );
  while ( true ) {
    switch ( _ptr->unicode() ) {
    case '\0':
      _tagType=Unknown;
      if ( _error!=QtbXMLClass::NoError ) {
        _ptr--; // Remain just before the end
        return;
      }
      finish(QtbXMLClass::ErrorEndTruncatedTag);
      return;
    case '\n':
      _line++;
      break;
    case '/':
      if(_ptr[1].unicode()!='>') {
        finish(QtbXMLClass::ErrorParsingAttributes);
        _tagType=Unknown;
        return;
      }
      attributeName.setEnd( _ptr );
      attributeName.trimmed();
      if(!attributeName.isEmpty()) {
        finish(QtbXMLClass::ErrorParsingAttributes);
        _tagType=Unknown;
        return;
      }
      _tagType=SingleTagAttributes;
      return;
    case '>':
      attributeName.setEnd( _ptr );
      attributeName.trimmed();
      if(!attributeName.isEmpty()) {
        finish(QtbXMLClass::ErrorParsingAttributes);
        _tagType=Unknown;
        return;
      }
      _tagType=OpenTagAttributes;
      return;
    case '=':
      attributeName.setEnd( _ptr );
      attributeName.trimmed();
      _ptr++;
      QtbXMLClass::skipBlanks(_ptr, _line);
      if ( _ptr->unicode() == '"' ) {
        _ptr++;
        _content.set(_ptr, 0);
        _specChar = 0;
        skipString();
        _content.setEnd( _ptr );
        if (_specChar) {
          restoreSpecChar();
        }
        _attributes.insert( attributeName, _content );
      } else {
        qtbApp << tr( "No value given for attribute %1 at line %2" ).arg(attributeName.toString()).arg(_line) << endl;
        _error=QtbXMLClass::ErrorParsingAttributes;
        _tagType=Unknown;
        return;
      }
      attributeName.set( _ptr + 1, 0 );
      break;
    default:
      break;
    }
    _ptr++;
  }
}

/*!
  Table for converting special character codes into QChar. \a ok is set to false in case of error.
*/
QChar QtbXMLParser::specialCharacter( const QtbStringSection& code, bool& ok)
{
  QTB_TRACE;
  if(code.size()<2) {
    ok = false;
    return QChar();
  }
  switch(code[0].unicode()) {
  case 'a':
    if ( code == "amp" ) return '&';
    else if ( code == "apos" ) return '\'';
    break;
  case 'g':
    if ( code == "gt" ) return '>';
    break;
  case 'l':
    if ( code == "lt" ) return '<';
    break;
  case 'q':
    if ( code == "quot" ) return '"';
    break;
  case '#':
    if ( code[1] =='x' ) {
      QString c( code.data()+2, code.size()-2 );
      return QChar( c.toInt(&ok, 16) );
    } else {
      QString c( code.data()+1, code.size()-1 );
      return QChar( c.toInt(&ok, 10) );
    }
  default:
    break;
  }
  ok = false;
  return QChar();
}

/*!
  _ptr points to first character '>' (End of an opening tag, no stacked)
*/
void QtbXMLParser::skip()
{
  int skippingLevel=_tagStack.count();
  _tagStack.push(_tag);
#ifdef XML_DEBUG
  printf("%5i === Skipping context %s, context size %i at line %i\n", __LINE__, _tag.toString().toAscii().data(), skippingLevel, _line);
#endif
  _ptr++;
  while ( true ) {
    switch ( _ptr->unicode() ) {
    case 0x000A:    // '\n'
      _line++;
      break;
    case 0x0026:    // '&'
      skipSpecChar();
      break;
    case 0x003C: {    // '<'
        _content.setEnd(_ptr);
        setTag();
        switch(_tagType) {
        case Attributes:
          _attributes.clear();
          qDeleteAll(_tmpStrings);
          _tmpStrings.clear();
          setAttributes();
          switch(_tagType) {
          case OpenTagAttributes:
            _tagStack.push(_tag);
            break;
          case SingleTagAttributes:
            // nothing to do just continue
            break;
          default:
            break;
          }
          break;
        case OpenTagNoAttributes:
          _tagStack.push(_tag);
          break;
        case CloseTag:
          if ( _tagStack.size() > 0 ) {
            if (_tag == _tagStack.top().tag()) {
              _tagStack.pop();
            } else {
              qtbApp << tr( "Looking for tag '%1', found '%2'" )
                .arg( currentContextTag() )
                .arg( _tag.toString() ) << endl;
              finish(QtbXMLClass::ErrorUnmatchedTag);
              return;
            }
          } else {
            finish(QtbXMLClass::ErrorEmptyContextStack);
            return;
          }
          if ( _tagStack.size() == skippingLevel ) {
#ifdef XML_DEBUG
            printf("%5i === End skipping context %s, context size %i at line %i\n", __LINE__, _tag.toString().toAscii().data(), skippingLevel, _line);
#endif
            return;
          }
          break;
        case SingleTagNoAttributes:
          // nothing to do just continue
          break;
        default:
          break;
        }
      }
      break;
    case 0x0000:    // END
      if (_error!=QtbXMLClass::NoError) {
      } else if ( _tag.isEmpty() ) {
        finish(QtbXMLClass::ErrorEndTruncatedTag);
      } else if (_tagStack.size() > 2 ) {
        finish(QtbXMLClass::ErrorEndTruncatedContext);
      } else if(!_object->xml_polished()) {
        finish(QtbXMLClass::ErrorEndTagNotFound);
      }
      return;
    default:
      break;
    }
    _ptr++;
  }
}
