/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-06-07
**
***************************************************************************/

#ifndef QTB_XMLPARSER_H
#define QTB_XMLPARSER_H

#include "qtbxmlclass.h"
#include "qtbtranslations.h"

class QtbXMLParser
{
  QTB_TRANSLATIONS("QtbXMLParser")
public:
  QtbXMLParser(QtbXMLStream * s, QtbXMLContext * context=0);
  ~QtbXMLParser();

  void setLine(int l) {_line=l;}
  void setPointer(const QChar * ptr) {_ptr=ptr;}
  QtbXMLClass::Error parse(QtbXMLClass * object);
private:
  void openContext();
  void closeContext();
  void smallContext();
  QString currentContextTag();
  void restoreBinaryData();
  void restoreBinaryData200510();
  void skipSpecChar();
  void skipString();
  void skip();
  void restoreSpecChar();
  void setTag();
  void setAttributes();
  void finish(QtbXMLClass::Error err);
  static QChar specialCharacter(const QtbStringSection& code, bool& ok);

  class Tag
  {
  public:
    Tag() : _member(QtbXMLMember::Unknown) {}
    Tag(const QtbStringSection& t) : _member(QtbXMLMember::Unknown), _tag(t) {}
    Tag(const QtbStringSection& t, const QtbXMLMember& m) : _member(m), _tag(t) {}
    Tag(const Tag& t) : _member(t._member), _tag(t._tag) {}

    const QtbXMLMember&  member() const {return _member;}
    const QtbStringSection&  tag() const {return _tag;}
  private:
    QtbXMLMember _member;
    QtbStringSection _tag;
  };

  QStack<Tag> _tagStack;
  QStack<QtbXMLClass *> _objectStack;

  QtbXMLClass * _object;
  QtbStringSection _tag;
  QtbXMLMember _member;
  QtbXMLContext * _context;
  QtbStringSection _content;
  QtbXMLRestoreAttributes _attributes;

  QtbXMLStream * _stream;
  const QChar * _ptr;
  int _line;
  QtbXMLClass::Error _error;
  const QChar * _specChar;
  enum TagType {Unknown, OpenTagNoAttributes, OpenTagAttributes, SingleTagNoAttributes, SingleTagAttributes,
                Attributes, QuestionTagNoAttributes, QuestionTagAttributes, CloseTag};
  TagType _tagType;
  /* If the content must be altered (restoreSpecChar),
     it is redirected to this temp string to keep using a QtbStringSection which requires a permanent string */
  QVector<QString *> _tmpStrings;
};

#endif // QTB_XMLPARSER_H
