/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-11-09
**
***************************************************************************/

#include <QMap>

#include "qtbxmlsaveattributes.h"
#include "qtbtrace.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbXMLSaveAttributes qtbxmlsaveattributes.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Returns the value corresponding to \a key. If \a key does not exist, a null string.
  Slow lookup, the complete list of attributes may be traversed.
*/
QString QtbXMLSaveAttributes::value( const QString& key )
{
  for(const_iterator it = begin(); it!=end();it++) {
    if( it->key() == key ) return it->value();
  }
  return QString::null;
}

/*!
  Returns true if it has all attributes of \a o. If \a o has no values for keys
  only the existence of the keys is checked (the values are not considered).
*/
bool QtbXMLSaveAttributes::isSimilar( const QtbXMLSaveAttributes& o ) const
{
  QTB_TRACE;
  // Cache available keys in a map
  QMap<QString, QString> map;
  for(const_iterator it = begin(); it!=end();it++) {
    map.insert( it->key(), it->value() );
  }
  QMap<QString, QString>::iterator itmap;
  for(const_iterator it = o.begin(); it!=o.end();it++) {
    itmap = map.find( it->key() );
    if (itmap==map.end() || ( !it->value().isEmpty() && itmap.value() != it->value() ) ) {
      return false;
    }
  }
  return true;
}

QString QtbXMLSaveAttributes::toString() const
{
  QTB_TRACE;
  QString tmp;
  for(const_iterator it = begin();it!=end();it++) {
    tmp += " ";
    tmp += it->key();
    tmp += "=\"";
    tmp += it->value();
    tmp += "\"";
  }
  return tmp;
}
