/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbtools.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
**  License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-11-09
**
***************************************************************************/

#ifndef QTB_XMLSAVEATTRIBUTES_H
#define QTB_XMLSAVEATTRIBUTES_H

#include <QList>
#include <QPair>
#include <QString>

#include "qtbtoolsdllexport.h"

class QTBTOOLS_EXPORT QtbXMLSaveAttribute : private QPair<QString,QString>
{
public:
  QtbXMLSaveAttribute() {}
  QtbXMLSaveAttribute( const QString& key, const QString& value ) { first = key; second = value; }

  QString& key() { return first; }
  QString& value() { return second; }

  const QString& key() const { return first; }
  const QString& value() const { return second; }

  bool operator==( const QtbXMLSaveAttribute& o ) const { return first == o.first && second == o.second; }
};

class QTBTOOLS_EXPORT QtbXMLSaveAttributes : private QList<QtbXMLSaveAttribute>
{
public:
  QtbXMLSaveAttributes() {}
  //QtbXMLSaveAttributes( const QString& key ) : QList<QtbXMLSaveAttribute>() { add(key); }

  void add( QString key, QString value = QString::null ) { append( QtbXMLSaveAttribute(key, value ) ); }

  const QString& key( int index ) const { return (*this)[index].key(); }
  const QString& value( int index ) const { return (*this)[index].value(); }
  QString& key( int index ) { return (*this)[index].key(); }
  QString& value( int index ) { return (*this)[index].value(); }
  QString value( const QString& key );

  QString toString() const;
  inline QString toEncodedString() const;

  void clear() { QList<QtbXMLSaveAttribute>::clear(); }
  bool isEmpty() const { return QList<QtbXMLSaveAttribute>::isEmpty(); }
  bool operator==( const QtbXMLSaveAttributes& o ) const { return QList<QtbXMLSaveAttribute>::operator==( o ); }
  bool isSimilar( const QtbXMLSaveAttributes& o ) const;
};

#endif // QTB_XMLSAVEATTRIBUTES_H
