/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-05-22
**
***************************************************************************/

#ifndef QTB_AUTOCORRCURVES_H
#define QTB_AUTOCORRCURVES_H

#include <QVector>

#include "qtbautocorrring.h"
#include "qtbmodalcurve.h"
#include "qtbxmlclass.h"
#include "qtbwavedllexport.h"
#include "qtbtranslations.h"

class QTBWAVE_EXPORT QtbAutocorrCurves : public QtbXMLClass
{
  QTB_TRANSLATIONS("QtbAutocorrCurves");
public:

  virtual const QString& xml_tagName() const { return xmlAutocorrCurvesTag;}
  static const QString xmlAutocorrCurvesTag;

  bool isEmpty() const { return _curves.isEmpty(); }
  int ringCount() const { return _rings.count(); }
  void addRing() { addRing( QtbAutocorrRing() ); }
  void addRing( const QtbAutocorrRing& r );
  void removeRing( int index );
  void add( const QtbAutocorrCurves& curves );
  void sort();

  const QtbAutocorrRing& ring( int index ) const { return _rings[index]; }
  QtbAutocorrRing& ring( int index ) { return _rings[index]; }
  int indexOf( const QtbAutocorrRing& r ) const { return _rings.indexOf(r); }
  const QVector<QtbAutocorrRing> * rings() const { return &_rings; }

  void addCurve( const QtbModalCurve& c );
  const QList<QtbModalCurve>& curves() const { return _curves; }
  QList<QtbModalCurve>& curves() { return _curves; }
  QtbModalCurve& lastCurve() { return _curves.last(); }
  QtbModalCurve dispersionCurve( int index, double kmin, double kmax ) const;

  QList<QtbModalCurve> ringCurves( int index ) const;
protected:
  void xml_writeChildren(XML_WRITECHILDREN_ARGS) const;
  QtbXMLMember xml_member(XML_MEMBER_ARGS);
private:
  QList<QtbModalCurve> _curves;
  QVector<QtbAutocorrRing> _rings;
};

#endif // QTB_AUTOCORRCURVES_H
