/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2009-02-04
**
***************************************************************************/

#include "qtbautocorrdispersion.h"
#include "qtbrootsolver.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbAutocorrDispersion qtbautocorrdispersion.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Calculates y(k)=verticalAutocorrelation(k)-autocorrRef.
*/
double QtbAutocorrDispersion::y( double k )
{
  QTB_TRACE;
  if ( isThinRing() ) {
    return verticalThinRing( k )-_autocorrRef;
  } else {
    return verticalThickRing( k )-_autocorrRef;
  }
}

/*!
  Returns all the dispersion curve solutions for \a autocorr between \a kmin and \a kmax.
*/
QVector<double> QtbAutocorrDispersion::dispersion( const QtbFactoryPoint& autocorr, double kmin, double kmax )
{
  QTB_TRACE;
  QVector<double> results;
  QtbRootSolver<QtbAutocorrDispersion> solver( this );
  solver.setPrecision( 1e-7 );
  solver.setRelativeStep( 0.1 );
  _autocorrRef = autocorr.y();
  solver.setPolarity(kmax);
  double invOmega = 0.5 / ( M_PI * autocorr.x() );
  while( solver.searchDownSlope( kmax, kmin, kmax ) ) {
    solver.neville();
    kmax = solver.lower();
    results.append( kmax*invOmega );
    solver.inversePolarity();
  }
  return results;
}
