/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-02-27
**
***************************************************************************/

#include "qtbautocorrfactory.h"
#include "qtbautocorrengine.h"
#include "qtbdispersionfactory.h"
#include "qtbmodalcurve.h"
#include "qtbmode.h"

/*!
  \class QtbAutocorrFactory qtbautocorrfactory.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
QtbAutocorrFactory::QtbAutocorrFactory(const QVector<QtbAutocorrRing> * rings)
{
  QTB_TRACE;
  _rings = rings;
  _vertical = 0;
  _radial = 0;
  _transverse = 0;
  // internals
  /*_jRR = 0;
  _jLR = 0;
  _jRT = 0;
  _jLT = 0;
  _alpha = 0;*/
}

/*!
  Description of destructor still missing
*/
QtbAutocorrFactory::~QtbAutocorrFactory()
{
  QTB_TRACE;
  int nRings = _rings->count();
  for (int i=0;i<nRings;i++) {
    if (_vertical) delete _vertical[i];
    if (_radial) delete _radial[i];
    if (_transverse) delete _transverse[i];
    /*if (_jRR) delete _jRR[i];
    if (_jLR) delete _jLR[i];
    if (_jRT) delete _jRT[i];
    if (_jLT) delete _jLT[i];*/
  }
  delete [] _vertical;
  delete [] _radial;
  delete [] _transverse;
  /*delete [] _jRR;
  delete [] _jLR;
  delete [] _jRT;
  delete [] _jLT;
  if (_alpha) {
    for (int i = 0; i<_nHorizontalModes; i++) {
      delete [] _alpha[i];
    }
    delete [] _alpha;
  }*/
}

void QtbAutocorrFactory::upgrade ( QtbModalStorage **& s, int ringIndex, int requestedMode )
{
  QTB_TRACE;
  if ( !s ) {
    int nRings = _rings->count();
    s = new QtbModalStorage * [nRings];
    for (int i = 0;i<nRings;i++) {
      s[i] = 0;
    }
  }
  QtbModalFactory::upgrade<QtbModalStorage>(s[ringIndex], requestedMode);
}

void QtbAutocorrFactory::setMode ( const QtbMode& m )
{
  QTB_TRACE;
  switch ( m.polarisation() ) {
  case QtbMode::Radial:
    upgrade( _radial, m.ringIndex(), m.index() ); break;
  case QtbMode::Transverse:
    upgrade( _transverse, m.ringIndex(), m.index() ); break;
  default:
    upgrade( _vertical, m.ringIndex(), m.index() ); break;
  }
}

const QtbValue * QtbAutocorrFactory::mode( const QtbMode& m) const
{
  switch ( m.polarisation() ) {
  case QtbMode::Radial:
    return _radial ? _radial[m.ringIndex()]->mode( m.index() ) : 0;
  case QtbMode::Transverse:
    return _transverse ? _transverse[m.ringIndex()]->mode( m.index() ) : 0;
  default:
    return _vertical ? _vertical[m.ringIndex()]->mode( m.index() ) : 0;
  }
}

int QtbAutocorrFactory::maxModeCount( QtbModalStorage ** s ) const
{
  QTB_TRACE;
  if ( s ) {
    int nr = _rings->count();
    int maxModeCount = 0;
    for ( int iRing = 0;iRing < nr; iRing++ ) {
      if ( s[iRing]) {
        int nm = s[iRing]->modeCount();
        if (nm>maxModeCount) maxModeCount=nm;
      }
    }
    return maxModeCount;
  } else return 0;
}

int QtbAutocorrFactory::horizontalModeCount() const
{
  QTB_TRACE;
  int nm = maxModeCount( _radial );
  int nTransverse = maxModeCount( _transverse );
  if ( nTransverse < nm ) nm = nTransverse;
  return nm;
}

QtbModalStorage ** QtbAutocorrFactory::newHorizontalStorage()
{
  QTB_TRACE;
  int nRings = _rings->count();
  QtbModalStorage ** s = new QtbModalStorage * [nRings];
  for (int i = 0;i<nRings;i++) {
    if ( _radial[i] && _transverse[i] ) {
      s[i] = newStorage( _nHorizontalModes );
    } else {
      s[i] = 0;
    }
  }
  return s;
}

void QtbAutocorrFactory::setInvalid( QtbModalStorage ** s )
{
  QTB_TRACE;
  if (s) {
    int nx = _x.count();
    int nr = _rings->count();
    for ( int iRing = 0;iRing < nr; iRing++ ) {
      if ( s[iRing] ) {
        int nm = s[iRing]->modeCount();
        for ( int iMode = 0;iMode < nm;iMode++ ) {
          QtbValue * points = s[iRing]->mode(iMode);
          for ( int i = 0;i < nx;i++ ) {
            points[i].setValid(false);
          }
        }
      }
    }
  }
}

void QtbAutocorrFactory::init()
{
  QTB_TRACE;
  if ( isRadial() || isTransverse() ) {
    //int nx = _x.count();
    _nHorizontalModes = horizontalModeCount();
    setInvalid( _radial );
    setInvalid( _transverse );
    //_jRR = newHorizontalStorage();
    //_jLR = newHorizontalStorage();
    //_jRT = newHorizontalStorage();
    //_jLT = newHorizontalStorage();
    /*_alpha = new QtbStatValue *[_nHorizontalModes];
    for (int i = 0; i<_nHorizontalModes; i++) {
      QtbStatValue * s = new QtbStatValue[nx];
      for (int j = 0; j<nx; j++) {
        s[j].setValid(false);
      }
      _alpha[i] = s;
    }*/
  }
  if (isVertical()) {
    _nVerticalModes = verticalModeCount();
    setInvalid( _vertical );
  }
}

/*!
  Calculate the autocorrelation curves from a dispersion curve for all possible modes
  Min(nModes(),disp.nModes())
*/
void QtbAutocorrFactory::calculateVertical( QtbDispersionFactory * disp )
{
  QTB_TRACE;
  QtbDispersion& dispRayleigh = *disp->phaseRayleigh();
  int nr = _rings->count();
  int nx = _x.count();
  QTB_ASSERT( _nVerticalModes <= dispRayleigh.modeCount() );
  QtbAutocorrEngine eng;
  for ( int iMode = 0;iMode < _nVerticalModes;iMode++ ) {
    QtbValue * dispR = dispRayleigh.mode( iMode );
    for ( int iRing = 0;iRing < nr; iRing++ ) {
      if ( _vertical[iRing] ) {
        eng.setVerticalRing( _rings->at( iRing ) );
        QtbValue * autocorrV = vertical(iRing).mode(iMode);
        if ( eng.isThinRing() ) {
          for ( int ix = 0; ix < nx; ix++ )
            autocorrV[ix] = eng.verticalThinRing( x()->at(ix), dispR[ix] );
        } else {
          for ( int ix = 0; ix < nx; ix++ )
            autocorrV[ix] = eng.verticalThickRing( x()->at(ix), dispR[ix] );
        }
      }
    }
  }
}

void QtbAutocorrFactory::calculateHorizontal( double alphaR, QtbDispersionFactory * disp )
{
  QTB_TRACE;
  double alphaL = 1.0 - alphaR;
  QtbDispersion& dispRayleigh = *disp->phaseRayleigh();
  QtbDispersion& dispLove = *disp->phaseLove();
  int nr = _rings->count();
  int nx = _x.count();
  QTB_ASSERT( _nHorizontalModes <= dispRayleigh.modeCount() );
  QTB_ASSERT( _nHorizontalModes <= dispLove.modeCount() );
  QtbAutocorrEngine eng;
  for ( int iMode = 0;iMode < _nHorizontalModes;iMode++ ) {
    QtbValue * dispR = dispRayleigh.mode( iMode );
    QtbValue * dispL = dispLove.mode( iMode );
    for ( int iRing = 0;iRing < nr; iRing++ ) {
      if ( _radial[iRing] && _transverse[iRing]) {
        eng.setHorizontalRing( _rings->at( iRing ) );
        QtbValue * autocorrR = radial(iRing).mode(iMode);
        QtbValue * autocorrT = transverse(iRing).mode(iMode);
        if ( eng.isThinRing() ) {
          for ( int ix = 0; ix < nx; ix++ ) {
            if ( eng.initHorizontalThinRing( x()->at(ix), dispR[ix], dispL[ix] ) ) {
              autocorrR[ix] = eng.radial( alphaR, alphaL );
              autocorrT[ix] = eng.transverse( alphaR, alphaL );
            } else {
              autocorrR[ix].setValid( false );
              autocorrT[ix].setValid( false );
            }
          }
        } else {
          for ( int ix = 0; ix < nx; ix++ ) {
            if ( eng.initHorizontalThickRing( x()->at(ix), dispR[ix], dispL[ix] ) ) {
              autocorrR[ix] = eng.radial( alphaR, alphaL );
              autocorrT[ix] = eng.transverse( alphaR, alphaL );
            } else {
              autocorrR[ix].setValid( false );
              autocorrT[ix].setValid( false );
            }
          }
        }
      }
    }
  }
}

/*!
  Calculation radial and transverse autocorrelations for the \a iOmega th frequency and for fundamental mode
*/
void QtbAutocorrFactory::calculateHorizontal( int iOmega, double alphaR, QtbDispersionFactory * disp )
{
  QTB_TRACE;
  double alphaL = 1.0 - alphaR;
  QtbDispersion& dispRayleigh = *disp->phaseRayleigh();
  QtbDispersion& dispLove = *disp->phaseLove();
  int nr = _rings->count();
  QTB_ASSERT( _nHorizontalModes <= dispRayleigh.modeCount() );
  QTB_ASSERT( _nHorizontalModes <= dispLove.modeCount() );
  QtbAutocorrEngine eng;
  QtbValue * dispR = dispRayleigh.mode( 0 );
  QtbValue * dispL = dispLove.mode( 0 );
  for ( int iRing = 0;iRing < nr; iRing++ ) {
    if ( _radial[iRing] && _transverse[iRing]) {
      eng.setHorizontalRing( _rings->at( iRing ) );
      QtbValue * autocorrR = radial(iRing).mode(0);
      QtbValue * autocorrT = transverse(iRing).mode(0);
      if ( eng.isThinRing() ) {
        if ( eng.initHorizontalThinRing( x()->at(iOmega), dispR[iOmega], dispL[iOmega] ) ) {
          autocorrR[iOmega] = eng.radial( alphaR, alphaL );
          autocorrT[iOmega] = eng.transverse( alphaR, alphaL );
        } else {
          autocorrR[iOmega].setValid( false );
          autocorrT[iOmega].setValid( false );
        }
      } else {
        if ( eng.initHorizontalThickRing( x()->at(iOmega), dispR[iOmega], dispL[iOmega] ) ) {
          autocorrR[iOmega] = eng.radial( alphaR, alphaL );
          autocorrT[iOmega] = eng.transverse( alphaR, alphaL );
        } else {
          autocorrR[iOmega].setValid( false );
          autocorrT[iOmega].setValid( false );
        }
      }
    }
  }
}

void QtbAutocorrFactory::calculate( double alphaR, QtbDispersionFactory * disp )
{
  QTB_TRACE;
  if ( isVertical() ) calculateVertical( disp );
  if ( isRadial() || isTransverse() ) calculateHorizontal( alphaR, disp );
}

void QtbAutocorrFactory::calculate( QtbDispersionFactory * disp )
{
  QTB_TRACE;
  if ( isVertical() ) calculateVertical( disp );
  /* To be implemented correctly
  if ( isRadial() || isTransverse() ) {
    QtbDispersion& dispRayleigh = *disp->phaseRayleigh();
    QtbDispersion& dispLove = *disp->phaseLove();
    int nr = _rings->count();
    int nx = _x.count();
    QTB_ASSERT( _nHorizontalModes <= dispRayleigh.modeCount() );
    QTB_ASSERT( _nHorizontalModes <= dispLove.modeCount() );
    QtbAutocorrEngine eng( x() );
    for ( int iMode = 0;iMode < _nHorizontalModes;iMode++ ) {
      autocorr.setDispersion( dispRayleigh.mode( iMode ), dispLove.mode( iMode ) );
      for ( int iRing = 0;iRing < nr; iRing++ ) {
        if ( _radial[iRing] && _transverse[iRing]) {
          autocorr.setRing( _rings->at( iRing ), radial(iRing).mode(iMode), transverse(iRing).mode(iMode),
                            _jRR[iRing]->mode(iMode), _jLR[iRing]->mode(iMode), _jRT[iRing]->mode(iMode), _jLT[iRing]->mode(iMode) );
          if ( autocorr.isThinRing() ) {
            for ( int j = 0;j < nx;j++ ) autocorr.horizontalThinRing( j );
          } else {
            for ( int j = 0;j < nx;j++ ) autocorr.horizontalThickRing( j );
          }
        }
      }
    }
  }*/
}

#if 0
/*!
  Used only by testing code in QtbTargetList (direct inversion of 3C Spac
*/
void QtbAutocorrFactory::setAlpha( const QList<QtbModalCurve> curves )
{
  QTB_TRACE;
  for (QList<QtbModalCurve>::ConstIterator itCurve = curves.begin(); itCurve!= curves.end(); itCurve++ ) {
    const QtbModalCurve& curve = *itCurve;
    const QList<QtbMode>& modes = curve.modes();
    for (QList<QtbMode>::ConstIterator itMode = modes.begin(); itMode!= modes.end(); itMode++ ) {
      const QtbMode& mode = *itMode;
      double factor;
      int iRing = mode.ringIndex();
      int iMode = mode.index();
      QtbValue * j1, *j2;
      QtbStatValue alpha;
      switch ( mode.polarisation() ) {
      case QtbMode::Radial:
        j1 = _jLR[iRing]->mode(iMode);
        j2 = _jRR[iRing]->mode(iMode);
        for (QtbModalCurve::ConstIterator it = curve.begin(); it != curve.end(); it++ ) {
          const QtbFactoryPoint& p = *it;
          int index = p.index();
          double j1Value = j1[index].value();
          factor = 1.0 / ( j1Value - j2[index].value() );
          alpha.setMean( factor * ( j1Value - p.mean() ) );
          alpha.setStddev( factor * p.stddev() );
          alpha.setWeight( p.weight() );
          if (alpha.mean()>1.0) alpha.setMean(1.0);
          else if (alpha.mean()<0.0) alpha.setMean(0.0);
          printf("radial alpha mode %i index %i %lg / %lg\n",iMode, index, alpha.mean(),alpha.stddev());
          _alpha[iMode][index].average(alpha);
        }
        break;
      case QtbMode::Transverse:
        j1 = _jLT[iRing]->mode(iMode);
        j2 = _jRT[iRing]->mode(iMode);
        for (QtbModalCurve::ConstIterator it = curve.begin(); it != curve.end(); it++ ) {
          const QtbFactoryPoint& p = *it;
          int index = p.index();
          double j1Value = j1[index].value();
          factor = 1.0 / ( j1Value - j2[index].value() );
          alpha.setMean( factor * ( j1Value - p.mean() ) );
          alpha.setStddev( factor * p.stddev() );
          alpha.setWeight( p.weight() );
          if (alpha.mean()>1.0) alpha.setMean(1.0);
          else if (alpha.mean()<0.0) alpha.setMean(0.0);
          printf("transv alpha mode %i index %i %lg / %lg\n",iMode, index, alpha.mean(),alpha.stddev());
          _alpha[iMode][index].average(alpha);
        }
        break;
      default:
        break;
      }
    }
  }
}

/*!
  Used only by testing code in QtbTargetList (direct inversion of 3C Spac)
*/
void QtbAutocorrFactory::setHorizontalAutocorr()
{
  QTB_TRACE;
  int nr = _rings->count();
  int nx = _x.count();
  for ( int iMode = 0;iMode < _nHorizontalModes;iMode++ ) {
    QtbStatValue * alpha = _alpha[iMode];
    for ( int iRing = 0;iRing < nr; iRing++ ) {
      if ( _radial[iRing] && _transverse[iRing]) {
        QtbValue * autocorrR = radial(iRing).mode(iMode);
        QtbValue * autocorrT = transverse(iRing).mode(iMode);
        QtbValue * jRR = _jRR[iRing]->mode(iMode);
        QtbValue * jLR = _jLR[iRing]->mode(iMode);
        QtbValue * jRT = _jRT[iRing]->mode(iMode);
        QtbValue * jLT = _jLT[iRing]->mode(iMode);
        for ( int j = 0;j < nx;j++ ) {
          double alphaR = alpha[j].mean();
          if (alphaR<0.0) alphaR=0.0;
          else if (alphaR>1.0) alphaR=1.0;
          double alphaL = 1.0 - alphaR;
          autocorrR[ j ].setValue ( alphaR * jRR[ j ].value() + alphaL * jLR[ j ].value() );
          autocorrT[ j ].setValue ( alphaR * jRT[ j ].value() + alphaL * jLT[ j ].value() );
        }
      }
    }
  }
}
#endif

/*!
  Unoptimized litteral impletation, for testing only
*/
double QtbAutocorrFactory::radialAutocorr( double omega, double r1, double r2, double alpha, double cr, double cl)
{
  QTB_TRACE;
  //printf("%lf: %lf %lf %lf %lf %lf\n",omega, r1, r2, alpha, cr,cl);
  double omegaR1=omega*r1/cr; 
  double omegaR2=omega*r2/cr; 
  double omegaL1=omega*r1/cl; 
  double omegaL2=omega*r2/cl; 
  return 4/(r2*r2-r1*r1)*(alpha*cr/omega*(cr/omega*(j0(omegaR2)-j0(omegaR1))+(r2*j1(omegaR2)-r1*j1(omegaR1)))-(1-alpha)*cl*cl/(omega*omega)*(j0(omegaL2)-j0(omegaL1)));
}

/*!
  Unoptimized litteral impletation, for testing only
*/
double QtbAutocorrFactory::transverseAutocorr( double omega, double r1, double r2, double alpha, double cr, double cl)
{
  QTB_TRACE;
  double omegaR1=omega*r1/cr; 
  double omegaR2=omega*r2/cr; 
  double omegaL1=omega*r1/cl; 
  double omegaL2=omega*r2/cl; 
  return 4/(r2*r2-r1*r1)*(-alpha*cr*cr/(omega*omega)*(j0(omegaR2)-j0(omegaR1))+(1-alpha)*cl/omega*(cl/omega*(j0(omegaL2)-j0(omegaL1))+(r2*j1(omegaL2)-r1*j1(omegaL1))));
}
