/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-02-27
**
***************************************************************************/

#ifndef QTB_AUTOCORRFACTORY_H
#define QTB_AUTOCORRFACTORY_H

#include "qtbdispersion.h"
#include "qtbmodalfactory.h"
#include "qtbautocorrring.h"
#include "qtbwavedllexport.h"

class QtbDispersionFactory;

class QTBWAVE_EXPORT QtbAutocorrFactory : public QtbModalFactory
{
public:
  QtbAutocorrFactory(const QVector<QtbAutocorrRing> * rings);
  ~QtbAutocorrFactory();

  int verticalModeCount() const { return maxModeCount(_vertical); }
  int horizontalModeCount() const;

  void init();

  bool isVertical() const { return _vertical; }
  bool isRadial() const { return _radial; }
  bool isTransverse() const { return _transverse; }

  int ringCount() const { return _rings->count(); }
  QtbModalStorage& vertical( int ringIndex ) const { return *_vertical[ringIndex]; }
  QtbModalStorage& radial( int ringIndex ) const { return *_radial[ringIndex]; }
  QtbModalStorage& transverse( int ringIndex ) const { return *_transverse[ringIndex]; }

  void calculate( QtbDispersionFactory * disp );
  void calculate( double alphaR, QtbDispersionFactory * disp );
  void calculateVertical( QtbDispersionFactory * disp );
  void calculateHorizontal( double alphaR, QtbDispersionFactory * disp );
  void calculateHorizontal( int iOmega, double alphaR, QtbDispersionFactory * disp );
  //void setAlpha( const QList<QtbModalCurve> curves );
  //void setHorizontalAutocorr();

  virtual void setMode ( const QtbMode& m );
  virtual const QtbValue * mode( const QtbMode& m ) const;
  virtual int storageCount() const { return _rings->count()*3; }
  inline virtual void writeReportHeader( QDataStream& s ) const;
  inline virtual QtbModalStorage * storage( int storageIndex ) const;

  double radialAutocorr( double omega, double r1, double r2, double alpha, double cr, double cl);
  double transverseAutocorr( double omega, double r1, double r2, double alpha, double cr, double cl);
protected:
  virtual QtbModalStorage * newStorage( int nModes ) { return new QtbModalStorage ( nModes, x() ); }
  QtbModalStorage ** newHorizontalStorage();
  void upgrade (  QtbModalStorage **& s, int ringIndex, int requestedMode );
private:
  int maxModeCount( QtbModalStorage ** s ) const;
  void setInvalid( QtbModalStorage ** s );

  const QVector<QtbAutocorrRing> * _rings;
  QtbModalStorage ** _vertical;
  QtbModalStorage ** _radial;
  QtbModalStorage ** _transverse;
  // internals
  int _nVerticalModes;
  int _nHorizontalModes;
  /*QtbModalStorage ** _jRR;
  QtbModalStorage ** _jLR;
  QtbModalStorage ** _jRT;
  QtbModalStorage ** _jLT;
  QtbStatValue ** _alpha;*/
};

inline void QtbAutocorrFactory::writeReportHeader( QDataStream& s ) const
{
  QTB_TRACE;
  s << _rings->count();
}

inline QtbModalStorage * QtbAutocorrFactory::storage( int storageIndex ) const
{
  QTB_TRACE;
  int iRing = storageIndex/3;
  int iComponent = storageIndex-iRing*3;
  switch ( iComponent ) {
  case 1:
    if (_radial) return _radial[iRing]; else return 0;
  case 2:
    if (_transverse) return _transverse[iRing]; else return 0;
  default:
    if (_vertical) return _vertical[iRing]; else return 0;
  }
}

#endif // QTB_AUTOCORRFACTORY_H
