/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-09-25
**
***************************************************************************/

#ifndef QTB_AUTOCORRRING_H
#define QTB_AUTOCORRRING_H

#include "qtbxmlclass.h"
#include "qtbfactorypoint.h"
#include "qtbwavedllexport.h"

class QTBWAVE_EXPORT QtbAutocorrRing : public QtbXMLClass
{
public:
  inline QtbAutocorrRing();
  inline QtbAutocorrRing( const QtbAutocorrRing& o );
  inline QtbAutocorrRing( double rMin, double rMax );

  inline QtbAutocorrRing& operator=( const QtbAutocorrRing& o );
  inline bool operator==( const QtbAutocorrRing& o ) const;
  inline bool operator<( const QtbAutocorrRing& o ) const;

  virtual const QString& xml_tagName() const { return xmlAutocorrRingTag; }
  static const QString xmlAutocorrRingTag;

  inline void setRadii( double rMin, double rMax );
  double minRadius() const { return _minRadius;}
  double maxRadius() const { return _maxRadius;}
protected:
  virtual void xml_writeProperties( XML_WRITEPROPERTIES_ARGS ) const;
  virtual QtbXMLMember xml_member( XML_MEMBER_ARGS );
  virtual bool xml_setProperty( XML_SETPROPERTY_ARGS );
private:
  double _minRadius, _maxRadius;
};

inline QtbAutocorrRing::QtbAutocorrRing()
  : QtbXMLClass()
{
  _minRadius = 1.0;
  _maxRadius = 2.0;
}

inline QtbAutocorrRing::QtbAutocorrRing( const QtbAutocorrRing& o )
  : QtbXMLClass()
{
  setRadii( o.minRadius(), o.maxRadius() );
}

inline QtbAutocorrRing::QtbAutocorrRing( double rMin, double rMax )
  : QtbXMLClass()
{
  setRadii( rMin, rMax );
}

inline QtbAutocorrRing& QtbAutocorrRing::operator=( const QtbAutocorrRing& o )
{
  QTB_TRACE;
  _minRadius=o._minRadius;
  _maxRadius=o._maxRadius;
  return *this;
}

inline bool QtbAutocorrRing::operator==( const QtbAutocorrRing& o ) const
{
  QTB_TRACE;
  return _minRadius==o._minRadius && _maxRadius==o._maxRadius;
}

inline bool QtbAutocorrRing::operator<( const QtbAutocorrRing& o ) const
{
  QTB_TRACE;
  return _minRadius<o._minRadius || ( _minRadius==o._minRadius && _maxRadius<o._maxRadius );
}

inline void QtbAutocorrRing::setRadii( double rMin, double rMax )
{
  if ( rMin < rMax ) {
    _minRadius = rMin;
    _maxRadius = rMax;
  } else {
    _maxRadius = rMin;
    _minRadius = rMax;
  }
}

#endif // QTB_AUTOCORRRING_H
