/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-09-26
**
***************************************************************************/

#ifndef QTB_DISPERSIONFACTORY_H
#define QTB_DISPERSIONFACTORY_H

#include "qtbmodalfactory.h"
#include "qtbdispersion.h"
#include "qtbwavedllexport.h"

class QtbEllipticityFactory;
class QtbAutocorrFactory;

class QTBWAVE_EXPORT QtbDispersionFactory : public QtbModalFactory
{
public:
  QtbDispersionFactory();
  ~QtbDispersionFactory();

  QtbDispersion * phaseRayleigh() const { return _phaseRayleigh; }
  QtbDispersion * groupRayleigh() const { return _groupRayleigh; }
  QtbDispersion * phaseLove() const { return _phaseLove; }
  QtbDispersion * groupLove() const { return _groupLove; }

  void validate( QtbEllipticityFactory * ellFactory, QtbAutocorrFactory * autocorr );
  bool calculate( const QtbLayeredModel * model, QtbEllipticityFactory * ellFactory );

  virtual void setMode ( const QtbMode& m );
  virtual const QtbValue * mode( const QtbMode& m ) const;
  virtual int storageCount() const { return 4; }
  inline virtual QtbModalStorage * storage( int storageIndex ) const;
protected:
  virtual QtbModalStorage * newStorage( int nModes ) { return new QtbDispersion( nModes, x() ); }
private:
  QtbDispersion * _phaseRayleigh;
  QtbDispersion * _groupRayleigh;
  QtbDispersion * _phaseLove;
  QtbDispersion * _groupLove;
};

inline QtbModalStorage *  QtbDispersionFactory::storage( int storageIndex ) const
{
  QTB_TRACE;
  switch ( storageIndex ) {
  case 1:
    return _groupRayleigh;
  case 2:
    return _phaseLove;
  case 3:
    return _groupLove;
  default:
    return _phaseRayleigh;
  }
}

#endif // QTB_DISPERSIONFACTORY_H
