/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-10-13
**
***************************************************************************/

#include "qtblayeredmodelexpressioncontext.h"
#include "qtblayeredmodel.h"
#include "qtbtrace.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbLayeredModelExpressionContext qtblayeredmodelexpressioncontext.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
QtbLayeredModelExpressionContext::QtbLayeredModelExpressionContext()
    : QtbExpressionContext()
{
  QTB_TRACE;
  addVariable( "n", new QtbLayeredModelExpressionN(this) );
  addVariable( "h", new QtbLayeredModelExpressionH(this) );
  addVariable( "vp", new QtbLayeredModelExpressionVp(this) );
  addVariable( "vs", new QtbLayeredModelExpressionVs(this) );
  addVariable( "rho", new QtbLayeredModelExpressionRho(this) );
}

QStringList QtbLayeredModelExpressionContext::functions() const
{
  QTB_TRACE;
  QStringList l;
  return l;
}

bool QtbLayeredModelExpressionContext::isValidFunction(const QString& ) const
{
  QTB_TRACE;
  return false;
}

QVariant QtbLayeredModelExpressionContext::functionValue(const QString& , const QVector<QtbExpressionAction *>& ) const
{
  QTB_TRACE;
  return QVariant();
}

QVariant QtbLayeredModelExpressionN::value(int) const
{
  QTB_TRACE;
  return _context->model()->layerCount();
}

QVariant QtbLayeredModelExpressionH::value(int index) const
{
  QTB_TRACE;
  QtbLayeredModel * m = _context->model();
  if( index>=0 && index < m->layerCount()-1 )
    return m->h(index);
  else
    return QVariant();
}

void QtbLayeredModelExpressionH::setValue(int index, const QVariant& val)
{
  QTB_TRACE;
  QtbLayeredModel * m = _context->model();
  if( index>=0 && index < m->layerCount()-1 ) {
    m->setH( index, val.toDouble() );
  }
}

QVariant QtbLayeredModelExpressionVp::value(int index) const
{
  QTB_TRACE;
  QtbLayeredModel * m = _context->model();
  if( index>=0 && index < m->layerCount() )
    return 1.0/m->slowP(index);
  else
    return QVariant();
}

void QtbLayeredModelExpressionVp::setValue(int index, const QVariant& val)
{
  QTB_TRACE;
  QtbLayeredModel * m = _context->model();
  if( index>=0 && index < m->layerCount() ) {
    m->setSlowP(index, 1.0/val.toDouble() );
  }
}

QVariant QtbLayeredModelExpressionVs::value(int index) const
{
  QTB_TRACE;
  QtbLayeredModel * m = _context->model();
  if( index>=0 && index < m->layerCount() )
    return 1.0/m->slowS(index);
  else
    return QVariant();
}

void QtbLayeredModelExpressionVs::setValue(int index, const QVariant& val)
{
  QTB_TRACE;
  QtbLayeredModel * m = _context->model();
  if( index>=0 && index < m->layerCount() ) {
    m->setSlowS( index, 1.0/val.toDouble() );
  }
}

QVariant QtbLayeredModelExpressionRho::value(int index) const
{
  QTB_TRACE;
  QtbLayeredModel * m = _context->model();
  if( index>=0 && index < m->layerCount() )
    return m->rho(index);
  else
    return QVariant();
}

void QtbLayeredModelExpressionRho::setValue(int index, const QVariant& val)
{
  QTB_TRACE;
  QtbLayeredModel * m = _context->model();
  if( index>=0 && index < m->layerCount() ) {
    m->setRho( index, val.toDouble() );
  }
}
