/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-10-13
**
***************************************************************************/

#ifndef QTB_LAYEREDMODELEXPRESSIONCONTEXT_H
#define QTB_LAYEREDMODELEXPRESSIONCONTEXT_H

#include "qtbexpressioncontext.h"
#include "qtbexpressionstorage.h"
#include "qtbwavedllexport.h"

class QtbLayeredModel;

class QTBWAVE_EXPORT QtbLayeredModelExpressionContext : public QtbExpressionContext
{
public:
  QtbLayeredModelExpressionContext();

  void setModel( QtbLayeredModel * m ) { _model = m; }
  QtbLayeredModel * model() const { return _model; }

  virtual QStringList functions() const;
  virtual bool isValidFunction(const QString& name) const;
  virtual QVariant functionValue(const QString& name, const QVector<QtbExpressionAction *>& args) const;
private:
  QtbLayeredModel * _model;
};

class QTBWAVE_EXPORT QtbLayeredModelExpressionN : public QtbExpressionStorage
{
public:
  QtbLayeredModelExpressionN( QtbLayeredModelExpressionContext * context ) { _context = context; }

  virtual bool isReadOnly() const { return true; }
  virtual QVariant value(int) const;
  virtual void setValue(int, const QVariant& ) {}
private:
  QtbLayeredModelExpressionContext * _context;
};

class QTBWAVE_EXPORT QtbLayeredModelExpressionH : public QtbExpressionStorage
{
public:
  QtbLayeredModelExpressionH( QtbLayeredModelExpressionContext * context ) { _context = context; }

  virtual bool isReadOnly() const { return false; }
  virtual QVariant value(int index) const;
  virtual void setValue(int, const QVariant& );
private:
  QtbLayeredModelExpressionContext * _context;
};

class QTBWAVE_EXPORT QtbLayeredModelExpressionVp : public QtbExpressionStorage
{
public:
  QtbLayeredModelExpressionVp( QtbLayeredModelExpressionContext * context ) { _context = context; }

  virtual bool isReadOnly() const { return false; }
  virtual QVariant value(int index) const;
  virtual void setValue(int, const QVariant& );
private:
  QtbLayeredModelExpressionContext * _context;
};

class QTBWAVE_EXPORT QtbLayeredModelExpressionVs : public QtbExpressionStorage
{
public:
  QtbLayeredModelExpressionVs( QtbLayeredModelExpressionContext * context ) { _context = context; }

  virtual bool isReadOnly() const { return false; }
  virtual QVariant value(int index) const;
  virtual void setValue(int, const QVariant& );
private:
  QtbLayeredModelExpressionContext * _context;
};

class QTBWAVE_EXPORT QtbLayeredModelExpressionRho : public QtbExpressionStorage
{
public:
  QtbLayeredModelExpressionRho( QtbLayeredModelExpressionContext * context ) { _context = context; }

  virtual bool isReadOnly() const { return false; }
  virtual QVariant value(int index) const;
  virtual void setValue(int, const QVariant& );
private:
  QtbLayeredModelExpressionContext * _context;
};

#endif // QTB_LAYEREDMODELEXPRESSIONCONTEXT_H
