/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-09-22
**
***************************************************************************/

#ifndef QTB_MODALCURVE_H
#define QTB_MODALCURVE_H

#include "qtbcurve.h"
#include "qtbmode.h"
#include "qtbfactorypoint.h"
#include "qtbtranslations.h"
#include "qtbwavedllexport.h"

class QtbModalFactory;

class QTBWAVE_EXPORT QtbModalCurve : public QtbCurve<QtbFactoryPoint>, public QtbXMLClass
{
  QTB_TRANSLATIONS("QtbModalCurve");
public:
  QtbModalCurve() {}
  inline QtbModalCurve( const QtbModalCurve& o );
  QtbModalCurve( int n ) : QtbCurve<QtbFactoryPoint>(n) {}
  inline QtbModalCurve( const QtbCurve<QtbStatPoint>& o );
  virtual ~QtbModalCurve();

  inline bool operator==( const QtbModalCurve& o) const;
  void operator=( const QtbCurve<QtbStatPoint>& o);

  virtual const QString& xml_tagName() const { return xmlModalCurveTag;}
  static const QString xmlModalCurveTag;

  void clear();
  void sort() { setFunction(); }

  void addMode( const QtbMode& m );
  void removeMode( int index );
  void linkX( const QVector<double>& x );
  const QList<QtbMode>& modes() const { return _modes; }
  QList<QtbMode>& modes() { return _modes; }
  bool hasMode( const QtbMode& m ) const;
  int indexOf( int factoryIndex ) const;

  void setName( QString n ) { _name = n; }
  QString name() const { return _name; }

  void addLog( QString s ) { _log+=s; }
  QString log() const { return _log; }

  double misfit( int& nValues, int& nData, const QtbModalFactory& f, QtbStatValue::MisfitType type, double minMisfit ) const;
protected:
  void xml_writeProperties(XML_WRITEPROPERTIES_ARGS) const;
  void xml_writeChildren(XML_WRITECHILDREN_ARGS) const;
  QtbXMLMember xml_member(XML_MEMBER_ARGS);
  bool xml_setProperty(XML_SETPROPERTY_ARGS);
private:
  QString _name;
  QList<QtbMode> _modes;
  QString _log;
};

inline QtbModalCurve::QtbModalCurve( const QtbModalCurve& o ) : QtbCurve<QtbFactoryPoint>(o), QtbXMLClass()
{
  QTB_TRACE;
  _name = o._name;
  _modes = o._modes;
  _log = o._log;
}

inline QtbModalCurve::QtbModalCurve( const QtbCurve<QtbStatPoint>& o ) : QtbCurve<QtbFactoryPoint>(), QtbXMLClass()
{
  QTB_TRACE;
  *this = o;
}

inline bool QtbModalCurve::operator==( const QtbModalCurve& o) const
{
  return _name == o._name &&
         _modes == o._modes &&
         _log == o._log &&
         QtbCurve<QtbFactoryPoint>::operator==(o);
}

#endif // QTB_MODALCURVE_H
