/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-09-26
**
***************************************************************************/

#ifndef QTB_MODALFACTORY_H
#define QTB_MODALFACTORY_H

#include <QVector>

#include "qtbwavedllexport.h"
#include "qtbtranslations.h"

class QtbModalStorage;
class QtbModalCurve;
class QtbValue;
class QtbMode;

class QTBWAVE_EXPORT QtbModalFactory
{
  QTB_TRANSLATIONS("QtbModalFactory");
public:
  QtbModalFactory() {}
  virtual ~QtbModalFactory() {}

  void setX ( QtbModalFactory& f );
  void setX ( const QList<QtbModalCurve>& curves );
  void linkX ( QList<QtbModalCurve>& curves ) const;
  void setModes ( const QList<QtbModalCurve>& curves );

  void setX ( const QtbModalCurve& c );
  void linkX ( QtbModalCurve& c ) const;
  void setModes ( const QtbModalCurve& c );
  void setAngularFrequency();

  virtual void setMode ( const QtbMode& m ) = 0;
  virtual const QtbValue * mode( const QtbMode& m ) const = 0;
  virtual int storageCount() const = 0;
  virtual void writeReportHeader( QDataStream& ) const {}
  virtual QtbModalStorage * storage( int storageIndex ) const = 0;

  const QVector<double> * x() const { return &_x; }
protected:
  virtual QtbModalStorage * newStorage( int nModes ) = 0;
  template <class StorageType> void upgrade ( StorageType *& s, int requestedMode );

  QVector<double> _x;
};

template <class StorageType>
void QtbModalFactory::upgrade ( StorageType *& s, int requestedMode )
{
  QTB_TRACE;
  if ( !s || requestedMode >= s->modeCount() ) {
    StorageType * tmp_s = static_cast<StorageType *>(newStorage( requestedMode + 1 ));
    delete s;
    s = tmp_s;
  }
}

#endif // QTB_MODALFACTORY_H
