/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2004-10-22
**
***************************************************************************/

#ifndef QTB_MODALREFINE_H
#define QTB_MODALREFINE_H

#include <QList>

#include "qtbvalue.h"
#include "qtbwavedllexport.h"
#include "qtbglobal.h"
#include "qtbtrace.h"

class QTBWAVE_EXPORT QtbModalRefine
{
public:
  QtbModalRefine() { _nModes = 0; _values = 0; }
  inline QtbModalRefine( int nModes, double x );
  inline QtbModalRefine( const QtbModalRefine& o );
  ~QtbModalRefine() { delete [] _values; }

  inline void operator=( const QtbModalRefine& o );
  bool operator<( const QtbModalRefine& o ) const { return _x < o._x; }

  double x() const { return _x;}

  const QtbValue& value( int iMode ) const { QTB_ASSERT(iMode<_nModes); return _values[ iMode ];}
  void setValue( int iMode, double val ) { QTB_ASSERT(iMode<_nModes); _values[ iMode ].setValue( val ); }
  void setValid( int iMode, bool v ) { QTB_ASSERT(iMode<_nModes); _values[ iMode ].setValid( v ); }
  inline void setValid( bool v );
private:
  double _x;
  int _nModes;
  QtbValue * _values;
};

typedef QList<QtbModalRefine> QtbRefineList;
typedef QList<QtbModalRefine>::ConstIterator QtbRefineIterator;

inline QtbModalRefine::QtbModalRefine( int nModes, double x )
{
  QTB_TRACE;
  _nModes = nModes;
  _values = new QtbValue [ _nModes ];
  _x = x;
}

inline QtbModalRefine::QtbModalRefine( const QtbModalRefine& o )
{
  QTB_TRACE;
  _x = o._x;
  _nModes = o._nModes;
  _values = new QtbValue [ _nModes ];
  for (int i=0;i<_nModes;i++) {
    _values[i]=o._values[i];
  }
}

inline void QtbModalRefine::operator=( const QtbModalRefine& o )
{
  QTB_TRACE;
  _x = o._x;
  _nModes = o._nModes;
  delete [] _values;
  _values = new QtbValue [ _nModes ];
  for (int i=0;i<_nModes;i++) {
    _values[i]=o._values[i];
  }
}

inline void QtbModalRefine::setValid( bool v )
{
  QTB_TRACE;
  for (int i=0;i<_nModes;i++) {
    _values[i].setValid(v);
  }
}

#endif // QTB_MODALREFINE_H
