/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-09-26
**
***************************************************************************/

#include "qtbmode.h"

const QString QtbMode::xmlModeTag = "Mode";

/*!
  \class QtbMode qtbmode.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Return a user-friendly string for slowness
*/
QString QtbMode::slownessString() const
{
  QTB_TRACE;
  switch ( _slowness ) {
  case Group:
    return tr("Group");
    break;
  default:
    return tr("Phase");
    break;
  }
  return QString::null;
}

/*!
  Return a user-friendly string for polarisation
*/
QString QtbMode::polarisationString() const
{
  QTB_TRACE;
  switch ( _polarisation ) {
  case Vertical:
    return tr("Vertical");
  case Radial:
    return tr("Radial");
  case Rayleigh:
    return tr("Rayleigh");
  case Transverse:
    return tr("Transverse");
  case Love:
    return tr("Love");
  }
  return QString::null;
}

/*!
  Set slowness from a user-friendly string
*/
void QtbMode::setSlowness( QString s )
{
  QTB_TRACE;
  if (s==tr("Group")) {
    _slowness = Group;
  } else {
    _slowness = Phase;
  }
}

/*!
  Set polarisatio from a user-friendly string
*/
void QtbMode::setPolarisation( QString p )
{
  QTB_TRACE;
  if (p==tr("Vertical")) {
    _polarisation = Vertical;
  } else if (p==tr("Radial")) {
    _polarisation = Radial;
  } else if (p==tr("Transverse")) {
    _polarisation = Transverse;
  } else if (p==tr("Love")) {
    _polarisation = Love;
  } else {
    _polarisation = Rayleigh;
  }
}

/*!
  Return a user-friendly string describing this mode index
*/
QString QtbMode::indexString() const
{
  QTB_TRACE;
  switch (_index) {
  case 0:
    return tr("fundamental mode");
  case 1:
    return tr("1st higher mode");
  case 2:
    return tr("2nd higher mode");
  case 3:
    return tr("3rd higher mode");
  default:
    return tr("higher mode %1").arg(_index);
  }
}

/*!
  Return a user-friendly string completely describing this mode
*/
QString QtbMode::toString() const
{
  QTB_TRACE;
  QString str;
  switch (_polarisation) {
  case Vertical:
  case Radial:
  case Transverse:
    str+=QString("Ring %1").arg(_ringIndex);
    break;
  case Love:
  case Rayleigh:
    switch (_slowness) {
    case Group:
      str+=tr("Group");
      break;
    case Phase:
      str+=tr("Phase");
      break;
    }
  }
  str+=" ";
  switch (_polarisation) {
  case Vertical:
    str+=tr("Vertical");
    break;
  case Radial:
    str+=tr("Radial");
    break;
  case Rayleigh:
    str+=tr("Rayleigh");
    break;
  case Transverse:
    str+=tr("Transverse");
    break;
  case Love:
    str+=tr("Love");
    break;
  }
  str+=" ";
  switch (_index) {
  case 0:
    str+=tr("fundamental");
    break;
  case 1:
    str+=tr("1st higher");
    break;
  case 2:
    str+=tr("2nd higher");
    break;
  case 3:
    str+=tr("3rd higher");
    break;
  default:
    str+=tr("higher %1").arg(_index);
    break;
  }
  return str;
}

void QtbMode::xml_writeProperties(XML_WRITEPROPERTIES_ARGS) const
{
  QTB_TRACE;
  Q_UNUSED(context);
  switch (_slowness) {
  case Group:
    writeProperty(s,"slowness", "Group");
    break;
  case Phase:
    writeProperty(s,"slowness", "Phase");
    break;
  }
  switch (_polarisation) {
  case Vertical:
    writeProperty(s,"polarisation","Vertical");
    break;
  case Radial:
    writeProperty(s,"polarisation","Radial");
    break;
  case Rayleigh:
    writeProperty(s,"polarisation","Rayleigh");
    break;
  case Transverse:
    writeProperty(s,"polarisation","Transverse");
    break;
  case Love:
    writeProperty(s,"polarisation","Love");
    break;
  }
  writeProperty(s,"ringIndex",_ringIndex);
  writeProperty(s,"index",_index);
}

QtbXMLMember QtbMode::xml_member(XML_MEMBER_ARGS)
{
  QTB_TRACE;
  Q_UNUSED(attributes)
  Q_UNUSED(context);
  if (tag=="slowness") return QtbXMLMember(0);
  else if (tag=="polarisation") return QtbXMLMember(1);
  else if (tag=="ringIndex") return QtbXMLMember(2);
  else if (tag=="index") return QtbXMLMember(3);
  else return QtbXMLMember(QtbXMLMember::Unknown);
}

bool QtbMode::xml_setProperty(XML_SETPROPERTY_ARGS)
{
  QTB_TRACE;
  Q_UNUSED(attributes)
  Q_UNUSED(context);
  switch (memberID) {
  case 0: if (content == "Group") _slowness = Group; else _slowness = Phase; return true;
  case 1:
    if (content == "Rayleigh") _polarisation = Rayleigh;
    else if (content == "Love") _polarisation = Love;
    else if (content == "Vertical") _polarisation = Vertical;
    else if (content == "Radial") _polarisation = Radial;
    else if (content == "Transverse") _polarisation = Transverse;
    else return false;
    return true;
  case 2: _ringIndex = content.toInt(); return true;
  case 3: _index = content.toInt(); return true;
  default: return false;
  }
}
