/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2006-09-26
**
***************************************************************************/

#ifndef QTB_MODE_H
#define QTB_MODE_H

#include "qtbwavedllexport.h"
#include "qtbxmlclass.h"
#include "qtbtranslations.h"

class QTBWAVE_EXPORT QtbMode : public QtbXMLClass
{
  QTB_TRANSLATIONS("QtbMode");
public:
  enum Polarisation {Vertical=1, Radial=2, Rayleigh=3, Transverse=4, Love=5};
  enum Slowness {Phase=0, Group=1};

  inline QtbMode();
  inline QtbMode( Slowness s, Polarisation p, int index);
  inline QtbMode( Polarisation p, int ringIndex, int index);
  inline QtbMode(const QtbMode& o);

  virtual const QString& xml_tagName() const { return xmlModeTag;}
  static const QString xmlModeTag;

  bool operator==(const QtbMode& o) const;

  void setPolarisation( Polarisation p ) { _polarisation = p; }
  void setPolarisation( QString p );
  Polarisation polarisation() const { return _polarisation; }
  QString polarisationString() const;

  void setSlowness( Slowness s ) { _slowness = s; }
  void setSlowness( QString s );
  Slowness slowness() const { return _slowness; }
  QString slownessString() const;

  void setRingIndex( int i ) { _ringIndex = i; }
  int ringIndex() const { return _ringIndex; }

  void setIndex( int i ) { _index = i; }
  int index() const { return _index; }
  QString indexString() const;
  QString toString() const;
protected:
  void xml_writeProperties(XML_WRITEPROPERTIES_ARGS) const;
  QtbXMLMember xml_member(XML_MEMBER_ARGS);
  bool xml_setProperty(XML_SETPROPERTY_ARGS);
private:
  Slowness _slowness;
  Polarisation _polarisation;
  int _ringIndex;
  int _index;
};

inline QtbMode::QtbMode()
{
  QTB_TRACE;
  _polarisation = Rayleigh;
  _slowness = Phase;
  _ringIndex = 0;
  _index = 0;
}

inline QtbMode::QtbMode( Polarisation p, int ringIndex, int index)
{
  QTB_TRACE;
  _polarisation = p;
  _slowness = Phase;
  _ringIndex = ringIndex;
  _index = index;
}

inline QtbMode::QtbMode( Slowness s, Polarisation p, int index)
{
  QTB_TRACE;
  _polarisation = p;
  _slowness = s;
  _ringIndex = 0;
  _index = index;
}

inline QtbMode::QtbMode(const QtbMode& o) : QtbXMLClass()
{
  QTB_TRACE;
  _polarisation = o._polarisation;
  _slowness = o._slowness;
  _ringIndex = o._ringIndex;
  _index = o._index;
}

inline bool QtbMode::operator==(const QtbMode& o) const
{
  QTB_TRACE;
  return _polarisation == o._polarisation &&
         _slowness == o._slowness &&
         _ringIndex == o._ringIndex &&
         _index == o._index;
}

#endif // QTB_MODE_H
