/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2005-12-21
**
***************************************************************************/

#ifndef QTB_PROFILE_H
#define QTB_PROFILE_H

#include <QVector>
#include <QDataStream>
#include <QTextStream>

#include "qtbwavedllexport.h"
#include "qtbcurve.h"
#include "qtbtrace.h"

class QtbPoint2D;

class QTBWAVE_EXPORT QtbProfile
{
public:
  inline void operator=( const QtbProfile& p );
  inline void setDepth( int i, double depth);
  inline void setValue( int i, double value);
  inline void resize( int n );
  inline void collectDepths(QVector<double>& depths) const;
  inline void writeReport(QDataStream& s) const;
  inline void readReport(QDataStream& s);
  void resample( const QtbProfile& p, const QVector<double>& baseD );
  void resample( const QVector<double>& baseD );
  inline void resample( const QVector<double>& baseD1, const QVector<double>& baseD2 );
  void setSamples( const QVector<double>& baseD );
  const QVector<double>& depths() const {return _depths;}
  const QVector<double>& values() const {return _values;}
  static void report2plot(QDataStream& s, QtbPoint2D * points);
  double interpoleAt(double depth);
  double uniformAt(double depth);
  void average();
  void inverse();
  int count() const { return _depths.count(); }
  void toStream(QTextStream& s, bool plotMode) const;
  QtbCurve<QtbPoint2D> curve() const;
  void print() const;
private:
  QVector<double> _depths;
  QVector<double> _values;
};

inline void QtbProfile::setDepth( int i, double depth)
{
  QTB_TRACE;
  _depths[i]=depth;
}

inline void QtbProfile::setValue( int i, double value)
{
  QTB_TRACE;
  _values[i]=value;
}

inline void QtbProfile::collectDepths(QVector<double>& depths) const
{
  QTB_TRACE;
  depths << _depths;
}

inline void QtbProfile::resize( int n )
{
  QTB_TRACE;
  _depths.resize(n);
  _values.resize(n);
}

inline void QtbProfile::writeReport(QDataStream& s) const
{
  QTB_TRACE;
  int n=_depths.count();
  s << n;
  for (int i=0;i<n;i++) {
    s << _depths.at(i);
    s << _values.at(i);
  }
}

inline void QtbProfile::readReport(QDataStream& s)
{
  QTB_TRACE;
  int n;
  s >> n;
  if(n<0 || n>10000) {
    // Probably a corrupted file
    return;
  }
  resize(n);
  double d,v;
  for (int i=0;i<n;i++) {
    s >> d >> v;
    setDepth( i, d );
    setValue( i, v );
  }
}

inline void QtbProfile::operator=( const QtbProfile& p )
{
  QTB_TRACE;
  _depths = p._depths;
  _values = p._values;
}

inline void QtbProfile::resample( const QVector<double>& baseD1, const QVector<double>& baseD2 )
{
  QTB_TRACE;
  resample( baseD1 );
  resample( baseD2 );
}

#endif // QTB_PROFILE_H
