/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-11-02
**
***************************************************************************/

#include "qtbrefractioncurve.h"
#include "qtbrefractionfactory.h"
#include "qtbvalue.h"
#include "qtbmemorychecker.h"

const QString QtbRefractionCurve::xmlRefractionCurveTag = "RefractionCurve";

/*!
  \class QtbRefractionCurve qtbrefractioncurve.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

void QtbRefractionCurve::clear()
{
  QTB_TRACE;
  QtbCurve<QtbFactoryPoint>::clear();
  _log = "";
  _name = "";
  _source = 0.0;
}

/*!
  Set the index in vector \a xSrc of source. Source x must be in \a xSrc.
*/
void QtbRefractionCurve::linkSource( const QVector<double>& xSrc )
{
  QTB_TRACE;
  _sourceIndex = xSrc.indexOf( _source );
  QTB_ASSERT( _sourceIndex>-1 );
}

/*!
  Set the index in vector \a xRec of each sample of the curve. Each sample of the curve must have its
  corresponding value in vector \a xRec.
*/
void QtbRefractionCurve::linkReceivers( const QVector<double>& xRec )
{
  QTB_TRACE;
  int n = count();
  int nx = xRec.count();
  int ix = 0;
  for (int i = 0; i< n; i++) {
    while (ix<nx && xRec.at(ix) < at(i).x()) ix++;
    QTB_ASSERT( at(i).x() == xRec.at(ix) );
    (*this)[i].setIndex(ix);
  }
}

/*!
  Calculate the squared misfit of \a values with respect to this curve.

  \a nValues is incremented by the number of valid values where the comparison was possible.
  \a nData is incremented by the number of valid data where the comparison is possible. In
  all cases, \a nData >= \a nValues and for the best matching case, \a nValues == \a nData.

  \a minMisfit2 is the square of the minimum misfit value that can be reached for each sample individualy.
*/
double QtbRefractionCurve::misfit( int& nValues, int& nData, const QtbRefractionFactory& f, QtbStatValue::MisfitType type, double minMisfit ) const
{
  QTB_TRACE;
  double res=0;
  int nx = count();
  const QtbValue * values = f.source( _sourceIndex );
  for (int i=0;i<nx;i++) {
    const QtbFactoryPoint& p = at(i);
    double pointMisfit = p.misfit( nValues, nData, values[ p.index() ], type, minMisfit );
    if (pointMisfit>0.0) {
      res+=pointMisfit;
    }
  }
  return res;
}

void QtbRefractionCurve::xml_writeProperties(XML_WRITEPROPERTIES_ARGS) const
{
  QTB_TRACE;
  Q_UNUSED(context);
  writeProperty(s,"name",_name);
  writeProperty(s,"log",_log);
  writeProperty(s,"source",_source);
}

void QtbRefractionCurve::xml_writeChildren(XML_WRITECHILDREN_ARGS) const
{
  QTB_TRACE;
  const_iterator it;
  for ( it=begin(); it!=end(); ++it ) {
    it->xml_save(s, context);
  }
}

QtbXMLMember QtbRefractionCurve::xml_member(XML_MEMBER_ARGS)
{
  QTB_TRACE;
  Q_UNUSED(attributes)
  Q_UNUSED(context);
  if (tag=="StatPoint") {
    append( QtbFactoryPoint() );
    return QtbXMLMember( &last() );
  } else if (tag=="name") return QtbXMLMember(0);
  else if (tag=="log") return QtbXMLMember(1);
  else if (tag=="source") return QtbXMLMember(2);
  else return QtbXMLMember(QtbXMLMember::Unknown);
}

bool QtbRefractionCurve::xml_setProperty(XML_SETPROPERTY_ARGS)
{
  QTB_TRACE;
  Q_UNUSED(attributes)
  Q_UNUSED(context);
  switch (memberID) {
  case 0: _name=content.toString(); return true;
  case 1: _log=content.toString(); return true;
  case 2: _source=content.toDouble(); return true;
  default: return false;
  }
}
