/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-11-02
**
***************************************************************************/

#ifndef QTB_REFRACTIONCURVE_H
#define QTB_REFRACTIONCURVE_H

#include "qtbcurve.h"
#include "qtbxmlclass.h"
#include "qtbwavedllexport.h"
#include "qtbfactorypoint.h"

class QtbRefractionFactory;
class QTextStream;

class QTBWAVE_EXPORT QtbRefractionCurve : public QtbCurve<QtbFactoryPoint>, public QtbXMLClass
{
public:
  QtbRefractionCurve() { _source = 0.0; }
  inline QtbRefractionCurve( const QtbRefractionCurve& o );
  QtbRefractionCurve( int n ) : QtbCurve<QtbFactoryPoint>(n) { _source = 0.0; }
  virtual ~QtbRefractionCurve() {}

  inline bool operator==( const QtbRefractionCurve& o) const;

  virtual const QString& xml_tagName() const { return xmlRefractionCurveTag;}
  static const QString xmlRefractionCurveTag;

  void clear();
  void sort() { setFunction(); }

  void setName( QString n ) { _name = n; }
  QString name() const { return _name; }

  void addLog( QString s ) { _log+=s; }
  QString log() const { return _log; }

  void setSource( double s ) { _source = s; }
  double source() const { return _source; }

  void linkSource( const QVector<double>& xSrc );
  void linkReceivers( const QVector<double>& xRec );

  double misfit( int& nValues, int& nData, const QtbRefractionFactory& f, QtbStatValue::MisfitType type, double minMisfit ) const;
protected:
  void xml_writeProperties(XML_WRITEPROPERTIES_ARGS) const;
  void xml_writeChildren(XML_WRITECHILDREN_ARGS) const;
  QtbXMLMember xml_member(XML_MEMBER_ARGS);
  bool xml_setProperty(XML_SETPROPERTY_ARGS);
private:
  QString _name;
  QString _log;
  double _source;
  int _sourceIndex;
};

inline QtbRefractionCurve::QtbRefractionCurve( const QtbRefractionCurve& o ) : QtbCurve<QtbFactoryPoint>(o), QtbXMLClass()
{
  QTB_TRACE;
  _name = o._name;
  _log = o._log;
  _source = o._source;
}

inline bool QtbRefractionCurve::operator==( const QtbRefractionCurve& o) const
{
  return _name == o._name &&
         _log == o._log &&
         _source == o._source &&
         QtbCurve<QtbFactoryPoint>::operator==(o);
}

#endif // QTB_REFRACTIONCURVE_H
