/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-11-20
**
***************************************************************************/

#include <QSet>

#include "qtbrefractionfactory.h"
#include "qtbrefractioncurve.h"
#include "qtbtiltlayeredmodel.h"
#include "qtbtiltnode.h"
#include "qtbcoreapplication.h"
#include "qtbmemorychecker.h"

/*!
  \class QtbRefractionFactory qtbrefractionfactory.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
QtbRefractionFactory::QtbRefractionFactory()
{
  QTB_TRACE;
  _travelTimes = 0;
}

/*!
  Description of destructor still missing
*/
QtbRefractionFactory::~QtbRefractionFactory()
{
  QTB_TRACE;
  delete [] _travelTimes;
}

void QtbRefractionFactory::setX( const QList<QtbRefractionCurve>& curves )
{
  QTB_TRACE;
  //  Gather all the sources and receivers
  _sources.clear();
  _receivers.clear();
  for (QList<QtbRefractionCurve>::const_iterator itCurve = curves.begin(); itCurve!=curves.end(); itCurve++) {
    const QtbRefractionCurve& curve = *itCurve;
    _sources.append( curve.source() );
    for(QtbRefractionCurve::const_iterator itRec = curve.begin(); itRec != curve.end(); itRec++) {
      _receivers.append( itRec->x() );
    }
  }
  Qtb::uniq( _sources );
  Qtb::uniq( _receivers );
  // Allocates values for storing arrival times
  _travelTimes = new QtbValue[ _sources.count()*_receivers.count() ];
}

void QtbRefractionFactory::linkX( QList<QtbRefractionCurve>& curves ) const
{
  QTB_TRACE;
  // Set references to values
  for (QList<QtbRefractionCurve>::iterator itCurve = curves.begin(); itCurve!=curves.end(); itCurve++) {
    QtbRefractionCurve& curve = *itCurve;
    curve.linkSource( _sources );
    curve.linkReceivers( _receivers );
  }
}

/*!

*/
bool QtbRefractionFactory::calculate( QtbTiltLayeredModel * model )
{
  QTB_TRACE;
  model->setXLeft( _receivers.first() );
  model->setXRight( _receivers.last() );
  model->begin();
  QtbTiltNode srcNode, recNode;
  int deepestLayer;
  QtbValue * values = _travelTimes;
  for (QVector<double>::const_iterator itSrc = _sources.begin(); itSrc!=_sources.end(); itSrc++) {
    srcNode.init(*itSrc, *model);
    for(QVector<double>::const_iterator itRec = _receivers.begin(); itRec!=_receivers.end(); itRec++) {
      recNode.init(*itRec,*model);
      (values++)->setValue( model->travelTime( srcNode, recNode, deepestLayer) );
    }
  }
  model->end();
  return true;
}

void QtbRefractionFactory::writeReport( QDataStream& s ) const
{
  QTB_TRACE;
  int nSrc = sourceCount();
  int nRec = receiverCount();
  s << nSrc << nRec;
  for ( int i = 0; i < nSrc; i++ ) {
    s << _sources[i];
  }
  for ( int i = 0; i < nRec; i++ ) {
    s << _receivers[i];
  }
  int n = nSrc * nRec;
  for ( int i = 0; i < n; i++ ) {
    s << _travelTimes[i].value();
  }
}

void QtbRefractionFactory::readReport( QDataStream& s )
{
  QTB_TRACE;
  int nSrc, nRec, n;
  s >> nSrc >> nRec;
  n = nSrc * nRec;
  _sources.resize( nSrc );
  for( int i = 0; i < nSrc; i++ ) {
    s >> _sources[i];
  }
  _receivers.resize( nRec );
  for( int i = 0; i < nRec; i++ ) {
    s >> _receivers[i];
  }
  double tt;
  delete _travelTimes;
  _travelTimes = new QtbValue[n];
  for ( int i = 0; i < n; i++ ) {
    s >> tt;
    _travelTimes[i].setValue(tt);
  }
}

void QtbRefractionFactory::toStream( int sourceIndex, QTextStream& sOut ) const
{
  QTB_TRACE;
  const QtbValue * tt = source( sourceIndex );
  QtbPoint2D p;
  int n = _receivers.count();
  for ( int i = 0; i < n; i++ ) {
    p.setX( _receivers[i] );
    p.setY( tt[i].value() );
    sOut << p << "\n";
  }
  sOut << flush;
}

