/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-11-20
**
***************************************************************************/

#ifndef QTB_REFRACTIONFACTORY_H
#define QTB_REFRACTIONFACTORY_H

#include "qtbwavedllexport.h"
#include "qtbvalue.h"
#include "qtbtranslations.h"

class QTextStream;
class QtbTiltLayeredModel;
class QtbRefractionCurve;

class QTBWAVE_EXPORT QtbRefractionFactory
{
  QTB_TRANSLATIONS("QtbRefractionFactory");
public:
  QtbRefractionFactory();
  ~QtbRefractionFactory();

  int sourceCount() const { return _sources.count(); }
  int receiverCount() const { return _receivers.count(); }
  const QVector<double>& sources() const { return _sources; }
  const QVector<double>& receivers() const { return _receivers; }
  const QtbValue * source( int index ) const { return _travelTimes + index * _receivers.count(); }

  void setX( const QList<QtbRefractionCurve>& curves );
  void linkX( QList<QtbRefractionCurve>& curves ) const;
  double xLeft() const { return _receivers.first(); }
  double xRight() const { return _receivers.last(); }

  bool calculate( QtbTiltLayeredModel * model );

  void writeReport( QDataStream& s ) const;
  void readReport( QDataStream& s );
  void toStream( int sourceIndex, QTextStream& sOut ) const;
private:
  QVector<double> _sources;
  QVector<double> _receivers;
  QtbValue * _travelTimes;
};

inline uint qHash( const double& key )
{
  return qHash( static_cast<const quint64&>( key ) );
}

#endif // QTB_REFRACTIONFACTORY_H
