/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2007-11-12
**
***************************************************************************/

#ifndef QTB_THEORETICALFK_H
#define QTB_THEORETICALFK_H

#include <math.h>

#include <QVector>

#include "qtbabstractfunction2.h"
#include "qtbpoint.h"
#include "qtbwavedllexport.h"

class QTBWAVE_EXPORT QtbTheoreticalFK : public QtbAbstractFunction2
{
public:
  QtbTheoreticalFK(QVector<QtbPoint> * stations);

  inline virtual double value(double kx, double ky) const;
protected:
  QVector<QtbPoint> * _stations;
private:
  double _fac;
};

inline double QtbTheoreticalFK::value(double kx, double ky) const
{
  QTB_TRACE;
  double csum=0, ssum=0, val;
  QVector<QtbPoint>& stations=*_stations;
  int nStations=stations.count();
  for (int pi=0;pi<nStations;pi++) {
    val=kx*stations[pi].x()+ky*stations[pi].y();
    csum+=cos(val);
    ssum+=sin(val);
  }
  return (csum*csum+ssum*ssum)*_fac;
}

#endif // QTB_THEORETICALFK_H
