/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-08-06
**
***************************************************************************/

#include "qtbtiltnode.h"
#include "qtbtiltpath.h"

/*!
  \class QtbTiltNode qtbtiltnode.h
  \brief Brief description of class still missing

  Store the left and right ray paths for a node (either a source or a receiver)
*/

/*!
  Description of constructor still missing
*/
QtbTiltNode::QtbTiltNode()
{
  QTB_TRACE;
  _leftPaths=0;
  _rightPaths=0;
}

/*!
  Description of destructor still missing
*/
QtbTiltNode::~QtbTiltNode()
{
  QTB_TRACE;
  delete [] _leftPaths;
  delete [] _rightPaths;
}

double QtbTiltNode::x() const
{
  QTB_TRACE;
  return _leftPaths[0].origin();
}

void QtbTiltNode::init(double x0, const QtbTiltLayeredModel& model)
{
  QTB_TRACE;
  x0-=model.xLeft();
  initPaths( QtbTiltLayeredModel::LeftToRight, _leftPaths, x0, model );
  initPaths( QtbTiltLayeredModel::RightToLeft, _rightPaths, x0, model );
}

void QtbTiltNode::initPaths( QtbTiltLayeredModel::Direction dir, QtbTiltPath *& paths,
                             double x0, const QtbTiltLayeredModel& model)
{
  QTB_TRACE;
  double x, dt;
  paths=new QtbTiltPath [model.layerCount()-1];
  for (int iPath=1;iPath<model.layerCount();iPath++) {
    paths[iPath-1].setDeepestLayer(iPath);
    paths[iPath-1].setOrigin(x0);
    //printf("left path[%i] at %i x %lg t %lg\n",iPath,0,x0,0.0);
    if (model.isCriticalLayer(dir, iPath)) {
      x=x0;
      for (int iLayer=1;iLayer<=iPath;iLayer++) {
        dt=model.propagate(dir, iLayer, iPath, x);
        //printf("left path[%i] at %i x %lg t %lg\n",iPath,iLayer,x,dt);
        paths[iPath-1].setIntercept(iLayer, x, dt);
      }
    }
    else paths[iPath-1].setIntercept(0, x0, 1e99);
  }
}
