/***************************************************************************
**
**  Copyright (C) 2002-2009 Marc Wathelet. All rights reserved.
**
**  This file is part of qtbwave.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  See http://www.geopsy.org for more information.
**
**  Created : 2008-08-06
**
***************************************************************************/

#ifndef QTB_TILTPATH_H
#define QTB_TILTPATH_H

#include <QVector>

#include "qtbtrace.h"

class QtbTiltPath
{
public:
  double time() const { return _dt; }
  double abscissa( int iLayer ) const { return _x[iLayer]; }
protected:
  friend class QtbTiltNode;
  QtbTiltPath();

  inline void setOrigin( double x0 );
  double origin() const { return _x[0]; }

  inline void setDeepestLayer( int toLayer );
  inline void setIntercept(int iLayer, double x, double dt);
private:
  QVector<double> _x;
  double _dt;
};

inline void QtbTiltPath::setOrigin( double x0 )
{
  QTB_TRACE;
  _x[0]=x0;
}

inline void QtbTiltPath::setDeepestLayer( int toLayer )
{
  QTB_TRACE;
  _x.resize(toLayer+1);
}

inline void QtbTiltPath::setIntercept( int iLayer, double x, double dt )
{
  QTB_TRACE;
  _x[iLayer]=x;
  _dt+=dt;
}

#endif // QTB_TILTPATH_H
